#!bash
# add path (important: if path is missing, no command will be found) and remove doubles
export PATH=/usr/local/bin:/usr/bin:/bin:/lib:`echo "$PATH" | /bin/sed "s=/usr/local/bin:/usr/bin:/bin:/lib:==g"`

export TEAM_DIR=`echo $0 |sed 's@\(.*\)/Make.*@\1@' |sed 's@/cygdrive/\(.\)\(.*\)@\1:\2@'`
#TEAM_DIR is for example c:/GT2004 or c:/DT2005

echo "! copyfiles.bash $*"
echo "!"

# defaults
HELP=0
FORCE=0
UPD=""
PAUSE=0
NOPLAYER=0
ERS7=0
RM=0
EJECT=0
WLAN=WLAN
UNSAFE=0

# parse options
CONT=1
while test $CONT = 1; do
  case $1x in
    --helpx)
      shift
      HELP=1
      ;;
    --forcex)
      shift
      FORCE=1
      ;;
    --updatex)
      shift
      UPD="-u"
      ;;
    --pausex)
      shift
      PAUSE=1
      ;;
    --noplayerx)
      shift
      NOPLAYER=1
      ;;
    --rmx)
      shift
      RM=1
      ;;
    --wconsolex)
      shift
      WLAN=WCONSOLE
      ;;
    --unsafex)
      shift
      UNSAFE=1
      ;;
    --ejectx)
      shift
      EJECT=1
      ;;
    -dx)
      shift
      export TEAM_DIR=$1
      shift
      ;;
    -mx)
      shift
      export MSD=$1
      shift
      ;;
    ERS210x)
      shift
      ERS7=0
      ;;
    ERS7x)
      shift
      ERS7=1
      ;;
    x)
      CONT=0
      ;;
    *)
      HELP=1
      CONT=0
      ;;
  esac
done

if test $HELP = 1; then
  echo "usage:"
  echo "  copyfiles [--help | [-d <TEAM_DIR>] [-m <MS>] [--force | --rm | --noplayer]"
  echo "            [--pause] [--eject] [ERS210 | ERS7 | --update] ]"
  echo "examples:"
  echo "  copyfiles.bash             update unchangeable files (wav), copy changeable ones (bin, conf, system), use system from GT.bash"
  echo "  copyfiles.bash ERS210      the same, use system for ERS210"
  echo "  copyfiles.bash ERS7        the same, use system for ERS7"
  echo "  copyfiles.bash --update    only copies newer files, do not copy system"
  echo "  copyfiles.bash --help      shows this help"
  echo "  copyfiles.bash -m H:       use H: as memorystick drive"
  echo "  copyfiles.bash -d T:/GT04  use T:/GT04 as TEAM_DIR"
  echo "  copyfiles.bash --force     formats the stick and copies everything, ensure right drive!"
  echo "  copyfiles.bash --noplayer  never touch player stuff"
  echo "  copyfiles.bash --pause     wait for key press after copying"
  echo "  copyfiles.bash --rm        rm -rf $MSD:/open-r before copying"
  echo "  copyfiles.bash --eject     call ct's deveject at the end"
else
  if test x$MSD = x; then
    case $HOSTNAME in
    #
    # ADD YOUR MEMORYSTICK DRIVELETTER IN THE FOLLOWING LINES
    #
    # Put in your computers hostname and export MSD with your memorystick-driveletter
    #
      SAM|PIGFINN)
        export MSD=d:
        ;;
      ANNE|calvin|clara|lap59|lap62|knut|ProjectPC)
        export MSD=e:
        ;;
      reinhold|komet|baudrillard|saviour|SAVIOUR|ds43|ds42)
        export MSD=f:
        ;;
      max3|Vernon|vaio|sparky|nb-avmovie)
        export MSD=g:
        ;;
      ute|ltroefer|louis|soda|gilbert)
        export MSD=h:
        ;;
      TINA)
        export MSD=o:
        ;;
      pluto|max|BLUTWURST|blutwurst)
        export MSD=i:
        ;;
      harry|jayjay)
        export MSD=f:
        ;;
      KILRATH|deepmo)
        export MSD=k:
        ;;
      calculon|farnsworth)
        export MSD=c:/tmp/MemoryStick
        ;;
      sokmob)
        export MSD=t:/memstick
        ;;
      *)

      # try to find out the memory-stick device (search for openr or memstick.ind)
      
      echo No Entry for $HOSTNAME - trying to find memory-stick
      for DRIVE in /cygdrive/* 
      do
            if test -d "$DRIVE/open-r" -o -e "$DRIVE/memstick.ind"
            then 
               MSD=$(basename $DRIVE):
               echo "Found memory-stick on $MSD."
               if test $RM = 1
               then
                  echo "--rm selected. Press Return to continue or Ctrl-C to quit"
                  read
               fi
            fi
      done
      if test -z $MSD 
      then 
          export MSD=e:
      fi
        ;;
    esac
  fi

  MSOPENR=$TEAM_DIR/Build/MS/open-r

  if test -d $MSD; then
    if test $FORCE = 1; then
      echo "! format $MSD and copy $MSOPENR directory to $MSD ..."
      echo "" | cmd /C format $MSD /V: /X /Q >/dev/null
    else
      if test $RM = 1; then
        echo "! rm -rf $MSD/open-r and copy $MSOPENR there ..."
        rm -rf $MSD/open-r
      else
        echo "! copy $MSOPENR directory to $MSD ..."
      fi
    fi

    if test x$UPD = x; then
      DEBUG_MODE=`grep DebugMode $MSOPENR/APP/CONF/CONFIG.CFG | sed "s%DebugMode %%"`
      if test $ERS7 = 0; then
        if test -d /usr/local/OPEN_R_SDK_ERS7; then # 1.1.4 / 1.1.5 dual installation?
          MSTREE=/usr/local/OPEN_R_SDK_ERS210/OPEN_R/MS/$WLAN/$DEBUG_MODE/
        else
          if test -d /usr/local/OPEN_R_SDK/OPEN_R/MS_ERS200; then # 1.1.5 installation?
            MSTREE=/usr/local/OPEN_R_SDK/OPEN_R/MS_ERS200/$WLAN/$DEBUG_MODE/
          else # 1.1.4 installation
            MSTREE=/usr/local/OPEN_R_SDK/OPEN_R/MS/$WLAN/$DEBUG_MODE/
          fi
        fi
      else
        if test -d /usr/local/OPEN_R_SDK_ERS7; then # 1.1.4 / 1.1.5 dual installation?
          MSTREE=/usr/local/OPEN_R_SDK_ERS7/OPEN_R/MS_ERS7/$WLAN/$DEBUG_MODE/
        else # 1.1.5 installation
          MSTREE=/usr/local/OPEN_R_SDK/OPEN_R/MS_ERS7/$WLAN/$DEBUG_MODE/
        fi
      fi
      cp -r $MSTREE/open-r $MSD
      if test $UNSAFE = 1; then
        chmod u+w $MSD/OPEN-R/SYSTEM/CONF/EXTOBJ.CFG
        grep -v EMGCYMON $MSTREE/OPEN-R/SYSTEM/CONF/EXTOBJ.CFG >$MSD/OPEN-R/SYSTEM/CONF/EXTOBJ.CFG
        chmod u-w $MSD/OPEN-R/SYSTEM/CONF/EXTOBJ.CFG
      fi
    fi
    

    #needed to avoid entering "start":
    mkdir -p $MSD/open-r/OTHERS/PSH
    echo "start" > $MSD/open-r/OTHERS/PSH/PSH.RC
    echo " " > $MSD/open-r/system/conf/MAV.CFG ;
    echo " " > $MSD/open-r/system/conf/OBJECT2.CFG ;
    if test $ERS7 = 1; then
      echo "JamDetectionHighThreshold" > $MSD/open-r/system/conf/VRCOMM.CFG
    fi
    
    #own binary stuff has to be copied always if --update is not given
    cp $UPD -r -f $MSOPENR/APP $MSD/open-r

    # copy tinyftpd server
    cp -u $TEAM_DIR/Util/TinyFTPD/bin/TINYFTPD.BIN $MSD/open-r/app/objs/tinyftpd.bin
   
    #sound files only have to be updated
    mkdir -p $MSD/open-r/app/conf/Sounds
    cp -u $TEAM_DIR/Config/Sounds/*.wav $MSD/open-r/app/conf/Sounds

    cp $UPD $TEAM_DIR/Config/empty_debug_request.dat $MSD/open-r/app/conf/requests.dat
    cp $UPD $TEAM_DIR/Config/spec_act.dat $MSD/open-r/app/conf/spec_act.dat
    cp $UPD $TEAM_DIR/Config/nc_pop1.dat $MSD/open-r/app/conf/nc_pop1.dat
    rm -f $MSD/open-r/app/conf/udset7.dat
    cp $UPD $TEAM_DIR/Config/udset7.dat $MSD/open-r/app/conf/udset7.dat
    cp $UPD `ls $TEAM_DIR/Config/*.cfg | grep -v player |grep -v obotgw` $MSD/open-r/app/conf

    cp -u $TEAM_DIR/Config/robotgw.cfg $MSD/open-r/mw/conf

    mkdir -p $MSD/open-r/app/conf/Xabsl2
    cp -r $UPD $TEAM_DIR/Config/Xabsl2/* $MSD/open-r/app/conf/Xabsl2

    mkdir -p $MSD/open-r/app/conf/Pfield/GT2004
    cp $UPD $TEAM_DIR/Config/Pfield/GT2004/*.pfc $MSD/open-r/app/conf/Pfield/GT2004

    LOCATION=`grep -v "\*" $TEAM_DIR/Config/location.cfg | grep "[A-Za-z0-9]"`
    if test x$LOCATION != x; then
      mkdir -p $MSD/open-r/app/conf/Location/$LOCATION
      # copy model-unspecific files only if they exist
			if test -n "$(ls -p1 $TEAM_DIR/Config/Location/$LOCATION/ | grep -v /)"; then
      	cp $TEAM_DIR/Config/Location/$LOCATION/*.* $MSD/open-r/app/conf/Location/$LOCATION
      fi
      if test $ERS7 = 0; then
      	MODEL="ers210"
      else
      	MODEL="ers7"
      fi
      mkdir -p $MSD/open-r/app/conf/Location/$LOCATION/$MODEL
      cp $TEAM_DIR/Config/Location/$LOCATION/$MODEL/*.* $MSD/open-r/app/conf/Location/$LOCATION/$MODEL
    fi
    cp $UPD $TEAM_DIR/Config/*.tab $MSD/open-r/app/conf

    if test $NOPLAYER = 0; then
      cp $UPD  $TEAM_DIR/Config/player.cfg $MSD/open-r/app/conf
      if test -f $TEAM_DIR/Config/wlanconf.txt; then
        cp $UPD $TEAM_DIR/Config/wlanconf.txt $MSD/open-r/system/conf
      else
        echo "$TEAM_DIR/Config/wlanconf.txt not found, create it, e.g. with PlayersToolbar:Save"
      fi
    fi

    echo "! ... done:"

    # give some post-copy info
    #
    if test -f $MSD/open-r/app/conf/player.cfg; then
      echo "player.cfg:"
      sed 's/^/  /' $MSD/open-r/app/conf/player.cfg
    else
      echo "warning: no player.cfg found on stick!"
    fi

    if test -f $MSD/open-r/system/conf/wlanconf.txt; then
      echo "wlanconf.txt:"
      sed 's/^/  /' $MSD/open-r/system/conf/wlanconf.txt
    else
      echo "warning: no wlanconf.txt found on stick!"
    fi

    if test $PAUSE = 1; then
      cmd /C pause
    fi

    if test $EJECT = 1; then
      $TEAM_DIR/Make/deveject.exe -EjectDrive:$MSD
      sleep 2
      $TEAM_DIR/Make/deveject.exe -EjectDrive:$MSD
    fi

  else
    echo ERROR: $MSD does not exist. Insert MemoryStick!
    cmd /C pause
  fi
fi

#
# Change Log:
#
# $Log: copyfiles.bash,v $
# Revision 1.21  2004/09/21 11:34:14  spranger
# improved
#
# Revision 1.20  2004/09/09 10:45:26  spranger
# added blutwurst
#
# Revision 1.19  2004/07/01 09:37:55  koh
# added ProjectPC to set MSD e:
#
# Revision 1.18  2004/06/30 11:47:32  cesarz
# removed copying of *.kst files in config folder
#
# Revision 1.17  2004/06/27 17:41:01  goetzke
# Added penalty-shooter.
#
# Revision 1.16  2004/06/15 12:27:21  nistico
# *** empty log message ***
#
# Revision 1.15  2004/06/15 10:52:15  thomas
# added my lap62
#
# Revision 1.14  2004/06/14 10:41:49  risler
# lap59
#
# Revision 1.13  2004/06/04 15:20:29  spranger
# changed blutwurst msd, once and for all ;)
#
# Revision 1.12  2004/06/02 17:24:18  spranger
# motionrequest cleanup
#
# Revision 1.11  2004/05/27 13:58:42  thomas
# remove messages for files or subdirectories on copy-commands
#
# Revision 1.10  2004/05/27 11:42:21  juengel
# bug fixed
#
# Revision 1.9  2004/05/27 11:15:36  thomas
# fix path for location-files
#
# Revision 1.7  2004/05/27 10:08:07  thomas
# added model-specific locations
#
# Revision 1.6  2004/05/25 19:44:46  jhoffman
# bug fix
#
# Revision 1.5  2004/05/25 19:33:49  wachter
# Added stick-detection for hosts without special entry.
# If --rm is used the user is asked if he really wants to delete the detected device ;)
#
# Revision 1.4  2004/05/25 14:10:03  spranger
# changed ms-drive for blutwurst
#
# Revision 1.3  2004/05/25 12:58:17  tim
# added missing potential field files and example behavior
#
# Revision 1.2  2004/05/23 20:18:16  loetzsch
# now also copies Xabsl2/HeadCtrl/* files
#
# Revision 1.1.1.1  2004/05/22 17:11:43  cvsadm
# created new repository GT2004_WM
#
# Revision 1.59  2004/04/21 06:00:38  dueffert
# UD parameterset for ERS7 with calibrated forward speeds
#
# Revision 1.58  2004/04/08 15:32:59  wachter
# GT04 checkin of Microsoft-Hellounds
#
# Revision 1.57  2004/03/26 16:19:58  park
# added host sparky
#
# Revision 1.56  2004/03/25 12:09:36  kindler
# - unsafe option should work now
#   (removed write protection of extobj.cfg)
#
# Revision 1.55  2004/03/23 10:28:04  roefer
# Minor bug fixed
#
# Revision 1.54  2004/03/22 21:55:40  roefer
# New options
#
# Revision 1.53  2004/03/19 21:17:10  altmeyer
# added host deepmo
#
# Revision 1.52  2004/03/16 14:00:15  juengel
# Integrated Improvments from "Gnne"
# -ATH2004ERS7Behavior
# -ATHHeadControl
# -KickSelectionTable
# -KickEditor
#
# Revision 1.51  2004/03/15 23:51:13  kindler
# - TinyFTPd should work again on the ERS-7
#    (object is now loaded as an app instead of mw, new ftppwd.cfg added)
#

