/**
* @file DefaultOptionRating.cpp
* 
* Implementation of class DefaultOptionRating
*
*/

#include "DefaultOptionRating.h"
#include "Tools/DynamicTeamTactic/RateableOptions.h"

// defined in OpenChallengeSymbols.cpp
#include "../GT2004Symbols/OpenChallengeSymbols.h"
extern OCSetStatus2 ocstate;

DefaultOptionRating::DefaultOptionRating(const BehaviorControlInterfaces& interfaces,
																				 CollectedBeliefs& collectedBeliefs,
                                         ChooserRaterInterfaces& chooserRaterInterfaces)
: OptionRating(interfaces, collectedBeliefs, chooserRaterInterfaces)
{
  for(int i=0; i<5; i++)
    ocrating[i] = 100;
}

// doing some preCalculations... used in rateOptions()
void DefaultOptionRating::doPreCals()
{
  if (ocstate == dtt_initial)
  {
    ocrating[0] = 0;
    ocrating[1] = 0;
    ocrating[2] = 0;
    ocrating[3] = 0;
    ocrating[4] = 0;
    ocrating[getPlayer().getPlayerNumber()] = 100;
  }
}

void DefaultOptionRating::rateOptions()
{
  int tmpno = (int)(getPlayer().getPlayerNumber());
  if (tmpno != Player::undefinedPlayerNumber)
  {
	  SingleBeliefs& mySingleBeliefs = collectedBeliefs.singleBeliefs[tmpno];
    int i;
    if (gameControlData.data.state == ROBOCUP_STATE_PENALIZED)
    {
      for (i=0; i < RateableOptions::numOfOptions; ++i) 
        mySingleBeliefs.ratedOptions[i] = 0; 
    }
    else
    {
   
      // common calculations, that can be used in more than one Rating to
	    // reduce redundant calculations, the variables are deklared in the 
	    // header to enabled subroutines to read/write these variables...
      doPreCals();

    /***********************************************************************************/
      // rate all your options here ;-)
      // from 0 to 100; 0 is bad, 100 is good
      //
      for (i=0; i < RateableOptions::numOfOptions; ++i) {
        switch((RateableOptions::OptionID)i) {

          case RateableOptions::stand: 
            mySingleBeliefs.ratedOptions[i] = 100; 
            break;

          case RateableOptions::doNothing: 
            mySingleBeliefs.ratedOptions[i] = 100; 
            break;

          case RateableOptions::keepOption: 
            mySingleBeliefs.ratedOptions[i] = 100; 
            break;


          case RateableOptions::intro: 
            mySingleBeliefs.ratedOptions[i] = 100;
            if ((ocstate == dtt_initial)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::extro: 
            mySingleBeliefs.ratedOptions[i] = 100;
            if ((ocstate != dtt_finished)
               &&(ocstate != dtt_finishedWalkWithBridge)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::finished: 
            mySingleBeliefs.ratedOptions[i] = 100;
            if ((ocstate != dtt_finished)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::gotoBitePos1:
            mySingleBeliefs.ratedOptions[i] = ocrating[0];
            if ( (ocstate != dtt_finishedIntro) &&
                 (ocstate != dtt_finishedGoToBitePoint)
               &&(ocstate != dtt_finishedBite)
               &&(ocstate != dtt_loseBitePoints)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::gotoBitePos2: 
            mySingleBeliefs.ratedOptions[i] = ocrating[1];
            if ((ocstate != dtt_finishedIntro) &&
                (ocstate != dtt_finishedGoToBitePoint)
               &&(ocstate != dtt_finishedBite)
               &&(ocstate != dtt_loseBitePoints)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::gotoBitePos3: 
            mySingleBeliefs.ratedOptions[i] = ocrating[2];
            if ((ocstate != dtt_finishedIntro) &&
                 (ocstate != dtt_finishedGoToBitePoint)
               &&(ocstate != dtt_finishedBite)
               &&(ocstate != dtt_loseBitePoints)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::gotoBitePos4: 
            mySingleBeliefs.ratedOptions[i] = ocrating[3];
            if ((ocstate != dtt_finishedIntro) &&
                 (ocstate != dtt_finishedGoToBitePoint)
               &&(ocstate != dtt_finishedBite)
               &&(ocstate != dtt_loseBitePoints)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::bitePos1: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos1];
            if ((ocstate != dtt_finishedGoToBitePoint)
               &&(ocstate != dtt_finishedBite)
               &&(ocstate != dtt_loseBitePoints)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::bitePos2: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos2];;
            if ((ocstate != dtt_finishedGoToBitePoint)
               &&(ocstate != dtt_finishedBite)
               &&(ocstate != dtt_loseBitePoints)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::bitePos3: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos3];
            if ((ocstate != dtt_finishedGoToBitePoint)
               &&(ocstate != dtt_finishedBite)
               &&(ocstate != dtt_loseBitePoints)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::bitePos4: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos4];
            if ((ocstate != dtt_finishedGoToBitePoint)
               &&(ocstate != dtt_finishedBite)
               &&(ocstate != dtt_loseBitePoints)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::movePos1: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::bitePos1];
            if ((ocstate != dtt_finishedBite)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::movePos2: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::bitePos2];
            if ((ocstate != dtt_finishedBite)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::movePos3: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::bitePos3];
            if ((ocstate != dtt_finishedBite)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::movePos4: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::bitePos4];
            if ((ocstate != dtt_finishedBite)
               &&(ocstate != dtt_finishedExtro)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::gotoBridge: 
            mySingleBeliefs.ratedOptions[i] = ocrating[4];
            if ((ocstate != dtt_finishedIntro)&&
               (ocstate != dtt_finishedExtro)
               &&(ocstate != dtt_finishedGoToBridge)
               &&(ocstate != dtt_finishedClimbBridge)
               &&(ocstate != dtt_finishedWalkWithBridge)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::climbBridge: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::gotoBridge];
            if ((ocstate != dtt_finishedExtro)
               &&(ocstate != dtt_finishedGoToBridge)
               &&(ocstate != dtt_finishedClimbBridge)
               &&(ocstate != dtt_finishedWalkWithBridge)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::moveBridge: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::climbBridge];
            if ((ocstate != dtt_finishedExtro)
               &&(ocstate != dtt_finishedClimbBridge)
               &&(ocstate != dtt_finishedWalkWithBridge)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;
          case RateableOptions::waitForBiteDogs: 
            mySingleBeliefs.ratedOptions[i] = mySingleBeliefs.ratedOptions[RateableOptions::climbBridge];
            if ((ocstate != dtt_finishedExtro)
               &&(ocstate != dtt_finishedClimbBridge)
               &&(ocstate != dtt_finishedWalkWithBridge)
               )  mySingleBeliefs.ratedOptions[i] = 0;
            break;

          default: 
            mySingleBeliefs.ratedOptions[i] = 0;
            break;
        }
      }
      mySingleBeliefs.ratedOptions[chooserRaterInterfaces.lastChoosenOption] +=
        RateableOptions::getRatingBoost(chooserRaterInterfaces.lastChoosenOption);

  LEDRequest::BlinkMode bmode = LEDRequest::oooo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos1] > 25)
    bmode = LEDRequest::looo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos1] > 50)
    bmode = LEDRequest::lloo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos1] > 75)
    bmode = LEDRequest::lllo;
  ledRequest.faceLED[1] = bmode;

  bmode = LEDRequest::oooo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos2] > 25)
    bmode = LEDRequest::looo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos2] > 50)
    bmode = LEDRequest::lloo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos2] > 75)
    bmode = LEDRequest::lllo;
  ledRequest.faceLED[0] = bmode;

  bmode = LEDRequest::oooo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos3] > 25)
    bmode = LEDRequest::looo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos3] > 50)
    bmode = LEDRequest::lloo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos3] > 75)
    bmode = LEDRequest::lllo;
  ledRequest.faceLED[3] = bmode;

  bmode = LEDRequest::oooo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos4] > 25)
    bmode = LEDRequest::looo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos4] > 50)
    bmode = LEDRequest::lloo;
  if(mySingleBeliefs.ratedOptions[RateableOptions::gotoBitePos4] > 75)
    bmode = LEDRequest::lllo;
  ledRequest.faceLED[2] = bmode;

    } 
	  // set timestamp - only used locally. The TimeStamps of the other dogs
	  // come from TeamMessageCollection and are the TimeStamps, when the
	  // data was send. Don't worry. This doesn't cause any problems. Just
	  // remember that we're single threadet ;-)
	  mySingleBeliefs.timeStamp = SystemCall::getCurrentSystemTime();
  } // endif playernumber
}


/*
 * Change log :
 * $Log: DefaultOptionRating.cpp,v $
 * Revision 1.1  2004/07/22 22:38:31  kerdels
 * added DTT used by Open Challenge, RIP and Xabsl-Options will follow
 *
 *
 */
