/**
 * @file DefaultTacticChooser.h
 * 
 * Implementation of class DefaultTacticChooser.
 *
 * @author <A href=mailto:jochen@kerdels.de>Jochen Kerdels</A>
 */

#ifndef __DefaultTacticChooser_h_
#define __DefaultTacticChooser_h_

#include "Tools/DynamicTeamTactic/TacticChooser.h"
#include "Tools/DynamicTeamTactic/TacticEntry.h"
#include "Tools/DynamicTeamTactic/ChooserRaterInterfaces.h"



/**
* @class DefaultTacticChooser
*
* A simple implementation of the DefaultTacticChooser class.
*
*/
class DefaultTacticChooser : public TacticChooser
{
private:
	int facOfNumOfPlayerNumbers;
	int* dogPermutation[Player::numOfPlayerNumbers];
public:
  /** 
   * Constructor.
   * @param interfaces The parameters of the TacticChooser class.
   * @param collectedBeliefs The collected-Beliefs to work on.
   * @param chooserRaterInterfaces We want to talk to the Option-Rating.
   */
  DefaultTacticChooser(const BehaviorControlInterfaces& interfaces,
                       CollectedBeliefs& collectedBeliefs,
                       ChooserRaterInterfaces& chooserRaterInterfaces);

	/*
	 * Destructor.
	 */
	~DefaultTacticChooser();

  /** the TacticEntryArray to work with... */
  TacticEntryArray tacticEntryArray;

  /** if true, the robots should stand still, e.g. using option stand */
  int freeze;

  /** choosing one option in here... */
  RateableOptions::OptionID chooseOption();

  // open challenge
  /** role of team player*/
  RateableOptions::OptionID ocplayerrole[Player::numOfPlayerNumbers];

  /** receiving new TacicEntryArrays in here */
  virtual bool handleMessage(InMessage& message);
};

#endif // __DefaultTacticChooser_h_

/*
 * Change log :
 * $Log: DefaultTacticChooser.h,v $
 * Revision 1.1  2004/07/22 22:38:31  kerdels
 * added DTT used by Open Challenge, RIP and Xabsl-Options will follow
 *
 *
 */
