/** 
* @file GT2004DTTSymbols.cpp
*
* Implementation of class GT2004DTTSymbols.
*
* @author Jochen Kerdels
*/

#include "GT2004DTTSymbols.h"
#include "Tools/DynamicTeamTactic/RateableOptions.h"

GT2004DTTSymbols::GT2004DTTSymbols(const BehaviorControlInterfaces& interfaces,
																		 TacticChooser** tacticChooser)
: BehaviorControlInterfaces(interfaces)
{
	actualTacticChooser = tacticChooser;
	actualOption = RateableOptions::noOption;
  actualOptionType = RateableOptions::special;
}


void GT2004DTTSymbols::registerSymbols(Xabsl2Engine& engine)
{
  // "default-dtt"
	engine.registerEnumeratedInputSymbol("default-dtt",(int *)&actualOption);
	// add here new Options
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","doStand"                       ,(int)RateableOptions::stand);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","doNothing"                 ,(int)RateableOptions::doNothing);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","noOption"                  ,(int)RateableOptions::noOption);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCintro"                ,(int)RateableOptions::intro);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCextro"                ,(int)RateableOptions::extro);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCfinished"             ,(int)RateableOptions::finished);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCgotoBitePos1"         ,(int)RateableOptions::gotoBitePos1);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCgotoBitePos2"         ,(int)RateableOptions::gotoBitePos2);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCgotoBitePos3"         ,(int)RateableOptions::gotoBitePos3);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCgotoBitePos4"         ,(int)RateableOptions::gotoBitePos4);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCbitePos1"             ,(int)RateableOptions::bitePos1);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCbitePos2"             ,(int)RateableOptions::bitePos2);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCbitePos3"             ,(int)RateableOptions::bitePos3);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCbitePos4"             ,(int)RateableOptions::bitePos4);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCmovePos1"             ,(int)RateableOptions::movePos1);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCmovePos2"             ,(int)RateableOptions::movePos2);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCmovePos3"             ,(int)RateableOptions::movePos3);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCmovePos4"             ,(int)RateableOptions::movePos4);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCgotoBridge"           ,(int)RateableOptions::gotoBridge);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCclimbBridge"          ,(int)RateableOptions::climbBridge);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCmoveBridge"           ,(int)RateableOptions::moveBridge);
  engine.registerEnumeratedInputSymbolEnumElement("default-dtt","OCWaitForBiteDogs"      ,(int)RateableOptions::waitForBiteDogs);


  // "dtt-option-type"
	engine.registerEnumeratedInputSymbol("dtt-option-type",(int *)&actualOptionType);
	// add here new Options
  engine.registerEnumeratedInputSymbolEnumElement("dtt-option-type","special"        ,(int)RateableOptions::special);
  engine.registerEnumeratedInputSymbolEnumElement("dtt-option-type","gotoPos"        ,(int)RateableOptions::gotoPos);
  engine.registerEnumeratedInputSymbolEnumElement("dtt-option-type","gotoBallPos"    ,(int)RateableOptions::gotoBallPos);
  engine.registerEnumeratedInputSymbolEnumElement("dtt-option-type","kickAction"     ,(int)RateableOptions::kickAction);
  engine.registerEnumeratedInputSymbolEnumElement("dtt-option-type","goalieAction"   ,(int)RateableOptions::goalieAction);

}

void GT2004DTTSymbols::update()
{
	actualOption = (*actualTacticChooser)->chooseOption();
  
  actualOptionType = RateableOptions::getOptionType(actualOption);
}

  

/*
* Change Log
* 
* $Log: GT2004DTTSymbols.cpp,v $
* Revision 1.1  2004/07/22 22:38:20  kerdels
* added DTT used by Open Challenge, RIP and Xabsl-Options will follow
*
*
*
*/

