/** 
* @file OpenChallengeSymbols.h
*
* Declaration of class BallSymbols.
*
* @author 
*/

#ifndef __OpenChallengeSymbols_h_
#define __OpenChallengeSymbols_h_

#include "../../BehaviorControl.h"
#include "Tools/Xabsl2/Xabsl2Engine/Xabsl2Engine.h"
#include "Tools/SlidingAverageValue.h"

// this will be changed until release...
enum OCSetStatus2
{
  dtt_initial = 0,
  dtt_start,
  dtt_finishedIntro,
  dtt_finishedGoToBridge,
  dtt_finishedClimbBridge,
  dtt_finishedWalkWithBridge,
  dtt_finishedGoToBitePoint,
  dtt_finishedBite,
  dtt_loseBitePoints,
  dtt_finishedExtro,
  dtt_finished
};

typedef struct OC
{
  double type;
  double x;
  double y;
  double rotation;
} OC;

class OpenChallengeSymbols : public Xabsl2FunctionProvider, public BehaviorControlInterfaces
{
public:

  enum OCStatus
  {
    initial = 0,
    intro,
    goToBridge,
    goToBitePoint,
    allBitePointsReady,
    walkWithBridge,
    extro,
    finished
  };

  enum OCSetStatus
  {
    setInitial = 0,
    setIntro,
    setGoToBridge,
    setGoToBitePoint,
    setMyBitePointIsReady,
    setLostMyBitePoint,
    setWalkWithBridge,
    setExtro,
    setFinished
  };

  enum CRIP{
      unchanged,
      rip
  };

/*
* Constructor.
* @param interfaces The parameters of the BehaviorControl module.
  */
  OpenChallengeSymbols(const BehaviorControlInterfaces& interfaces);
  
  /** registers the symbols at an engine */
  void registerSymbols(Xabsl2Engine& engine);

  /** updates the symbols */
  void update();

  //!@name Input symbols
  //!@{
  // bridge
  OCBridge::Side getLastSeenSide();    /*!< A function for the symbol "bridge.last-seen-side" */
  double getSeenDistanceToBridge();    /*!< A function for the symbol "bridge.seen.distance" */
  double getSeenAngleToBridge();       /*!< A function for the symbol "bridge.seen.angle" */
  double getTimeSinceLastSeenBridge(); /*!< A function for the symbol "bridge.time-since-last-seen" */

  // bite point
  BitePoint::Position getBitePointPosition(); /*!< A function for the symbol "bite-point.position" */
  double getSeenDistanceToBitePoint();        /*!< A function for the symbol "bite-point.seen.distance" */
  double getSeenAngleToBitePoint();           /*!< A function for the symbol "bite-point.seen.angle" */
  double getPreciseShiftToBitePoint();        /*!< A function for the symbol "bite-point.precise-shift" */
  double getTimeSinceLastSeenBitePoint();     /*!< A function for the symbol "bite-point.time-since-last-seen" */

  // red line
  double redLineInImageAngle();               /*!< A function for the symbol "bite-point.time-since-last-seen" */
  double redLineAngle();                      /*!< A function for the symbol "bite-point.time-since-last-seen" */
  double getTimeSinceLastSeenRedLine();     /*!< A function for the symbol "bite-point.time-since-last-seen" */
  double getSeenDistanceToRedLine();    /*!< A function for the symbol "bridge.seen.distance" */

  // PSD data
  double getSensorPSDNear();            /*!< A function for the symbol "open-challenge.headpsd.near" */
  double getSensorPSDFar();             /*!< A function for the symbol "open-challenge.headpsd.far" */
  double getDistanceToSIFOC();          /*!< A function for the symbol "open-challenge.distance-to-sifoc" */
  
  // acceleration data
  double getSensorAccelerationX();      /*!< A function for the symbol "open-challenge.accelerator.x" */
  double getSensorAccelerationY();      /*!< A function for the symbol "open-challenge.accelerator.y" */
  double getSensorAccelerationZ();      /*!< A function for the symbol "open-challenge.accelerator.z" */

  // ramp
  double getAiboOnRamp();               /*!< A function for the symbol "open-challenge.on-ramp" */
  double getRampAngleOfRedLine();       /*!< A function for the symbol "ramp.angle-of-line" */
  double getRampLowestLinePointX();     /*!< A function for the symbol "ramp.lowest-line-point.x" */
  double getRampLowestLinePointY();     /*!< A function for the symbol "ramp.lowest-line-point.y" */
  double getRampHighestLinePointX();    /*!< A function for the symbol "ramp.highest-line-point.x" */
  double getRampHighestLinePointY();    /*!< A function for the symbol "ramp.highest-line-point.y" */
  double getRampDistanceToStartPoint(); /*!< A function for the symbol "ramp.lowest-line-point.distance" */
  double getRampTimeSinceLastSeen();    /*!< A function for the symbol "ramp.time-since-last-seen" */

  // switch ImageProcessor
  void setRIP(int t);                   /*!< A function for the symbol "openchallenge-image" */
  // set stati
  void setOCStatusDtt(int aStatus);     /*!< A function for the symbol "open-challenge.set-status-dtt" */

  
  //!@}


  /** Varaible to hold the open challenge status */
  OCStatus ocStatus;

  /* A function for the output symbol "open-challenge.set-status". */
  void setOCStatus(int aStatus);


private:
  OC m_walkRequest;
  double m_headrot;

  int bitePointToUse[Player::numOfPlayerNumbers];
  bool bitePointReady[BitePoint::numOfPositions];
  int masterDog;

  // ramp
  SlidingAverageValue xspeed;     // used at getAiboOnRamp()

  // alread-moved-bridge
  int ambVal;
  // learn acceleration data
  int learnAcc;
  SlidingAverageValue groundVal;


};




#endif // OpenChallengeSymbols

/*
* Change Log
* 
* $Log: OpenChallengeSymbols.h,v $
* Revision 1.4  2004/08/09 13:51:27  hamerla
* import OpenChallen Behavior to WM
*
* Revision 1.3  2004/07/22 22:38:20  kerdels
* added DTT used by Open Challenge, RIP and Xabsl-Options will follow
*
* Revision 1.2  2004/06/01 13:58:14  schumann
* special percept type redLine added,
* part 1 of open challenge added
*
* Revision 1.1.1.1  2004/05/22 17:18:04  cvsadm
* created new repository GT2004_WM
*
* Revision 1.4  2004/05/17 18:20:00  kerdels
* adjusted symbols for open challenge to enable free mapping of players to bitePoints- and controllerpositions
*
* Revision 1.3  2004/05/17 17:17:50  kerdels
* added symbols for the sequence control of the open challenge,
* fixed a bug concerning 5-dog-mode and distribution of option-ratings,
* set one = 0 in enum Player::playerNumber --> just to be sure ;-)
*
* Revision 1.2  2004/05/17 01:56:15  kerdels
* prepared some symbols for the open challenge bridge detection
*
* Revision 1.1  2004/05/08 16:18:14  hamerla
* Open Challenge
*
*
*/

