/**
* @file SLAMSymbols.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/** 
* @file SLAMSymbols
*
* Implementation of class SLAMSymbols
*
* @author DAT
*/

#include "SLAMSymbols.h"

SLAMSymbols::SLAMSymbols(BehaviorControlInterfaces& interfaces)
: BehaviorControlInterfaces(interfaces)
{
}


void SLAMSymbols::registerSymbols(Xabsl2Engine& engine)
{
  engine.registerEnumeratedOutputSymbol("slam-mode-out",&slamData.slamModeOut); 
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.offline",SLAMData::offline);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.resetImageProcessor",SLAMData::resetImageProcessor);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.resetSelfLocator",SLAMData::resetSelfLocator);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.standby",SLAMData::standby);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.loadClusters",SLAMData::loadClusters);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.recordColors",SLAMData::recordColors);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.clusterColors",SLAMData::clusterColors);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.saveClusters",SLAMData::saveClusters);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.loadPanorama",SLAMData::loadPanorama);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.recordPanorama",SLAMData::recordPanorama);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.savePanorama",SLAMData::savePanorama);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.localize",SLAMData::localize);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-mode-out","slam-mode-out.normalize",SLAMData::normalize);

  engine.registerEnumeratedInputSymbol("slam-mode-in",&slamData.currentAction); 
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.offline",SLAMData::offline);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.resetImageProcessor",SLAMData::resetImageProcessor);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.resetSelfLocator",SLAMData::resetSelfLocator);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.standby",SLAMData::standby);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.loadClusters",SLAMData::loadClusters);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.recordColors",SLAMData::recordColors);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.clusterColors",SLAMData::clusterColors);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.saveClusters",SLAMData::saveClusters);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.loadPanorama",SLAMData::loadPanorama);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.recordPanorama",SLAMData::recordPanorama);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.savePanorama",SLAMData::savePanorama);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.localize",SLAMData::localize);
  engine.registerEnumeratedInputSymbolEnumElement("slam-mode-in","slam-mode-in.normalize",SLAMData::normalize);

  engine.registerEnumeratedOutputSymbol("learning-pose-x",&slamData.learningPoseX); 
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-x","x.2back",-2);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-x","x.1back",-1);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-x","x.center",0);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-x","x.1forth",1);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-x","x.2forth",2);

  engine.registerEnumeratedOutputSymbol("learning-pose-y",&slamData.learningPoseY); 
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-y","y.2right",-2);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-y","y.1right",-1);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-y","y.center",0);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-y","y.1left",1);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-y","y.2left",2);

  engine.registerEnumeratedOutputSymbol("learning-pose-phi",&slamData.learningPosePhi); 
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.0",0);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.15",15);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.30",30);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.45",45);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.60",60);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.75",75);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.90",90);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.105",105);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.120",120);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.135",135);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.150",150);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.165",165);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.180",180);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.195",195);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.210",210);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.225",225);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.240",240);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.255",255);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.270",270);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.285",285);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.300",300);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.315",315);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.330",330);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-phi","phi.345",345);

  engine.registerEnumeratedOutputSymbol("learning-pose-number",&slamData.learningPoseNumber); 
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.0",0);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.1",1);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.2",2);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.3",3);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.4",4);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.5",5);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.6",6);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.7",7);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.8",8);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.9",9);
  engine.registerEnumeratedOutputSymbolEnumElement("learning-pose-number","spot.10",10);

  engine.registerEnumeratedOutputSymbol("slam-bearing-mode",&slamData.bearingMode); 
  engine.registerEnumeratedOutputSymbolEnumElement("slam-bearing-mode","slam-bearing-mode.unchanged",SLAMData::unchangedBearing);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-bearing-mode","slam-bearing-mode.training",SLAMData::trainingBearing);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-bearing-mode","slam-bearing-mode.estimate",SLAMData::estimateBearing);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-bearing-mode","slam-bearing-mode.estimate-slow",SLAMData::estimateBearingSlow);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-bearing-mode","slam-bearing-mode.estimate-nodecay",SLAMData::estimateBearingNoDecay);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-bearing-mode","slam-bearing-mode.set-reference-0",SLAMData::setReference0);

  engine.registerEnumeratedOutputSymbol("slam-localization-mode",&slamData.localizationMode); 
  engine.registerEnumeratedOutputSymbolEnumElement("slam-localization-mode","slam-localization-mode.odometry",SLAMData::odometry);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-localization-mode","slam-localization-mode.training",SLAMData::trainingSpot);
  engine.registerEnumeratedOutputSymbolEnumElement("slam-localization-mode","slam-localization-mode.probabilistic",SLAMData::probabilistic);
}

/*
* Change Log
* 
* $Log: TaskVisionSymbols.cpp,v $
* Revision 1.7  2005/02/04 11:45:32  floris
* last updates made to task image processing algorithms
*
* Revision 1.6  2005/01/07 15:32:59  floris
* task-vision-request striker works much better then DT2004
*
* Revision 1.5  2004/12/08 17:22:53  floris
* Task Image Processor restructured
*
* Revision 1.4  2004/11/18 14:18:42  floris
* simple checking of pose for goalie-guard implemented
*
* Revision 1.3  2004/11/12 16:32:40  floris
* keeper further improved
*
* Revision 1.2  2004/11/05 09:44:55  floris
* taskVisionRequest implemented
*
* Revision 1.1  2004/11/01 08:26:58  floris
* playing goalie has acces to taskvisiondata
*
* Revision 1.1  2004/06/08 15:39:14  andrew
* Renamed GT2003 to DT2004
*
* Revision 1.2  2004/02/27 11:31:13  wcaarls
* Import of Utrecht changes as of 26-02-2004.
*
* Revision 1.1.1.1  2004/01/15 12:30:25  bottens
*
*
* Revision 1.1.1.1  2003/07/02 09:40:23  cvsadm
* created new repository for the competitions in Padova from the 
* tamara CVS (Tuesday 2:00 pm)
*
* removed unused solutions
*
* Revision 1.1  2003/05/07 17:54:46  juengel
* Added missing symbols to DT2004BehaviorControl.
*
*/

