/**
* @file ColorClustering.h
* 
* Definition of class ColorClustering
*
* @author Jurgen Sturm
*/


#ifndef __ColorClustering_h_
#define __ColorClustering_h_

#include "ScanningGrid.h"
#include "Modules/ImageProcessor/PanoramaImageProcessor/PanoramaImageProcessor.h"
#include "Representations/Perception/PanoramaParameters.h"

unsigned char colorDistance(Vector3<unsigned char> &color1,Vector3<unsigned char> &color2);

class ColorRecordScanParam {
	public:
		Vector3<unsigned char> lastColor;
		int histogram[256];
		int count;
		int threshold;
		ColorRecordScanParam();
		void reset();
		void recomputeThreshold();
};

class ColorCollection {
	public:
		int virtualLength;
		int length;
		Vector3<unsigned char> color[COLLECT_COLORS];

		ColorCollection();
		void insertColor(Vector3<unsigned char>color);
		void showCollectedColors(Image &debugImage);
};

class ColorHistogram {
	public:
		unsigned char count[COLOR_CUBES][COLOR_CUBES][COLOR_CUBES];
		ColorCollection strongColorsRemoved;

		ColorHistogram(ColorCollection &colorCollection);
		bool insertColor(Vector3<unsigned char>color);
};


class ColorClustering:public ScanningGrid
{
public:
  ColorClustering(const Image &image,const CameraMatrix &cameraMatrix,const CameraMatrix &prevCameraMatrix,Image &debugImage);

	// reserves memory for colorCollection
	void recordPrepare();
	// using the scanning grid, extracts interesting colors
	void recordExecute();
	// prepares colorHistogram and colorMatrix, finally destroys colorCollection and colorHistogram
	void clusteringPrepare();		
	// merge close clusters
  void clusteringExecute();
	// fills mean, weight, lookupTable, finally destroys ColorMatrix
	void clusteringFinish();

	ColorRecordScanParam recordScanParam;
	ColorCollection *colorCollection;
	ColorHistogram *colorHistogram;
	EMGaussianMixture mixture;

	double mergeDistance;


	Image &debugImage;

	inline bool scanPixel(
		int &is,int &ix,int &iy,
		int &x,int &y,
		Vector3<unsigned char> &color,double &radScanline,ColorRecordScanParam &param);

	// pass-through functions to EMGaussianMixture mixture
	unsigned char assignCluster(Vector3<unsigned char> &color);
	void saveLookupTable(const char* filename);
	bool loadLookupTable(const char* filename);
	void clusterImage();
	
};

#endif// __ColorClustering_h_

/*
*
*
*/
