/**
* @file FastPanoramaProcessor.h
* 
* Definition of class FastPanoramaProcessor
*
* @author Jurgen Sturm
*/


#ifndef __FastPanoramaProcessor_h_
#define __FastPanoramaProcessor_h_

#include "Modules/ImageProcessor/ImageProcessor.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Tools/Math/Geometry.h"
#include "Tools/Debugging/DebugImages.h"
#include "Modules/ImageProcessor/ImageProcessorTools/ColorCorrector.h"
#include "Modules/ImageProcessor/GT2004ImageProcessor/GT2004ImageProcessorTools.h"
#include "Representations/Perception/Image.h"
#include "Representations/Perception/PanoramaLibrary.h"

#include "ScanningGrid.h"
#include "ColorClustering.h"
#include "SignatureGenerator.h"
#include "HorizonEstimator.h"

using namespace Panorama;

class FastPanoramaProcessor : public ImageProcessor
{
public:
  FastPanoramaProcessor(const ImageProcessorInterfaces& interfaces);
  
  /** Executes the module */
  virtual void execute();

	void framePrepare();
	void frameExecute();
	void frameFinish();

private:

	ColorClustering cluster;
	SignatureGenerator signatureGenerator;
	HorizonEstimator horizonEstimator;
	CameraMatrix prevCameraMatrix;

	int statEmptyFrames;
	int statFrameCount;

	Image clusterDebug;
	Image signatureDebug;

  DECLARE_DEBUG_COLOR_CLASS_IMAGE(segmentedImage1);
  DECLARE_DEBUG_IMAGE(imageProcessorGeneral);
  DECLARE_DEBUG_IMAGE(imageProcessorBall);
  DECLARE_DEBUG_IMAGE(imageProcessorGoals);

};


#endif// __FastPanoramaProcessor_h_

/*
*
*
*/
