/**
* @file HorizonEstimator.h
* 
* Definition of class HorizonEstimator
*
* @author Jurgen Sturm
*/


#ifndef __HorizonEstimator_h_
#define __HorizonEstimator_h_

#include "ScanningGrid.h"
#include "ColorClustering.h"
#include "Representations/Perception/PanoramaLibrary.h"
#include "Representations/Perception/LandmarksPercept.h"

class HorizonScanData {
	public:
		// global data
		PanoramaHistogram &panoramaHistogram;
		int distribution[2][MAX_CLUSTERS]; // cluster distribution, above and below calculated horizon
		int horizonHeightSum;
		int horizonHeightCount;

		void resetDistribution();

		// per sector
		int sector;
		bool seenTop,seenBottom;			// sector fully visible?
		int freq[HORIZON_SIZE][MAX_CLUSTERS];
		void resetFrequencies();

		int estimateMLHorizonHeight();

		void resetSectorData();

		HorizonScanData(PanoramaHistogram &panoramaHistogram);
};

class HorizonEstimator:public ScanningGrid
{
	public:
		HorizonEstimator(
			const Image &image,
			const CameraMatrix &cameraMatrix,
			const CameraMatrix &prevCameraMatrix, 
			ColorClustering &cluster, 
			LandmarksPercept &landmarksPercept,
			const RobotPose &robotPose,
			Image &debugImage);

		// scans the image and finally copys panoramaHistogram to landmarksPercept
		void execute();	

		Image &debugImage;

		ColorClustering &cluster;

		HorizonScanData horizonScanData;

		LandmarksPercept &landmarksPercept;

		const RobotPose &robotPose;

		inline bool scanPixel(
			int &is,int &ix,int &iy,
			int &x,int &y,
			Vector3<unsigned char> &color,
			double &radScanline,
			HorizonScanData &param);

};


#endif// __HorizonEstimator_h_

/*
*
*
*/
