/**
* @file SignatureGenerator.cpp
* 
* 
* @author Jurgen Sturm
*/

#include "SignatureGenerator.h"
#include "Tools/Debugging/Debugging.h"

#define TO_BIN(a) (a<= 0 ?0:  \
									(a<= 18 ?1:	\
									(a<= 36 ?2:	\
									(a<= 60 ?3:	\
												  4))))

#define SET_PIXEL(x1,y1,c1,c2,c3)		\
{ \
	int sx=horizontal[min(x1,horizontalScanPoints-1)].x+vertical[1][min(y1,verticalScanPoints-1)].x; \
	int sy=horizontal[min(x1,horizontalScanPoints-1)].y+vertical[1][min(y1,verticalScanPoints-1)].y; \
	if(sx>=0 && sx<=cameraResolutionWidth_ERS7 && sy>=0 && sy<=cameraResolutionWidth_ERS7) { \
		 debugImage.image[sy][0][sx]=c1; \
		 debugImage.image[sy][1][sx]=c2; \
		 debugImage.image[sy][2][sx]=c3; \
	} \
}

#define DRAW_TRANS(ixStart,pos,c1,c2)		\
{	\
	int x;	\
	int y;	\
	int ix;	\
	int iy;	\
	int SIZE=13;	\
	for(x=0;x<SIZE;x++)	\
		for(y=0;y<SIZE;y++) {	\
			ix = ixStart+x+3;	\
			iy = y+10+17*pos;	\
			if(x==0 || y == 0 || x==SIZE-1 ||y==SIZE-1 || x==y) {	\
				SET_PIXEL(ix,iy,0,128,128);	\
			} else if(x<y) {	\
				SET_PIXEL(ix,iy,cluster.mixture.clusterColor[c1].x,cluster.mixture.clusterColor[c1].y,cluster.mixture.clusterColor[c1].z);	\
			} else {	\
				SET_PIXEL(ix,iy,cluster.mixture.clusterColor[c2].x,cluster.mixture.clusterColor[c2].y,cluster.mixture.clusterColor[c2].z);	\
			}	\
		}	\
}


SignatureScanData::SignatureScanData(PanoramaHistogram &panoramaHistogram)
	:	panoramaHistogram(panoramaHistogram) 
{
}


void SignatureScanData::reset() {
	lastCluster = -1;
	sector = -1;
	panoramaHistogram.reset();

	for(int i=0;i<SECTORS;i++) {
		pixelCount[i]=0;
		for(int cluster=0;cluster<CLUSTERS;cluster++)  {
			minmax[i][cluster][0] = DIAGONAL;
			minmax[i][cluster][1] = 0;
		}
	}
}

SignatureGenerator::SignatureGenerator(
			const Image &image,
			const CameraMatrix &cameraMatrix,
			const CameraMatrix &prevCameraMatrix, 
			ColorClustering &cluster, 
			LandmarksPercept &landmarksPercept,
			const RobotPose &robotPose,
			Image &debugImage)
: 
	ScanningGrid(image,cameraMatrix,prevCameraMatrix),
	cluster(cluster),
	debugImage(debugImage),
	robotPose(robotPose),
	landmarksPercept(landmarksPercept),
	signatureScanData(landmarksPercept.panoramaHistogram) 
{
//	OUTPUT(idText,text,"sizeof(SignatureGenerator)="<<sizeof(SignatureGenerator))  ;
	//OUTPUT(idText,text,"sizeof(LandmarksPercept)="<<sizeof(LandmarksPercept))  ;

//original values: y=4,x=1
	scanningDistanceY = SCANNING_Y;
	scanningDistanceX = SCANNING_X;
	WINDEBUG(scanningDistanceY = 1;)

}

void SignatureGenerator::execute() {
	WINDEBUG(
		int x;
		int y;
		for(y=0;y<cameraResolutionHeight_ERS7;y++) {
			for(x=0;x<cameraResolutionWidth_ERS7;x++) {
				debugImage.image[y][0][x] = image.image[y][0][x];
				debugImage.image[y][1][x] = image.image[y][1][x];
				debugImage.image[y][2][x] = image.image[y][2][x];
//				debugImage.image[y][1][x] = 128;
//				debugImage.image[y][2][x] = 128;
			}
		}
	)

	signatureScanData.reset();

//	STOPWATCH("SCAN_IMAGE ",SCAN_IMAGE(false,false,true,scanPixel,recordScanParam);	)
	SCAN_IMAGE(false,false,true,scanPixel,signatureScanData);	
//	SCAN_IMAGE_DETAIL(false,false,true,scanPixel,signatureScanData,true,scanPixelDetail,signatureScanData);


	// normalize
	int sector,m=0;
	for(sector=0;sector<SECTORS;sector++) 
		m = max(m,	signatureScanData.pixelCount[sector]);

	//OUTPUT(idText,text,"pixels in sector: "<<m);

	m = m*3/4;


	for(sector=0;sector<SECTORS;sector++) {
		SectorHistogram &currentSector = signatureScanData.panoramaHistogram.sector[sector];
		if(currentSector.empty) continue;
		if(signatureScanData.pixelCount[sector]<m) 
			currentSector.empty = true;

		for( int i=0;i<CLUSTERS;i++)
		for( int j=0;j<CLUSTERS;j++) {
			currentSector.freq[i][j] = (i==j)?TO_BIN(currentSector.freq[i][j]/5*scanningDistanceY):TO_BIN(currentSector.freq[i][j]*scanningDistanceY);
		}

	}
/*
// debug: paint sector transitions(only transitions in bin 5)
	for(sector=0;sector<SECTORS;sector++) {
		SectorHistogram &currentSector = signatureScanData.panoramaHistogram.sector[sector];
		if(currentSector.empty) continue;
		
		int ixStart = signatureScanData.ixStart[sector];
		int pos=0;
		for( int i=0;i<CLUSTERS;i++)
		for( int j=0;j<CLUSTERS;j++) {
			if(currentSector.freq[i][j]==4) {
				DRAW_TRANS(ixStart,pos,i,j);
				pos++;
			}
		}
	}
	*/

//	
//	WINDEBUG(
//		visualizeHistogramOnSoccerField();
//	)
}

inline bool SignatureGenerator::scanPixel(
		int &is,int &ix,int &iy,
		int &x,int &y,
		Vector3<unsigned char> &color,
		double &radScanline,
		SignatureScanData &signatureScanData) {

	unsigned char currentCluster = cluster.mixture.lookupTable[color.x>>2][color.y>>2][color.z>>2];

	static int change;
	if(iy==0) {
		change++;
		signatureScanData.lastCluster = currentCluster;
		int old=signatureScanData.sector;
		signatureScanData.sector = (int)((-radScanline/(2*PI) + 0.5)*SECTORS);// same formula as in PanoramaImageProcessor
		WINDEBUG(
			if(old!=signatureScanData.sector) {
				change = 0;
				signatureScanData.ixStart[signatureScanData.sector] = ix;
			}
			if((signatureScanData.sector/2) % 2 == 0) 
				debugImage.image[y][0][x] = 255;
			else
				debugImage.image[y][0][x] = 0;

			debugImage.image[y][1][x] = 128;
			debugImage.image[y][2][x] = 128;
		)
		return(false);
	} 

	WINDEBUG(
		if(signatureScanData.sector % 2 == 0) {
				debugImage.image[y][0][x] = max(0,image.image[y][0][x]-20);
		} else {
				debugImage.image[y][0][x] = min(255,image.image[y][0][x]+20);
		}

		if(change==0 || iy==1) {
			if((signatureScanData.sector/2) % 2 == 0) 
				debugImage.image[y][0][x] = 255;
			else
				debugImage.image[y][0][x] = 0;

			debugImage.image[y][1][x] = 128;
			debugImage.image[y][2][x] = 128;
		}
	)


	if(currentCluster != signatureScanData.lastCluster) {
		// cluster transition detected

		if(!PLAIN_COLORS_ONLY)
			signatureScanData.panoramaHistogram.insertTransition(signatureScanData.sector,signatureScanData.lastCluster,currentCluster);

		/*
		WINDEBUG(
			Vector3<unsigned char> c;
			int i=0;
			int x2;
			int y2;
			int mi=(int)scanningDistanceY/2;
			int id2=verticalIndex[iy];
			for(int id=verticalIndex[iy-1];id<id2;id++) {
				x2 = horizontal[ix].x + verticalDetailled[is][id].x;
				y2 = horizontal[ix].y + verticalDetailled[is][id].y;

				if(i<mi) {
					c = cluster.mixture.clusterColor[signatureScanData.lastCluster];
				} else if (i==mi) {
					if(signatureScanData.sector % 2==0) 
						c = Vector3<unsigned char>(255,0,0);
					else
						c = Vector3<unsigned char>(255,255,0);
				} else {
					c = cluster.mixture.clusterColor[currentCluster];
				}

				debugImage.image[y2][0][x2] = c.x;
				debugImage.image[y2][1][x2] = c.y;
				debugImage.image[y2][2][x2] = c.z;

				i++;
			}
		)
		*/

		signatureScanData.lastClusterBackup = signatureScanData.lastCluster;
		signatureScanData.lastCluster = currentCluster;

		return(true); // request a detailled scan
	} else {
		signatureScanData.panoramaHistogram.insertTransition(
			signatureScanData.sector,currentCluster,currentCluster);
	}

	

	return(false);
}

inline bool SignatureGenerator::scanPixelDetail(
	int &is,int &ix,int &iy,int &id,
	int &x,int &y,
	Vector3<unsigned char> &color,
	double &radScanline,
	SignatureScanData &param) {

	// available data:
	// last cluster, color number   = signatureScanData.lastClusterBackup 
	unsigned char currentCluster = cluster.mixture.lookupTable[color.x>>2][color.y>>2][color.z>>2];

	if(signatureScanData.lastClusterBackup != 255) {
		// phase one: look for last cluster end point	
		if(currentCluster != signatureScanData.lastClusterBackup) {
			signatureScanData.minmax[signatureScanData.sector][signatureScanData.lastClusterBackup][0] 
				= min(signatureScanData.minmax[signatureScanData.sector][signatureScanData.lastClusterBackup][0],id);
			signatureScanData.minmax[signatureScanData.sector][signatureScanData.lastClusterBackup][1] 
				= max(signatureScanData.minmax[signatureScanData.sector][signatureScanData.lastClusterBackup][1],id);
			signatureScanData.lastClusterBackup = 255;
			// paint cluster run end black
			WINDEBUG(
				if(signatureScanData.sector%4 == 0) {
					debugImage.image[y][0][x] = 0;
					debugImage.image[y][1][x] = 0;
					debugImage.image[y][2][x] = 255;
				} else if(signatureScanData.sector%4 == 1) {
					debugImage.image[y][0][x] = 0;
					debugImage.image[y][1][x] = 0;
					debugImage.image[y][2][x] = 128;
				} else if(signatureScanData.sector%4 == 2) {
					debugImage.image[y][0][x] = 0;
					debugImage.image[y][1][x] = 128;
					debugImage.image[y][2][x] = 0;
				} else if(signatureScanData.sector%4 == 3) {
					debugImage.image[y][0][x] = 0;
					debugImage.image[y][1][x] = 255;
					debugImage.image[y][2][x] = 0;
				}
			)
		}
	} else {
		// phase two: look for new cluster start point
		// when done completely, return true
		if(currentCluster == signatureScanData.lastCluster) {
			signatureScanData.minmax[signatureScanData.sector][currentCluster][0] 
				= min(signatureScanData.minmax[signatureScanData.sector][currentCluster][0],id);
			signatureScanData.minmax[signatureScanData.sector][currentCluster][1] 
				= max(signatureScanData.minmax[signatureScanData.sector][currentCluster][1],id);


			// paint cluster run start white
			WINDEBUG(
				if(signatureScanData.sector%4 == 0) {
					debugImage.image[y][0][x] = 255;
					debugImage.image[y][1][x] = 0;
					debugImage.image[y][2][x] = 255;
				} else if(signatureScanData.sector%4 == 1) {
					debugImage.image[y][0][x] = 255;
					debugImage.image[y][1][x] = 0;
					debugImage.image[y][2][x] = 128;
				} else if(signatureScanData.sector%4 == 2) {
					debugImage.image[y][0][x] = 255;
					debugImage.image[y][1][x] = 128;
					debugImage.image[y][2][x] = 0;
				} else if(signatureScanData.sector%4 == 3) {
					debugImage.image[y][0][x] = 255;
					debugImage.image[y][1][x] = 255;
					debugImage.image[y][2][x] = 0;
				}
			)
			return(true);
		} 
	}

	return(false); // continue		
}


void SignatureGenerator::drawTransition(int sector, int offset, int size, unsigned char bottom, unsigned char top) {
	double rad1 = ((double)sector)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad2 = ((double)sector+0.33)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad3 = ((double)sector+0.66)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad4 = ((double)sector+1)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double dist1 = offset+(-0.4)*size;
	double dist2 = offset+(0.0)*size;
	double dist3 = offset+(0.4)*size;

	Vector3<unsigned char> c1 = 	cluster.mixture.clusterColor[bottom],first;
	Vector3<unsigned char> c2 = 	cluster.mixture.clusterColor[top],second;
	Image::convertFromYCbCrToRGB(c1.x,c1.y,c1.z,first.x,first.y,first.z);
	Image::convertFromYCbCrToRGB(c2.x,c2.y,c2.z,second.x,second.y,second.z);


	OCTANGLE_RGB(selfLocatorField,
		sin(rad1)*dist1+robotPose.translation.x,cos(rad1)*dist1+robotPose.translation.y,
		sin(rad2)*dist1+robotPose.translation.x,cos(rad2)*dist1+robotPose.translation.y,
		sin(rad3)*dist1+robotPose.translation.x,cos(rad3)*dist1+robotPose.translation.y,
		sin(rad4)*dist1+robotPose.translation.x,cos(rad4)*dist1+robotPose.translation.y,
		sin(rad4)*dist2+robotPose.translation.x,cos(rad4)*dist2+robotPose.translation.y,
		sin(rad3)*dist2+robotPose.translation.x,cos(rad3)*dist2+robotPose.translation.y,
		sin(rad2)*dist2+robotPose.translation.x,cos(rad2)*dist2+robotPose.translation.y,
		sin(rad1)*dist2+robotPose.translation.x,cos(rad1)*dist2+robotPose.translation.y,
		first.x,
		first.y,
		first.z);
	OCTANGLE_RGB(selfLocatorField,
		sin(rad1)*dist2+robotPose.translation.x,cos(rad1)*dist2+robotPose.translation.y,
		sin(rad2)*dist2+robotPose.translation.x,cos(rad2)*dist2+robotPose.translation.y,
		sin(rad3)*dist2+robotPose.translation.x,cos(rad3)*dist2+robotPose.translation.y,
		sin(rad4)*dist2+robotPose.translation.x,cos(rad4)*dist2+robotPose.translation.y,
		sin(rad4)*dist3+robotPose.translation.x,cos(rad4)*dist3+robotPose.translation.y,
		sin(rad3)*dist3+robotPose.translation.x,cos(rad3)*dist3+robotPose.translation.y,
		sin(rad2)*dist3+robotPose.translation.x,cos(rad2)*dist3+robotPose.translation.y,
		sin(rad1)*dist3+robotPose.translation.x,cos(rad1)*dist3+robotPose.translation.y,
		second.x,
		second.y,
		second.z);
}

void SignatureGenerator::visualizeHistogramOnSoccerField() {
	int sort[MAX_CLUSTERS*MAX_CLUSTERS];
	int sector,i,j,temp;
	int dist,size;
	SectorHistogram &sum = signatureScanData.panoramaHistogram.sum;

	for(sector=0;sector<SECTORS;sector++) {
		SectorHistogram &currentSector = signatureScanData.panoramaHistogram.sector[sector];

		if(currentSector.empty) 
			continue;

	//	OUTPUT(idText,text,"sector "<<sector)  ;

		// linear init
		for(i=0;i<CLUSTERS*CLUSTERS;i++) 
			sort[i] = i;

		// set sort[]=-1 to exclude transitions
		 
		// bubble sort
		for(i=0;i<CLUSTERS*CLUSTERS;i++) {
			for(j=0;j<CLUSTERS*CLUSTERS-1;j++) {
				if(
				  (sort[j] != -1) &&
					(currentSector.freq[ sort[j] / CLUSTERS ]
														 [ sort[j] % CLUSTERS ]
						*
										 (sum.freq[ sort[j+1] / CLUSTERS ]
														 [ sort[j+1] % CLUSTERS ]+10)
				 < currentSector.freq[ sort[j+1] / CLUSTERS ]
														 [ sort[j+1] % CLUSTERS ]
						*
										 (sum.freq[ sort[j] / CLUSTERS ]
														 [ sort[j] % CLUSTERS ]+10)

					)) {
					temp = sort[j];
					sort[j] = sort[j+1];
					sort[j+1] = temp;
				}
			}
		}

		// display N most important transitions
		dist = 1500;
		for(i=0;i<20;i++) {
			if(currentSector.freq[ sort[i] / CLUSTERS ]
															 [ sort[i] % CLUSTERS ]==0) break;
			//OUTPUT(idText,text,(sort[i] / CLUSTERS) << "<->"<<(sort[i] % CLUSTERS)<<": "<<currentSector.freq[ sort[i] / CLUSTERS ]
			//												 [ sort[i] % CLUSTERS ])  ;
			size = currentSector.freq[ sort[i] / CLUSTERS ]
															 [ sort[i] % CLUSTERS ]*10+50;
			size = min(max(size,100),300);
			drawTransition(sector,dist,size,sort[i] / CLUSTERS,sort[i] % CLUSTERS);
			dist -= 300;

			if(dist < 500) break;
		}
	}
}



/*
*
*/
