/**
* @file SignatureGenerator.h
* 
* Definition of class SignatureGenerator
*
* @author Jurgen Sturm
*/


#ifndef __SignatureGenerator_h_
#define __SignatureGenerator_h_

#include "ScanningGrid.h"
#include "ColorClustering.h"
#include "Representations/Perception/PanoramaLibrary.h"
#include "Representations/Perception/LandmarksPercept.h"

class SignatureScanData {
	public:
		unsigned char lastCluster;
		unsigned char lastClusterBackup;
		int pixelCount[MAX_SECTORS];
		int minmax[MAX_SECTORS][MAX_CLUSTERS][2]; 
		int sector;
		int ixStart[MAX_SECTORS];
		PanoramaHistogram &panoramaHistogram;
		void reset();

		SignatureScanData(PanoramaHistogram &panoramaHistogram);
};

class SignatureGenerator:public ScanningGrid
{
	public:
		SignatureGenerator(
			const Image &image,
			const CameraMatrix &cameraMatrix,
			const CameraMatrix &prevCameraMatrix, 
			ColorClustering &cluster, 
			LandmarksPercept &landmarksPercept,
			const RobotPose &robotPose,
			Image &debugImage);

		// scans the image and finally copys panoramaHistogram to landmarksPercept
		void execute();	

		Image &debugImage;

		ColorClustering &cluster;

		SignatureScanData signatureScanData;

		LandmarksPercept &landmarksPercept;

		const RobotPose &robotPose;

		inline bool scanPixel(
			int &is,int &ix,int &iy,
			int &x,int &y,
			Vector3<unsigned char> &color,
			double &radScanline,
			SignatureScanData &param);

		inline bool scanPixelDetail(
			int &is,int &ix,int &iy,int &id,
			int &x,int &y,
			Vector3<unsigned char> &color,
			double &radScanline,
			SignatureScanData &param);

		void visualizeHistogramOnSoccerField();
		void drawTransition(int sector, int offset, int size, unsigned char bottom, unsigned char top);

};


#endif// __SignatureGenerator_h_

/*
*
*
*/
