/**
* @file ColorTableAuto.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/** 
* @file ColorTableAuto.cpp
* Implementation of class ColorTableAuto.
*
* @author <A href=mailto:juengel@informatik.hu-berlin.de>Matthias Jngel</A>
*/

#include "ColorTableAuto.h"
#include "Platform/SystemCall.h"


ColorTableAuto::ColorTableAuto()
{
  setThresholds(green, 100, 150, 100, 150, 100, 150);
}

ColorTableAuto::~ColorTableAuto()
{
}


int ColorTableAuto::getGreenMin(int channel) const
{
  return thresholdMin[green][channel];
}

int ColorTableAuto::getGreenMax(int channel) const
{
  return thresholdMax[green][channel];
}

int ColorTableAuto::getThresholdMin(colorClass color, int channel) const
{
  return thresholdMin[color][channel];
}

int ColorTableAuto::getThresholdMax(colorClass color, int channel) const
{
  return thresholdMax[color][channel];
}

int ColorTableAuto::getAverageGreen(int channel) const
{
  return averageGreen[channel];
}

void ColorTableAuto::setThresholds(colorClass color, int minY, int minU, int minV, int maxY, int maxU, int maxV)
{
  lastTimeWhenThresholdWasSet[color] = SystemCall::getCurrentSystemTime();
  thresholdMin[color][0] = minY; thresholdMin[color][1] = minU; thresholdMin[color][2] = minV;
  thresholdMax[color][0] = maxY; thresholdMax[color][1] = maxU; thresholdMax[color][2] = maxV;
  if(color == green) setGreen();
}

void ColorTableAuto::setGreen()
{

  int greenRange[3];
  greenRange[0] = thresholdMax[green][0] - thresholdMin[green][0];
  greenRange[1] = thresholdMax[green][1] - thresholdMin[green][1];
  greenRange[2] = thresholdMax[green][2] - thresholdMin[green][2];

  averageGreen[0] = (thresholdMax[green][0] + thresholdMin[green][0]) / 2;
  averageGreen[1] = (thresholdMax[green][1] + thresholdMin[green][1]) / 2;
  averageGreen[2] = (thresholdMax[green][2] + thresholdMin[green][2]) / 2;

  greenFieldAutoMin[0] = thresholdMin[green][0] - 15;
  greenFieldAutoMax[0] = thresholdMax[green][0] + 3;
  greenFieldAutoMin[1] = thresholdMin[green][1] - 3;
  greenFieldAutoMax[1] = thresholdMax[green][1] + 3;
  greenFieldAutoMin[2] = thresholdMin[green][2] - 3;
  greenFieldAutoMax[2] = thresholdMax[green][2] + 3;

  skyblueAutoMin[0] = thresholdMin[green][0] - 0;
  skyblueAutoMax[0] = 180;
  skyblueAutoMax[1] = averageGreen[1] + 20;
  skyblueAutoMin[2] = averageGreen[2] + 20;

  yellowAutoMin[0] = thresholdMin[green][0] - 30;
  yellowAutoMax[0] = thresholdMax[green][0] + 80;
  yellowAutoMin[1] = averageGreen[1];
  yellowAutoMax[2] = thresholdMin[green][2] - 10;

  pinkAutoMin[0] = averageGreen[0];
  pinkAutoMin[1] = averageGreen[1] + 20;
  pinkAutoMin[2] = averageGreen[2];

  whiteAutoMin[0] = averageGreen[0];
  whiteAutoMin[1] = averageGreen[1] - 2 * greenRange[1];
  whiteAutoMax[1] = averageGreen[1] + 2 * greenRange[1];
  whiteAutoMin[2] = averageGreen[2] - 2 * greenRange[2];
  whiteAutoMax[2] = averageGreen[2] + 2 * greenRange[2];

  whiteFixedMin[0] = 180;
  whiteFixedMin[1] = 127 - 20;
  whiteFixedMax[1] = 127 + 50;
  whiteFixedMin[2] = 127 - 70;
  whiteFixedMax[2] = 127 + 30;
}

colorClass ColorTableAuto::getColorClass
(
 const unsigned char y, 
 const unsigned char u, 
 const unsigned char v
 ) const
{
  if(
    y > thresholdMin[orange][0] - 10 && 
    y < thresholdMax[orange][0] + 10 &&   
    u > thresholdMin[orange][1] - 10 && 
    u < thresholdMax[orange][1] + 10 && 
    v > thresholdMin[orange][2] - 10 &&
    v < thresholdMax[orange][2] + 10 &&
    SystemCall::getTimeSince(lastTimeWhenThresholdWasSet[orange]) < 2500 )
  {
    return orange;
  }
  else if(
    y > thresholdMin[green][0] && 
    y < thresholdMax[green][0] &&   
    u > thresholdMin[green][1] && 
    u < thresholdMax[green][1] && 
    v > thresholdMin[green][2] &&
    v < thresholdMax[green][2] &&
    SystemCall::getTimeSince(lastTimeWhenThresholdWasSet[green]) < 2500000 )
  {
    return green;
  }
  else if(
    y > thresholdMin[white][0] && 
    y < thresholdMax[white][0] &&   
    u > thresholdMin[white][1] && 
    u < thresholdMax[white][1] && 
    v > thresholdMin[white][2] &&
    v < thresholdMax[white][2] &&
    SystemCall::getTimeSince(lastTimeWhenThresholdWasSet[white]) < 5000 )
  {
    return white;
  }  
  else if(
    y > thresholdMin[skyblue][0] && 
    y < thresholdMax[skyblue][0] &&   
    u > thresholdMin[skyblue][1] && 
    u < thresholdMax[skyblue][1] && 
    v > thresholdMin[skyblue][2] &&
    v < thresholdMax[skyblue][2] &&
    SystemCall::getTimeSince(lastTimeWhenThresholdWasSet[skyblue]) < 5000 )
  {
    return skyblue;
  }
  else if(
    y > thresholdMin[yellow][0] && 
    y < thresholdMax[yellow][0] &&   
    u > thresholdMin[yellow][1] && 
    u < thresholdMax[yellow][1] && 
    v > thresholdMin[yellow][2] &&
    v < thresholdMax[yellow][2] &&
    SystemCall::getTimeSince(lastTimeWhenThresholdWasSet[yellow]) < 5000 )
  {
    return yellow;
  }
  else return noColor;
}



colorClass ColorTableAuto::getColorClassAuto
(
 const unsigned char y, 
 const unsigned char u, 
 const unsigned char v
 ) const
{
  if(
    // green of the field
    y > greenFieldAutoMin[0] && y < greenFieldAutoMax[0] && 
    u > greenFieldAutoMin[1] && u < greenFieldAutoMax[1] &&
    v > greenFieldAutoMin[2] && v < greenFieldAutoMax[2]
    )
  {
    return green;
  }
  else if(y > skyblueAutoMin[0] && y < skyblueAutoMax[0] &&   u < skyblueAutoMax[1] && v > skyblueAutoMin[2])
  {
    return skyblue;
  }
  else if(y > yellowAutoMin[0] && y < yellowAutoMax[0] && u > yellowAutoMin[1] && v < yellowAutoMax[2])
  {
    return yellow;
  }
  else if(
    /*
    y > whiteMin[0] &&
    u > whiteMin[1] && u < whiteMax[1] &&
    v > whiteMin[2] && v < whiteMax[2] 
    ||
    */
    y > whiteFixedMin[0] &&
    u > whiteFixedMin[1] && u < whiteFixedMax[1] &&
    v > whiteFixedMin[2] && v < whiteFixedMax[2] 
    )
  {
    return white;
  }  
/*  else if(y > pinkAutoMin[0] && u > pinkAutoMin[1] && v > pinkAutoMin[2])
  {
    return pink;
  }
  */
  else return noColor;
}

void ColorTableAuto::generateColorClassImageAuto
(
 const Image& image, 
 ColorClassImage& colorClassImage) const
{
  colorClass color;
  for (register int y=0; y < image.cameraInfo.resolutionHeight; y++) 
  {
    for (register int x=0; x < image.cameraInfo.resolutionWidth; x++) 
    {
      color = getColorClassAuto(image.image[y][0][x],image.image[y][1][x],image.image[y][2][x]);
      colorClassImage.image[y][x] = color;
    }
  }
}

void ColorTableAuto::generateColorClassImage
(
 const Image& image, 
 ColorClassImage& colorClassImage) const
{
  colorClass color;
  for (register int y=0; y < image.cameraInfo.resolutionHeight; y++) 
  {
    for (register int x=0; x < image.cameraInfo.resolutionWidth; x++) 
    {
      color = getColorClass(image.image[y][0][x],image.image[y][1][x],image.image[y][2][x]);
      colorClassImage.image[y][x] = color;
    }
  }
}


In& operator>>(In& stream, ColorTableAuto& colorTableAuto)
{
  int thresholdMin[numOfColors][3];
  int thresholdMax[numOfColors][3];
  for(int color = 0; color < numOfColors; color++)
  {
    for(int channel = 0; channel < 3; channel++)
    {
      stream >> thresholdMin[color][channel];
      stream >> thresholdMax[color][channel];
    }
    colorTableAuto.setThresholds((colorClass)color, 
      thresholdMin[color][0],
      thresholdMin[color][1],
      thresholdMin[color][2],
      thresholdMax[color][0],
      thresholdMax[color][1],
      thresholdMax[color][2]);
  }


  return stream;
}

Out& operator<<(Out& stream, const ColorTableAuto& colorTableAuto)
{
  for(int color = 0; color < numOfColors; color++)
  {
    for(int channel = 0; channel < 3; channel++)
    {
      stream << colorTableAuto.getThresholdMin((colorClass)color, channel);
      stream << colorTableAuto.getThresholdMax((colorClass)color, channel);
    }
  }

  return stream;
}

/*
* Change log :
* 
* $Log: ColorTableAuto.cpp,v $
* Revision 1.2  2004/02/27 11:31:13  wcaarls
* Import of Utrecht changes as of 26-02-2004.
*
* Revision 1.1.1.1  2004/01/15 12:30:27  bottens
*
*
* Revision 1.4  2003/08/29 13:12:50  juengel
* changed time after which orange is forgotten
*
* Revision 1.3  2003/08/18 12:06:01  juengel
* Added streaming operators.
*
* Revision 1.2  2003/07/29 12:44:26  juengel
* Optimization
*
* Revision 1.1  2003/07/21 13:40:24  juengel
* Moved ColorTableAuto and GoalRecognizer to ImageProcessorTools.
*
* Revision 1.2  2003/07/19 19:31:04  juengel
* Improved AutoCalibration
*
* Revision 1.1.1.1  2003/07/02 09:40:23  cvsadm
* created new repository for the competitions in Padova from the 
* tamara CVS (Tuesday 2:00 pm)
*
* removed unused solutions
*
* Revision 1.6  2003/05/05 12:27:27  juengel
* Added method setGreen.
*
* Revision 1.5  2003/04/23 16:17:55  juengel
* ColorTableAuto::getColorClassForFlagsAndGoals returns a color class now.
*
* Revision 1.4  2003/04/03 18:50:08  juengel
* Improved AutoCalibration.
*
* Revision 1.3  2003/04/02 13:21:11  juengel
* Improved flag colors.
*
* Revision 1.2  2003/04/01 19:28:45  juengel
* Improved getColorClassFlagsAndGoals.
*
* Revision 1.1  2003/04/01 13:59:34  juengel
* Added ColorTableAuto
*
*
*/
