
/* 
 * wrapper classes for the anigauss-library of Jan-Mark Geusebroek
 */

#ifndef __AnisotropicFilter_h_
#define __AnisotropicFilter_h_

#include "Representations/Perception/CameraInfo.h"
#include "Representations/Perception/Image.h"

class ChannelImage {
public:
	int width,height;
	unsigned char image[cameraResolutionHeight_ERS7][cameraResolutionWidth_ERS7];
};


class AnisotropicImage {
public:
	double image[3][cameraResolutionHeight_ERS7][cameraResolutionWidth_ERS7];
	int width,height;

	// create an empty anisotropic image wrap
	AnisotropicImage();	

	// create an image wrap from existing aibo image
	AnisotropicImage(const Image &source);

// create an image from an existing wrapper by applying a filter
	AnisotropicImage(
		AnisotropicImage &source, 
		double sigmav, double sigmau, 
		double phi, int orderv, int orderu);

	// basic filtering
	void AnisotropicGauss(double sigmav, double sigmau, double phi, int orderv, int orderu);

	// write image back to an aibo image
	void copyToImage(Image &destination);

	// write image back to an aibo image
	enum CopyMode {direct,add128,add128div2,blackwhite,autoscale};
	void copyToImage(Image &destination,CopyMode copyMode);
	void copyToMagnitudeMap(ChannelImage &destination);
	void copyFromImage(const Image &source);
};

#endif
