#include "GlobalAlignment.h"
#include "Modules/ImageProcessor/PanoramaImageProcessor/anigauss.h"

#ifndef min
#define min(a,b) (a<=b?a:b)
#endif

#ifndef max
#define max(a,b) (a>=b?a:b)
#endif

GlobalAlignment::GlobalAlignment() {
	reset();	
}

void GlobalAlignment::reset() {
	historyLength = 0;
}

void GlobalAlignment::execute() {
	match();
	estimateSize();
	computePosition();
}

void GlobalAlignment::aquire(LocalAlignment &newAlignment) {
	memcpy(&history[1],&history[0],sizeof(LocalAlignment)*(HISTORY_BUFFER-1));
	history[0] = newAlignment;
	// but reverse path
	for(int i=0;i<newAlignment.length;i++)
		history[0].path[i] = newAlignment.path[newAlignment.length-1-i];
	historyLength=min(historyLength+1,HISTORY_BUFFER);
}

void GlobalAlignment::match() {
	if(historyLength==0) return;

	seenOffset[0] = 0;
	seenOffsetValid[0] = true;
	int i,j,alignment;
	for(alignment=1;alignment<historyLength;alignment++) {
		double bestOffset = 0;
		double bestValue = 0;
		for(int offsetIndex=-SEEK_STEPS/2;offsetIndex<SEEK_STEPS/2;offsetIndex++) {
			double offset = OFFSET_RANGE*offsetIndex/SEEK_STEPS;
			double value = 0;
			for(int reference=0;reference<alignment;reference++) {
				if(!seenOffsetValid[reference]) continue;
				//todo: algorithm is not cyclic!!
				i=0,j=0;
				while (true) {
					while(
							i<history[alignment].length
						&&	history[alignment].path[i].x < history[reference].path[j].x) 
					{
						i++;
					}
					if(i==history[alignment].length) break;

					while(
						    j<history[reference].length 
						&&	history[alignment].path[i].x > history[reference].path[j].x) 
					{
						j++;
					}
					if(j==history[reference].length) break;

					if(history[alignment].path[i].x == history[reference].path[j].x) {
						double diff =	  (history[alignment].path[i].y + offset)
										- (history[reference].path[j].y + seenOffset[reference]);
						if(fabs(diff) > OFFSET_RANGE/2) {
							value = 0;
							break;
						}
						historyOldestAlignmentUsed = reference;
						value += 
							sqr( diff )
							* 
							min(
								history[alignment].length/2-abs(i-history[alignment].length/2),
								history[reference].length/2-abs(j-history[reference].length/2)
							)
							*
							5/(alignment+1)
							*
							history[reference].signalToNoiseRatio
							;
						i++;
						j++;
					} 
				}
			}
			if(value != 0 && (bestValue==0 || value < bestValue)) {
				bestOffset = offset;
				bestValue = value;
			}
		}
		seenOffset[alignment] = bestOffset;
		seenOffsetValid[alignment] = (bestValue >0);
	}

	double sumOffset=0;
	for(alignment=0;alignment<historyLength;alignment++) 
		sumOffset += seenOffset[alignment];
	for(alignment=0;alignment<historyLength;alignment++) 
		seenOffset[alignment] -= sumOffset/historyLength;


	double worldSeenSum[MAX_SECTORS],worldSeenWeight[MAX_SECTORS];
	int sector;
	for(sector=0;sector<SECTORS;sector++) {
		worldSeenSum[sector] = worldSeenWeight[sector] = 0;
	}

	for(alignment=0;alignment<historyLength;alignment++) {
		if(!seenOffsetValid[alignment]) continue;
		for(i=0;i<history[alignment].length;i++) {
			double seen = history[alignment].path[i].y + seenOffset[alignment];
			double weight = history[alignment].length/2-abs(i-history[alignment].length/2);

			worldSeenSum[history[alignment].path[i].x] += seen*weight;
			worldSeenWeight[history[alignment].path[i].x] += weight;
		}
	}

	for(sector=0;sector<SECTORS;sector++) {
		if(worldSeenWeight[sector]>0) {
			worldSeen[sector] = worldSeenSum[sector] / worldSeenWeight[sector];
		} else
			worldSeen[sector] = -1;
	}


//	if(historyLength>5) {
//		OUTPUT(idText,text,seenOffset[0] << "  " <<seenOffset[1] << "  " <<seenOffset[2] << "  " <<seenOffset[3] << "  " <<seenOffset[4] << "  " <<seenOffset[5] << "  " );
//	}
}

void GlobalAlignment::visualizeMatch(Image &debugImage) {

	int ix,iy;
	for(iy=0;iy<cameraResolutionHeight_ERS7;iy++) 
	for(ix=0;ix<cameraResolutionWidth_ERS7;ix++) {
		debugImage.image[iy][0][ix] = 0;
		debugImage.image[iy][1][ix] = 0;
		debugImage.image[iy][2][ix] = 0;
	}
	iy = cameraResolutionHeight_ERS7/2;
	for(ix=0;ix<cameraResolutionWidth_ERS7;ix++) {
		debugImage.image[iy][0][ix] = 128;
		debugImage.image[iy][1][ix] = 128;
		debugImage.image[iy][2][ix] = 128;
	}
/*
	double offset = 0;
	for(int alignment=historyLength-1;alignment>=0;alignment--) {
		offset = seenOffset[alignment];
		for(int i=0;i<history[alignment].length;i++) {
			//todo: SECTORS should be panoramaSectors!
			ix = history[alignment].path[i].x * cameraResolutionWidth_ERS7 / SECTORS;
			iy = (int)((history[alignment].path[i].y+offset-history[alignment].path[i].x+SECTORS/2) * cameraResolutionHeight_ERS7 / SECTORS);

			if(alignment==0) {
				debugImage.image[iy][0][ix] = 255;
				debugImage.image[iy][1][ix] = 255;
				debugImage.image[iy][2][ix] = 255;
			} else {
				debugImage.image[iy][0][ix] = max(0,255-alignment*15);
				debugImage.image[iy][1][ix] = 0;
				debugImage.image[iy][2][ix] = max(0,min(alignment*15,255)-alignment*8);
			}
		}

	}
*/
	int world;
	for(world=0;world<SECTORS;world++) {
		if(worldSeen[world]<0) continue;
		for(ix = world*cameraResolutionWidth_ERS7/SECTORS;ix<(world+1)*cameraResolutionWidth_ERS7/SECTORS;ix++) {
			iy = (int)((worldSeen[world]-world+SECTORS/2)*cameraResolutionHeight_ERS7/SECTORS);
			debugImage.image[iy][0][ix] = 255;
			debugImage.image[iy][1][ix] = 255;
			debugImage.image[iy][2][ix] = 255;
			iy = (int)((worldSeenSmoothed[world]+SECTORS/2)*cameraResolutionHeight_ERS7/SECTORS);
			debugImage.image[iy][0][ix] = 0;
			debugImage.image[iy][1][ix] = 255;
			debugImage.image[iy][2][ix] = 0;
		}
	}

	for(world=0;world<SECTORS;world++) {
		if(worldSeen[world]<0) continue;
		iy = (int)((-seenSize[world]*5+SECTORS/2)*cameraResolutionHeight_ERS7/SECTORS);
		if(ix<0 || ix>cameraResolutionWidth_ERS7 || iy<0 || iy>cameraResolutionHeight_ERS7) continue;
		for(ix = world*cameraResolutionWidth_ERS7/SECTORS;ix<(world+1)*cameraResolutionWidth_ERS7/SECTORS;ix++) {
			debugImage.image[iy][0][ix] = 255;
			debugImage.image[iy][1][ix] = 0;
			debugImage.image[iy][2][ix] = 255;
		}
	}
}

void convolve_gaussian(double sigma,int kernel_size,int order,double * in,double * out) {
	double kernel[MAX_SECTORS];
	int temp,world;
	switch(order) {
		case 0:
			for(temp=-kernel_size;temp<=kernel_size;temp++) 
				kernel[temp+kernel_size] = exp(- 0.5 * sqr(temp) / sqr(sigma));
			break;
		case 1:
			for(temp=-kernel_size;temp<=kernel_size;temp++) 
				kernel[temp+kernel_size] = -temp* exp(- 0.5 * sqr(temp) / sqr(sigma));
			break;
	}

	double kernelSum=0;
	for(temp=-kernel_size;temp<=kernel_size;temp++) 
		kernelSum += fabs(kernel[temp+kernel_size]);

	for(world=10;world<SECTORS*3-10;world++) {
		double sumLeft=0,sumRight=0;
		double weightLeft=0,weightRight=0;
		for(temp=-kernel_size;temp<=0;temp++) {
			if(in[world+temp]>-SECTORS) {
				sumLeft += in[world+temp]*kernel[temp+kernel_size];
				weightLeft += fabs(kernel[temp+kernel_size]);
			}
		}
		for(temp=1;temp<=kernel_size;temp++) {
			if(in[world+temp]>-SECTORS) {
				sumRight += in[world+temp]*kernel[temp+kernel_size];
				weightRight += fabs(kernel[temp+kernel_size]);
			}
		}
		if((weightLeft <= kernelSum*0.25 || weightRight <= kernelSum*0.25)) {
			out[world] = -SECTORS;
			continue;
		}
		
		out[world] = 0.5*sumLeft/weightLeft + 0.5*sumRight/weightRight;
	}
}

void linearRegress(double *mapping,double &slope,double &b) {
	// linear regression, following http://people.hofstra.edu/faculty/Stefan_Waner/RealWorld/tutorialsf0/frames1_5.html
	double sumXY=0,sumX=0,sumY=0,sumSqrX=0,sumXSqr=0;

	double x,y;
	int points = 0,i;

	int distanceToVoid[MAX_SECTORS*3];
	int centerPoint = SECTORS/2+SECTORS;
	for(i=0;i<SECTORS*3;i++) {
		distanceToVoid[i] = (mapping[i]==-SECTORS) ? 0:SECTORS;
	}
	for(i=1;i<SECTORS*3-1;i++) {
		distanceToVoid[i] = min(distanceToVoid[i-1]+1,distanceToVoid[i]);
	}
	for(i=SECTORS*3-1;i>1;i--) {
		distanceToVoid[i] = min(distanceToVoid[i+1]+1,distanceToVoid[i]);
	}
	for(i=SECTORS;i<SECTORS*2;i++) {
		if(distanceToVoid[i]>distanceToVoid[centerPoint])
			centerPoint = i;
	}
	
	b= 0;

	for(i=SECTORS;i<SECTORS*2;i++) {
		x = i-SECTORS-centerPoint+SECTORS;
		y = mapping[i];

		if(y==-SECTORS) continue;

		points++;

		sumXY += x*y;
		sumX += x;
		sumY += y;
		sumSqrX += sqr(x);
	}
	sumXSqr = sqr(sumX);

	if(points<2) {
		slope = 0;
		b = sumY;
		return;
	}

	slope = (points * sumXY - sumX*sumY) / ((double)points*sumSqrX - sumXSqr);
	b = (sumY - slope*sumX) / points;
}

void GlobalAlignment::estimateSize() {
//void anigauss(SRCTYPE *input, DSTTYPE *output, int sizex, int sizey,
//	double sigmav, double sigmau, double phi, int orderv, int orderu)

	double in[MAX_SECTORS*3],out[MAX_SECTORS*3],out2[MAX_SECTORS*3];
	int world;
	for(world=0;world<SECTORS;world++) 
		in[world] = in[world+SECTORS] = in[world+SECTORS*2] = (worldSeen[world]==-1)?-SECTORS:worldSeen[world] - world;

	// compute average and fill holes
	convolve_gaussian(5,20,0,in,out);
	linearRegress(out,matchSlope,matchAverage);

	for(world=0;world<SECTORS;world++) {
		worldSeenSmoothed[world] = out[world+SECTORS];
		in[world] = in[world+SECTORS] = in[world+SECTORS*2] = out[world];
	}

	convolve_gaussian(5,20,1,in,out2);
	linearRegress(out2,scaleSlope,scaleAverage);

	for(world=0;world<SECTORS;world++) {
		seenSize[world] = out2[world+SECTORS];
	}

	//OUTPUT(idText,text,"matchSlope="<<matchSlope<<" matchAverage="<<matchAverage<<" scaleSlope="<<scaleSlope<<" scaleAverage="<<scaleAverage);
}

void GlobalAlignment::computePosition() {
	double worldDistance = matchSlope * 3000;
	double worldShift = scaleSlope * 3000;

	estimatedPose.rotation = matchAverage / SECTORS * 2*PI;
	estimatedPose.translation.x = cos(estimatedPose.rotation) * worldDistance;
	estimatedPose.translation.y = sin(estimatedPose.rotation) * worldDistance;
	estimatedPose.translation.x += -sin(estimatedPose.rotation) * worldShift;
	estimatedPose.translation.y += cos(estimatedPose.rotation) * worldShift;
	estimatedPose.setValidity(historyOldestAlignmentUsed/max(1,historyLength));

}
