/**
*
*/

#ifndef __HistogramDistributionLibrary_h_
#define __HistogramDistributionLibrary_h_

#include "Representations/Perception/PanoramaLibrary.h"
#include "Representations/Cognition/RobotPose.h"

class SectorHistogramDistribution {
public:
	short freq[MAX_CLUSTERS][MAX_CLUSTERS][BINS];
	short sum[MAX_CLUSTERS][MAX_CLUSTERS];
	bool empty;
	int zeroBinMask[MAX_CLUSTERS];//bit 0..MAX_CLUSTERS is ON => >95% of mass is located in bin 0
	bool invalidBitmask;
	double normalization;

	SectorHistogramDistribution();
	void reset();
	void updateBitmask();
	void insertTransition(unsigned char bottom,unsigned char top,int bin);
};

class PanoramaHistogramDistribution {
public:
	SectorHistogramDistribution sector[MAX_SECTORS];
	SectorHistogramDistribution sum;
	const PanoramaHistogram &panoramaHistogram;

	int zeroBinMaskSeen[MAX_SECTORS][MAX_CLUSTERS];

	RobotPose &robotPose;

	PanoramaHistogramDistribution(const PanoramaHistogram &panoramaHistogram,RobotPose &robotPose);
	void reset();
	inline void insertTransition(int sector, unsigned char bottom,unsigned char top,int count);
	void appendSectorToWorldModel(int worldSector,int seenSector);
	void updateBitmask();
	double computeLikelihood(int worldSector,int seenSector);

	void saveWorldModel(const char* filename);
	bool loadWorldModel(const char* filename);

};

In& operator>>(In& stream,PanoramaHistogramDistribution& panoramaHistogramDistribution);
Out& operator<<(Out& stream, const PanoramaHistogramDistribution& panoramaHistogramDistribution);

#endif// __HistogramDistributionLibrary_h_

/*
*/
