#include "LocalAlignment.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Tools/Debugging/Debugging.h"
#include "Tools/Debugging/Stopwatch.h"
#include "Representations/Perception/PanoramaLibrary.h"

#ifndef min
#define min(a,b) (a<=b?a:b)
#endif

#ifndef max
#define max(a,b) (a>=b?a:b)
#endif


//red dots
//#define SHOW_MATCHBUFFER

// assumed white line
//#define SHOW_MAPPING
// estimated white line
//#define SHOW_ESTIMATED_MAPPING

#define SHOW_SNR

// correlation
#define SHOW_MATCHING

LocalAlignment::LocalAlignment() {
	reset();
}

void LocalAlignment::reset() {
	signalToNoiseRatio = 0;
	rotationHead = 0;
	rotationOffset = 0;
  length = 0;
	for(int i=0;i<SECTORS;i++) 
		likelihood[i]=0;
}

PanoramaModel::PanoramaModel(
			Pose2D panoramaCenter, int panoramaRadius,
			const LandmarksPercept &landmarksPercept,
			RobotPose &robotPose,
			SLAMData &slamData) 
:
	panoramaHistogramDistribution((const PanoramaHistogramDistribution&) PanoramaHistogramDistribution(
		landmarksPercept.panoramaHistogram,
		robotPose)),
	panoramaHistogram(landmarksPercept.panoramaHistogram),
	robotPose(robotPose),
	slamData(slamData)
{
	this->panoramaCenter = panoramaCenter;
	this->panoramaRadius = panoramaRadius;
	reset();
	//OUTPUT(idText,text,"sizeof(PanoramaModel)="<<sizeof(PanoramaModel));
	//OUTPUT(idText,text,"sizeof(PanoramaHistogramDistribution)="<<sizeof(PanoramaHistogramDistribution));
}

int PanoramaModel::convertLocalToGlobalSector(double relativeSector) {
	 double relativeRad = -(((double)relativeSector)/SECTORS*2*PI - robotPose.rotation- PI);

	 double absoluteRad = relativeRad;
	 double absoluteSector = 	((-absoluteRad/(2*PI) + 0.5)*SECTORS);

	 return(((int)absoluteSector+SECTORS) % SECTORS);
}



void PanoramaModel::reset() {
	panoramaHistogramDistribution.reset();
	matchBufferPos = 0;
	int p;
	for(p=0;p<MATCH_BUFFER;p++) {
		matchBuffer[p].x = -1;
		matchBuffer[p].y = -1;
	}
	alignmentBufferPos = 0;
	for(p=0;p<ALIGNMENT_BUFFER;p++) {
		alignmentBuffer[p].x = -1;
		alignmentBuffer[p].y = -1;
	}

	for(int i=0;i<SECTORS;i++)
		for(int j=0;j<SECTORS;j++) {
			correlation[i][j]=0;
			age[i][j]=0;
		}

	numLikelihoods = 0;
	sumLikelihoods = 0;
}

void PanoramaModel::learn() {
	int wFirst,wLast;
	int sectorDiff=int(-robotPose.rotation/(2*PI)*SECTORS+SECTORS)%SECTORS;;
	for(int s=0;s<SECTORS;s++) {
		wFirst = convertLocalToGlobalSector(s);
//		wLast = convertLocalToGlobalSector(s+1);
		//if(wFirst==-1 || wLast == -1) continue;
		wFirst = (s + sectorDiff)%SECTORS;
		wLast = (s + sectorDiff+1)%SECTORS;
		for(int w=wFirst;w!=wLast;w=(w+1)%SECTORS) {
			panoramaHistogramDistribution.appendSectorToWorldModel(w,s);
			//OUTPUT(idText,text,"merging seen "<<s<<" with world "<<w);
		}
	}
	if(OPTIMIZATIONS) 
	for(int i=0;i<SECTORS;i++) {
		if(panoramaHistogramDistribution.sector[i].invalidBitmask) {
			panoramaHistogramDistribution.sector[i].updateBitmask();
//			OUTPUT(idText,text,"bitmask update sector "<<i);
		}
	}
}

void PanoramaModel::match() {
//	STOPWATCH("PanoramaModel::computeLikelihoods() ",computeLikelihoods();)
//	STOPWATCH("PanoramaModel::findDiagonal() ",findDiagonal();)
//	STOPWATCH("PanoramaModel::computeShortestPath() ",computeShortestPath();)
/*
	STOPWATCH("PanoramaModel::computeLikelihoods() ",computeLikelihoods();)
	STOPWATCH("PanoramaModel::computeShortestPath() ",computeShortestPath();)
	STOPWATCH("PanoramaModel::estimatePhase() ",estimatePhase();)
	STOPWATCH("PanoramaModel::computeLinearRegression() ",computeLinearRegression();)
	STOPWATCH("PanoramaModel::computePosition() ",computePosition();)
*/


	computeLikelihoods();
//	computeShortestPath();
	findDiagonal();
//	estimatePhase();
//	computeLinearRegression();
	//computeIntersections();
//	computePosition();
	
}

void PanoramaModel::computeShortestPath() {
	currentAlignment.reset();
	// todo: firstSeen,lastSeen problems when over the edge/cyclic
	int world,seen;
	double sumSeen[MAX_SECTORS],sumWorld[MAX_SECTORS],minValue=0;
	double sumdist[MAX_SECTORS][MAX_SECTORS];
	int firstSeen=SECTORS,lastSeen=0;

	for( world=0;world<SECTORS;world++) {
		for( seen=0;seen<=SECTORS;seen++) {
			route[world][seen] = 0;
		}
	}

	// determine visible range of seen sectors
	for(seen=0;seen<SECTORS;seen++) {
		if(panoramaHistogram.sector[seen].empty) continue;
		firstSeen = min(firstSeen,seen);
		lastSeen = max(lastSeen,seen);
	}
	currentAlignment.rotationHead = (lastSeen+firstSeen-80)*PI/SECTORS;

	// compute sum over world sectors (normalize columns)
	for( world=0;world<SECTORS;world++) {
		sumSeen[world] = 1;
		for( seen=firstSeen;seen<=lastSeen;seen++) {
			sumSeen[world] += fabs(correlation[world][seen]);
			minValue = min(minValue,correlation[world][seen]);
		}
	}

	// compute sum over seen sectors (normalize rows)
//	for( seen=firstSeen;seen<=lastSeen;seen++) {
	for( seen=0;seen<SECTORS;seen++) {
		sumWorld[seen] = 1;
		for( world=0;world<SECTORS;world++) {
			sumWorld[seen] += fabs(correlation[world][seen]);
		}
	}

	minValue = 0;
	for( world=0;world<SECTORS;world++) {
		for( seen=firstSeen;seen<=lastSeen;seen++) {
			minValue = min(minValue,correlation[world][seen]);
		}
	}

	// normalize and init distance array
	for( world=0;world<SECTORS;world++) {
//		for( seen=firstSeen;seen<lastSeen;seen++) {
		for( seen=0;seen<SECTORS;seen++) {
			if(correlation[world][seen]==0) 
				distance[world][seen] = -1;
			else
				distance[world][seen] = fabs(correlation[world][seen]/(sumWorld[seen]*sumSeen[world]));
		}
	}

	double COST_LEFT = 1;
	double COST_UP = 1;
	double COST_DIAG = 1;

	for( world=0;world<SECTORS;world++) {
		sumdist[world][firstSeen] = distance[world][firstSeen]*COST_DIAG;
		sumdist[world][lastSeen] = 1;
	}

	// propagate
	double distLeft=-1,distTop=-1,distDiag=-1;
	for( seen=firstSeen+1;seen<=lastSeen;seen++) {
		for( world=0;world<SECTORS;world++) {
			if(distance[world][seen] == -1) {
				route[world][seen] = -11;
				continue;
			}

			distLeft = (distance[(world-2+SECTORS) % SECTORS][(seen-1+SECTORS) % SECTORS]<0)?1:
					sumdist[(world-2+SECTORS) % SECTORS][(seen-1+SECTORS) % SECTORS]
					 + distance[world][seen]*COST_LEFT;

			distTop  = (distance[world][(seen-1+SECTORS) % SECTORS]<0)?1:
					sumdist[world][(seen-1+SECTORS) % SECTORS]
					 + distance[world][seen]*COST_UP;

			distDiag = (distance[(world-1+SECTORS) % SECTORS][(seen-1+SECTORS) % SECTORS]<0)?1:
					sumdist[(world-1+SECTORS) % SECTORS][(seen-1+SECTORS) % SECTORS]
					 + distance[world][seen]*COST_DIAG;

			if(distDiag != 1 && distDiag<=distLeft && distDiag<=distTop) {
				route[world][seen] = 0;//diag
				sumdist[world][seen] = distDiag;
			} else
			if(distTop !=1 && distTop<=distLeft && distTop<=distDiag) {
				route[world][seen] = 1;//top
				sumdist[world][seen] = distTop;
			} else 
			if(distLeft !=1 && distLeft<=distTop && distLeft<=distDiag) {
				route[world][seen] = -1;//left
				sumdist[world][seen] = distLeft;
			} else {
				route[world][seen] = -10;
				sumdist[world][seen] = 0;
			}
			
		}
	}

	// find best (biggest) value 
	int bestWorld=0;
	for( world=0;world<SECTORS;world++) {
		if(sumdist[world][lastSeen]<sumdist[bestWorld][lastSeen]) 
			bestWorld = world;
	}

	// find best noise value
	int bestNoise=(bestWorld+SECTORS/2) % SECTORS;//seed somewhere else
	for( world=0;world<SECTORS;world++) {
		if(sumdist[world][lastSeen]<sumdist[bestNoise][lastSeen] 
			&& ( abs(world-bestWorld) % SECTORS)>12) 
			bestNoise = world;
	}

	world=bestNoise;
	seen=lastSeen;

	//mark noise path
	int noiseLength=0;
	Vector2<int> noisePath[100];
	do {
		noisePath[noiseLength].x=world;
		noisePath[noiseLength].y=seen;
		noiseLength++;

		if(route[world][seen]==0) { world--,seen--; }
		else
			if(route[world][seen]==1) { seen--; }
		else
			if(route[world][seen]==-1) { world--;world--;seen--; }

		world = (world+SECTORS) % SECTORS;
		seen = (seen +SECTORS) % SECTORS;

	} while (seen>firstSeen && noiseLength<100);

	//http://en.wikipedia.org/wiki/Signal-to-noise_ratio
	if(sumdist[bestNoise][lastSeen]!=0)
		currentAlignment.signalToNoiseRatio = 
			3 * (1-(sumdist[bestWorld][lastSeen] / sumdist[bestNoise][lastSeen]));

	// now find path, reverse direction
	seen = lastSeen;
	world = bestWorld;

//	int pathLength=0;

//	Vector2<int> path[SECTORS+SECTORS];


	do {
		currentAlignment.path[currentAlignment.length].x = world;
		currentAlignment.path[currentAlignment.length].y = seen;
		currentAlignment.length++;

		if(route[world][seen]==0) { world--,seen--; }
		else
			if(route[world][seen]==1) { seen--; }
		else
			if(route[world][seen]==-1) { world--;world--;seen--; }
		else
			currentAlignment.length=0;//error

		world = (world+SECTORS) % SECTORS;
		seen = (seen +SECTORS) % SECTORS;

	} while (seen>firstSeen && currentAlignment.length<SECTORS+SECTORS && currentAlignment.length!=0);

	if(seen==firstSeen) {
		currentAlignment.path[currentAlignment.length].x = world;
		currentAlignment.path[currentAlignment.length].y = seen;
		currentAlignment.length++;

		// mark path
		double avgdist[MAX_SECTORS+MAX_SECTORS];
		double avgdist2[MAX_SECTORS+MAX_SECTORS];
		int i;
		for(i=0;i<currentAlignment.length;i++) {
			route[currentAlignment.path[i].x][currentAlignment.path[i].y-1] = -3;
			avgdist[i] = distance[currentAlignment.path[i].x][currentAlignment.path[i].y];
			if(i==0 || currentAlignment.path[i].x != currentAlignment.path[i-1].x) {
				alignmentBuffer[alignmentBufferPos++] = currentAlignment.path[i];
				alignmentBufferPos = alignmentBufferPos % ALIGNMENT_BUFFER;
			}
		}
		//mark noise path
		for(i=0;i<noiseLength;i++) {
			route[noisePath[i].x][noisePath[i].y-1] = -6;
		}

		// average distances along path
		for(i=0;i<currentAlignment.length;i++) {
			avgdist2[i] = (i>0)?avgdist[i-1]:0
						+ avgdist[i]
						+ (i<currentAlignment.length)?avgdist[i+1]:0;
		}
		
		int bestPos = 1;
		for(i=1;i<currentAlignment.length-1;i++) {
			if(avgdist2[i]<avgdist2[bestPos])
				bestPos = i;
		}
		bestPos = currentAlignment.length/2;//der beste platz ist immer in der mitte?

		route[currentAlignment.path[bestPos].x][currentAlignment.path[bestPos].y-1] = -4;

		currentAlignment.rotationOffset = -(double)((currentAlignment.path[bestPos].x - currentAlignment.path[bestPos].y))*(2*PI)/SECTORS;

		matchBuffer[matchBufferPos].x = currentAlignment.path[bestPos].x;
		matchBuffer[matchBufferPos].y = currentAlignment.path[bestPos].y;
		matchBuffer[matchBufferPos].slope = 
			(currentAlignment.path[1].x - currentAlignment.path[currentAlignment.length-2].x==0)?0:
			(currentAlignment.path[1].y - currentAlignment.path[currentAlignment.length-2].y)/((double)currentAlignment.path[1].x - currentAlignment.path[currentAlignment.length-2].x);
		matchBufferPos = (matchBufferPos+1) % MATCH_BUFFER;
	}


}


void PanoramaModel::findDiagonal() {
	int firstSeen=SECTORS,lastSeen=0;
	int seen,world;

	for(seen=0;seen<SECTORS;seen++) {
		if(panoramaHistogram.sector[seen].empty) continue;
		firstSeen = min(firstSeen,seen);
		lastSeen = max(lastSeen,seen);
	}
	currentAlignment.rotationHead = (lastSeen+firstSeen-80)*PI/SECTORS;

	WINDEBUG(
	for( world=0;world<SECTORS;world++) {
		for( seen=0;seen<=SECTORS;seen++) {
			route[world][seen] = 0;
		}
	}
	)

/*
	double sumSeen[SECTORS];
	double sumWorld[SECTORS];
	// compute sum over world sectors (normalize columns)
	for( world=0;world<SECTORS;world++) {
		sumSeen[world] = 1;
		for( seen=firstSeen;seen<=lastSeen;seen++) {
			sumSeen[world] += fabs(correlation[world][seen]);
		}
	}

	// compute sum over seen sectors (normalize rows)
	for( seen=0;seen<SECTORS;seen++) {
		sumWorld[seen] = 1;
		for( world=0;world<SECTORS;world++) {
			sumWorld[seen] += fabs(correlation[world][seen]);
		}
	}
*/
	// normalize and init distance array
	for( world=0;world<SECTORS;world++) {
		for( seen=0;seen<SECTORS;seen++) {
			if(correlation[world][seen]==0) 
				distance[world][seen] = -1;
			else
//				distance[world][seen] = fabs(correlation[world][seen]/(sumWorld[seen]*sumSeen[world]));
				distance[world][seen] = fabs(correlation[world][seen]);
		}
	}

	double diagonalSum[MAX_SECTORS];
	for(world=0;world<SECTORS;world++) {
		diagonalSum[world]=0;
		for(seen=firstSeen;seen<lastSeen;seen++) {
		  if(correlation[(world+seen-firstSeen) % SECTORS][seen]==0) {
				diagonalSum[world]=-1;
				break;
			}
			diagonalSum[world] = diagonalSum[world] + fabs(correlation[(world+seen-firstSeen) % SECTORS][seen]);
		}
	}

	int worldBest=-1;
	double average=0;
	int avgCount=1;
	double normLikelihood = 0;
	if ((numLikelihoods>0) && (sumLikelihoods != 0)) normLikelihood = numLikelihoods/sumLikelihoods;

	for(world=0;world<SECTORS;world++) {
		if(diagonalSum[world]<=0) continue;

		currentAlignment.likelihood[world] = diagonalSum[world] * normLikelihood;
		sumLikelihoods = sumLikelihoods + diagonalSum[world];
		numLikelihoods ++;

		if(diagonalSum[world]<diagonalSum[worldBest] || worldBest<0) {
			worldBest = world;
		}
		average = average + diagonalSum[world];
		avgCount++;
	}
	average = average/avgCount;

	currentAlignment.length = 0;
	for(seen=firstSeen;seen<lastSeen;seen++) {
		currentAlignment.path[currentAlignment.length].x = (worldBest+seen-firstSeen) % SECTORS;
		currentAlignment.path[currentAlignment.length].y = seen;
		currentAlignment.length++;
		route[worldBest+seen-firstSeen][seen] = -3;
	}

	if(diagonalSum[worldBest]!=0)
		currentAlignment.signalToNoiseRatio = (average/diagonalSum[worldBest]-1) * panoramaHistogramDistribution.sector[worldBest].normalization;
	currentAlignment.rotationOffset = -(double)((worldBest - firstSeen+SECTORS)% SECTORS)*(2*PI)/SECTORS;
	
}

void PanoramaModel::normalize() {
	int s,w;
	for(s=0;s<5;s++) {
		w = (s-2 + currentAlignment.path[0].x + SECTORS) % SECTORS;

		panoramaHistogramDistribution.sector[w].normalization = 
			(1-(0.1 * (3-abs(2-s)) )) * 
				panoramaHistogramDistribution.sector[w].normalization
			+
			((0.1 * (3-abs(2-s)) )) * 
				panoramaHistogramDistribution.sector[w].normalization
				/ currentAlignment.signalToNoiseRatio;
	}
}


void PanoramaModel::computeLikelihoods() {
	if(OPTIMIZATIONS )
		panoramaHistogramDistribution.updateBitmask();//prepare incoming image percept
	for(int i=0;i<SECTORS;i++) {
		if(panoramaHistogramDistribution.sector[i].empty) continue;
		for(int j=0;j<SECTORS;j++) {
			if(panoramaHistogram.sector[j].empty) continue;
			correlation[i][j] = panoramaHistogramDistribution.computeLikelihood(i,j);
		}
	}
}

void PanoramaModel::visualizeLikelihoods(Image &debugImage) {
	double brightness[MAX_SECTORS][MAX_SECTORS];


	int world,seen,ix,iy;
	double bmin=100000000,bmax=-1000000000;
	for( world=0;world<SECTORS;world++) {
		for( seen=0;seen<SECTORS;seen++) {
			brightness[world][seen] = distance[world][seen];
			if(distance[world][seen]>0) {
				bmin = min(bmin,brightness[world][seen]);
				bmax = max(bmax,brightness[world][seen]);
			}
		}
	}

	for(iy=0;iy<cameraResolutionHeight_ERS7;iy++) 
		for(ix=0;ix<cameraResolutionWidth_ERS7;ix++) {
			debugImage.image[iy][0][ix] = 0;
			debugImage.image[iy][1][ix] = 0;
			debugImage.image[iy][2][ix] = 0;
		}

#ifdef SHOW_MATCHING
	int b;
	//bmin += 3*(bmax-bmin)/4;
	for(iy=0;iy<cameraResolutionHeight_ERS7;iy++) 
		for(ix=0;ix<cameraResolutionWidth_ERS7;ix++) {
			world = (ix-cameraResolutionWidth_ERS7/2)/2+SECTORS/2;
			seen = (iy-cameraResolutionHeight_ERS7/2)/2+SECTORS/2;

			if(world<0 || world>=SECTORS || seen<0 || seen>=SECTORS)
				continue;

			b = (int)((brightness[world][seen]-bmin)*255/(bmax-bmin));
			b = 255-max(0,min(255,b));

			
			debugImage.image[iy][0][ix] = b;
			debugImage.image[iy][1][ix] = b;
			debugImage.image[iy][2][ix] = b;

			if(panoramaHistogram.sector[seen].empty) {
				debugImage.image[iy][0][ix] = b/2;
				debugImage.image[iy][1][ix] = b/2;
			}
			if(seen>0 && route[world][seen-1]==-6) {//noise
				debugImage.image[iy][0][ix] = 255;
				debugImage.image[iy][1][ix] = 0;
				debugImage.image[iy][2][ix] = 0;
			}
			if(seen>0 && route[world][seen-1]==-4) {//center point
				debugImage.image[iy][0][ix] = 128;
				debugImage.image[iy][1][ix] = 255;
				debugImage.image[iy][2][ix] = 128;
			}
			if(seen>0 && route[world][seen-1]==-3) {//alignment
				debugImage.image[iy][0][ix] = 0;
				debugImage.image[iy][1][ix] = 255;
				debugImage.image[iy][2][ix] = 0;
			}
			if(seen>0 && route[world][seen-1]==-5) {
				debugImage.image[iy][0][ix] = 255;
				debugImage.image[iy][1][ix] = 0;
				debugImage.image[iy][2][ix] = 255;
			}
			if(seen>0 && route[world][seen]==-2) {
				debugImage.image[iy][0][ix] = 128;
				debugImage.image[iy][1][ix] = 64;
				debugImage.image[iy][2][ix] = 64;
			}

		}

#endif

#ifdef SHOW_MATCHBUFFER
	int i;
	for(i=0;i<MATCH_BUFFER;i++)
		if(matchBuffer[i].x!=-1) {
			world = matchBuffer[i].x;
			seen = matchBuffer[i].y;
			ix = world*cameraResolutionWidth_ERS7/SECTORS;
			iy = seen*cameraResolutionHeight_ERS7/SECTORS;

			debugImage.image[iy][0][ix] = 255;
			debugImage.image[iy][1][ix] = 0;
			debugImage.image[iy][2][ix] = 0;

		}
#endif
	/*
	int i;

	for(i=0;i<ALIGNMENT_BUFFER;i++)
		if(alignmentBuffer[i].x!=-1) {
			world = alignmentBuffer[i].x;
			seen = alignmentBuffer[i].y;
			ix = world*cameraResolutionWidth_ERS7/SECTORS+1;
			iy = seen*cameraResolutionHeight_ERS7/SECTORS;

			debugImage.image[iy][0][ix] = 0;
			debugImage.image[iy][1][ix] = 255;
			debugImage.image[iy][2][ix] = 0;

		}
	*/ 

#ifdef SHOW_MAPPING
	for(seen=0;seen<SECTORS;seen++) {
		world = convertLocalToGlobalSector(seen);
		//if(panoramaHistogram.sector[seen].empty) {
			ix = world*cameraResolutionWidth_ERS7/SECTORS;
			iy = seen*cameraResolutionHeight_ERS7/SECTORS+1;

			debugImage.image[iy][0][ix] = 255;
			debugImage.image[iy][1][ix] = 255;
			debugImage.image[iy][2][ix] = 255;
		//}
	}
#endif


#ifdef SHOW_SNR
	for(ix = 0;ix<cameraResolutionWidth_ERS7;ix++)
		for(iy = 0;iy<10;iy++) {
			if(ix< currentAlignment.signalToNoiseRatio * cameraResolutionWidth_ERS7 * 0.5) {
				debugImage.image[iy][0][ix] = 255;
				debugImage.image[iy][1][ix] = 0;
				debugImage.image[iy][2][ix] = 0;
			} else {
				debugImage.image[iy][0][ix] = 0;
				debugImage.image[iy][1][ix] = 0;
				debugImage.image[iy][2][ix] = 0;
			}
		}

#endif
}

void PanoramaModel::drawTransition(int sector, int offset, int size, unsigned char bottom, unsigned char top) {
	double rad1 = ((double)sector)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad2 = ((double)sector+0.33)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad3 = ((double)sector+0.66)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double rad4 = ((double)sector+1)/SECTORS*2*PI - robotPose.rotation- PI/2;
	double dist1 = offset+(-0.4)*size;
	double dist2 = offset+(0.0)*size;
	double dist3 = offset+(0.4)*size;

	Vector3<unsigned char> c1 = 	slamData.clusterColor[bottom],first;
	Vector3<unsigned char> c2 = 	slamData.clusterColor[top],second;
	Image::convertFromYCbCrToRGB(c1.x,c1.y,c1.z,first.x,first.y,first.z);
	Image::convertFromYCbCrToRGB(c2.x,c2.y,c2.z,second.x,second.y,second.z);


	OCTANGLE_RGB(selfLocatorField,
		sin(rad1)*dist1+robotPose.translation.x,cos(rad1)*dist1+robotPose.translation.y,
		sin(rad2)*dist1+robotPose.translation.x,cos(rad2)*dist1+robotPose.translation.y,
		sin(rad3)*dist1+robotPose.translation.x,cos(rad3)*dist1+robotPose.translation.y,
		sin(rad4)*dist1+robotPose.translation.x,cos(rad4)*dist1+robotPose.translation.y,
		sin(rad4)*dist2+robotPose.translation.x,cos(rad4)*dist2+robotPose.translation.y,
		sin(rad3)*dist2+robotPose.translation.x,cos(rad3)*dist2+robotPose.translation.y,
		sin(rad2)*dist2+robotPose.translation.x,cos(rad2)*dist2+robotPose.translation.y,
		sin(rad1)*dist2+robotPose.translation.x,cos(rad1)*dist2+robotPose.translation.y,
		first.x,
		first.y,
		first.z);
	OCTANGLE_RGB(selfLocatorField,
		sin(rad1)*dist2+robotPose.translation.x,cos(rad1)*dist2+robotPose.translation.y,
		sin(rad2)*dist2+robotPose.translation.x,cos(rad2)*dist2+robotPose.translation.y,
		sin(rad3)*dist2+robotPose.translation.x,cos(rad3)*dist2+robotPose.translation.y,
		sin(rad4)*dist2+robotPose.translation.x,cos(rad4)*dist2+robotPose.translation.y,
		sin(rad4)*dist3+robotPose.translation.x,cos(rad4)*dist3+robotPose.translation.y,
		sin(rad3)*dist3+robotPose.translation.x,cos(rad3)*dist3+robotPose.translation.y,
		sin(rad2)*dist3+robotPose.translation.x,cos(rad2)*dist3+robotPose.translation.y,
		sin(rad1)*dist3+robotPose.translation.x,cos(rad1)*dist3+robotPose.translation.y,
		second.x,
		second.y,
		second.z);
}


void PanoramaModel::visualizeModel() {
	int sort[MAX_CLUSTERS*MAX_CLUSTERS],i,j,b;
	double importance[MAX_CLUSTERS][MAX_CLUSTERS];
	int dist,size;

	SectorHistogramDistribution &sum = panoramaHistogramDistribution.sum;
	for(int sector=0;sector<SECTORS;sector++) {

		SectorHistogramDistribution &currentSector = panoramaHistogramDistribution.sector[sector];

		// linear init
		for(i=0;i<CLUSTERS*CLUSTERS;i++) 
			sort[i] = i;

		// set sort[]=-1 to exclude transitions
		for(i=0;i<CLUSTERS;i++) 
		for(j=0;j<CLUSTERS;j++) {
			importance[i][j]=0;
			for(b=0;b<BINS;b++) {
				importance[i][j] += sqr(
														(currentSector.freq[i][j][b]/(1.+currentSector.sum[i][j]))-
														(sum.freq[i][j][b]/(1.+sum.sum[i][j])));
			}
		}
			 
		// bubble sort
		int temp;
		for(i=0;i<CLUSTERS*CLUSTERS;i++) {
			for(j=0;j<CLUSTERS*CLUSTERS-1;j++) {
				if((sort[j] != -1) && (
						importance[ sort[j] / CLUSTERS ]
							  			[ sort[j] % CLUSTERS ] 
						< 
						importance[ sort[j+1] / CLUSTERS ]
							  			[ sort[j+1] % CLUSTERS ] 
					)){
					temp = sort[j];
					sort[j] = sort[j+1];
					sort[j+1] = temp;
				}
			}
		}

		// display N most important transitions
		dist = 1500;
		for(i=0;i<1;i++) {
			if(currentSector.sum[ sort[i] / CLUSTERS ]
													[ sort[i] % CLUSTERS ]==0) break;
			//OUTPUT(idText,text,(sort[i] / CLUSTERS) << "<->"<<(sort[i] % CLUSTERS)<<": "<<currentSector.freq[ sort[i] / CLUSTERS ]
			//												 [ sort[i] % CLUSTERS ])  ;
			size = currentSector.sum[ sort[i] / CLUSTERS ]
															 [ sort[i] % CLUSTERS ]*10+50;
			size = min(max(size,100),300);
			drawTransition(sector,dist,size,sort[i] / CLUSTERS,sort[i] % CLUSTERS);
			dist -= 300;

			if(dist < 500) break;
		}
	}
}

void PanoramaModel::estimatePhase() {
	int distortion[MAX_SECTORS];
	int kernel[MAX_SECTORS];
	int filteredDistortion[MAX_SECTORS];
	bool voted[MAX_SECTORS];
	double sigma = SIGMA_PHASE/360.*SECTORS;

	int i,j;
	for(i=0;i<SECTORS;i++) {
		distortion[i] = 0;
		filteredDistortion[i] = 0;
		voted[i] = 0;
		kernel[i] = (int)
				(1e6 
					* 
							(1/(sigma*sqrt(2*PI))) 
						*	exp(-sqr(i)/(2*sqr(sigma))));
	}

	int offset;
	for(i=0;i<MATCH_BUFFER;i++) {
		if(matchBuffer[i].x!=-1) {
			if(voted[matchBuffer[i].x]) continue;
			voted[matchBuffer[i].x]=true;
			offset = matchBuffer[i].x - matchBuffer[i].y*SECTORS/SECTORS;
			distortion[(offset + SECTORS) % SECTORS]++;
		}
	}

	
	for(i=0;i<SECTORS;i++) {
		for(j=0;j<sigma;j++) {
			filteredDistortion[i] += 
				(distortion[(i-j+SECTORS)%SECTORS] +
				 distortion[(i+j+SECTORS)%SECTORS])*kernel[j];
		}
	}


	int bestPhase = 0;
	for(i=0;i<SECTORS;i++) {
		if(filteredDistortion[i]>filteredDistortion[bestPhase])
			bestPhase = i;
	}

	matchPhase = bestPhase;

/*	
	int world,seen;
	for(world=0;world<SECTORS;world++) {
		seen = (world - bestPhase*SECTORS/SECTORS+SECTORS)%SECTORS;
		route[world][seen] = -5;

	}

	double centerWorld = (SECTORS/2 + bestPhase*SECTORS/SECTORS+SECTORS)%SECTORS;

	double newsigma = 50*SECTORS / 360;//deg


	for(i=0;i<SECTORS;i++) {
		distortion[i] = 0;
		filteredDistortion[i] = 0;
		offset = matchBuffer[i].x - matchBuffer[i].y*SECTORS/SECTORS;
		voted[i] = fabs(offset) > sigma;
		kernel[i] = (int)
				(1e6 
					* 
							(1/(newsigma*sqrt(2*PI))) 
						*	exp(-sqr(i)/(2*sqr(newsigma))));
	}

	int xdiff,ydiff;
	for(i=0;i<MATCH_BUFFER;i++) {
		if(matchBuffer[i].x!=-1) {
			if(voted[matchBuffer[i].x]) continue;
			voted[matchBuffer[i].x]=true;
			xdiff = (int)(matchBuffer[i].x-centerWorld);
			ydiff = ((matchBuffer[i].y-SECTORS/2)*SECTORS/SECTORS);
			if(xdiff*ydiff<0) continue;
			offset = (int)(
				atan2(
					xdiff,
					ydiff
				)
				*SECTORS/(2*PI))+SECTORS/2;
			offset = (offset+SECTORS) % SECTORS;
			distortion[offset] += (int)sqrt(sqr(xdiff)+sqr(ydiff));
		}
	}

	
	for(i=0;i<SECTORS;i++) {
		for(j=0;j<sigma;j++) {
			filteredDistortion[i] += 
				(distortion[(i-j+SECTORS)%SECTORS] +
				 distortion[(i+j+SECTORS)%SECTORS])*kernel[j];
		}
	}

	int bestRegression = 0;
	for(i=0;i<SECTORS;i++) {
		if(filteredDistortion[i]>filteredDistortion[bestRegression])
			bestRegression = i;
	}
	double radRegression = bestRegression*2*PI/SECTORS;
	for(i=-30;i<30;i++) {
		world = (int)(centerWorld + (i*sin(radRegression)*SECTORS/SECTORS ));
		seen = (int)(SECTORS/2+		(i*cos(radRegression)*SECTORS/SECTORS ));
		world = (world + SECTORS)%SECTORS;
		seen = (seen+SECTORS)%SECTORS;
		route[world][seen] = -5;

	}
	route[(int)centerWorld][SECTORS/2] = -3;
*/
}

void PanoramaModel::linearRegress(Alignment point[],int points,double &slope,double &b) {
	// linear regression, following http://people.hofstra.edu/faculty/Stefan_Waner/RealWorld/tutorialsf0/frames1_5.html
	int sumXY=0,sumX=0,sumY=0,sumSqrX=0,sumXSqr=0;

	for(int i=0;i<points;i++) {
		sumXY += point[i].x*point[i].y;
		sumX += point[i].x;
		sumY += point[i].y;
		sumSqrX += sqr(point[i].x);
	}
	sumXSqr = sqr(sumX);

	slope = (points * sumXY - sumX*sumY) / ((double)points*sumSqrX - sumXSqr);
	b = (sumY - slope*sumX) / points;
}


void PanoramaModel::computeIntersections() {
	// center
	int worldHeading = (int)((SECTORS/2 - regressionB)/regressionSlope - SECTORS/2);
	worldHeading = (worldHeading + SECTORS) % SECTORS;

	int count[MAX_SECTORS][2];
	double mass[MAX_SECTORS][2];

	int i;
	for(i=0;i<SECTORS;i++) {
		count[i][0]=0;
		count[i][1]=0;
		mass[i][0]=0;
		mass[i][1]=0;
	}

	double tangent,offset;
	for(i=0;i<ALIGNMENT_BUFFER;i++) {
		tangent = regressionSlope * alignmentBuffer[i].x + regressionB;
		offset = alignmentBuffer[i].y - tangent;

		if(fabs(offset)>SIGMA_PHASE/2) continue; // outlier, ignore

		if ( offset > 0 ) {
			count[alignmentBuffer[i].x][0]++;// above
			mass[alignmentBuffer[i].x][0]+=fabs(offset);// above
		} else {
			count[alignmentBuffer[i].x][1]++;//below
			mass[alignmentBuffer[i].x][1]+=fabs(offset);// below
		}
	}
	
	int world,seen;
	for(world=0;world<SECTORS;world++) {
		seen = (int)(regressionSlope * world + regressionB);
		
		seen = (seen + SECTORS) % SECTORS;

		if(count[world][0]>count[world][1])
			route[world][seen] = -5;
		else
			route[world][seen] = -4;
	}

	// fixme: some smoothing?

	// compute the mass of the center region
	int sign = count[worldHeading][0] - count[worldHeading][1];
	double massSum = 0;

	for(i=(int)worldHeading;i!=(worldHeading-1+SECTORS)%SECTORS;i=(i+1+SECTORS)%SECTORS) {
		if(sign * (count[i][0] - count[i][1])<0) break;
		if(count[i][0]+count[i][1]==0) continue;
		massSum += (mass[i][0]+mass[i][1])/(count[i][0]+count[i][1]);
	}

	for(i=(int)worldHeading;i!=(worldHeading+1+SECTORS)%SECTORS;i=(i-1+SECTORS)%SECTORS) {
		if(sign * (count[i][0] - count[i][1])<0) break;
		if(count[i][0]+count[i][1]==0) continue;
		massSum += (mass[i][0]+mass[i][1])/(count[i][0]+count[i][1]);
	}

	if(sign<0) massSum *= -1;


	OUTPUT(idText,text,"center mass ="<<massSum)  ;

}

void PanoramaModel::computePosition() {
	double worldHeading = (SECTORS/2 - regressionB)/regressionSlope - SECTORS/2;
	double worldDistance = regressionSlope / SECTORS * SECTORS * panoramaRadius - panoramaRadius;
	double worldShift = regressionDiffLeftRight  / SECTORS * SECTORS * panoramaRadius;
	while(worldHeading<0) worldHeading += SECTORS;

	estimatedPose.rotation = -worldHeading*2*PI/SECTORS;
	if(estimatedPose.rotation>PI) estimatedPose.rotation -= 2*PI;
	if(estimatedPose.rotation<=-PI) estimatedPose.rotation += 2*PI;

	estimatedPose.translation.x = cos(estimatedPose.rotation) * worldDistance;
	estimatedPose.translation.y = sin(estimatedPose.rotation) * worldDistance;
	estimatedPose.translation.x += -sin(estimatedPose.rotation) * worldShift;
	estimatedPose.translation.y += cos(estimatedPose.rotation) * worldShift;
	estimatedPose.setValidity(1.0);

}

void average(Alignment point[],int points,Vector2<int> &average) {
	average = Vector2<int>(0,0);
	for(int i=0;i<points;i++)
		average += point[i];
	average /= points;
}

void averageSlope(Alignment point[],int points,double slope) {
	slope = 0;
	for(int i=0;i<points;i++)
		slope += point[i].slope;
	slope /= points;
}

void constructLine(Vector2<int> start,Vector2<int> end,double &slope,double &b) {
	if(end.x-start.x==0) {
		slope =0;
		b = 0;
	} else {
		slope = (end.y-start.y)/(double)(end.x-start.x);
		b = start.y + slope*(-start.x);
	}
}

void PanoramaModel::computeLinearRegression() {
	bool worldSectorUsed[MAX_SECTORS];
	Alignment point[MATCH_BUFFER];
	int points=0;

	// filter away double world sectors
	int i;
	for(i=0;i<SECTORS;i++)
		worldSectorUsed[i] = false;

	// filter away outliers
	int phaseDiff;
	for(i=0;i<MATCH_BUFFER;i++) {
		phaseDiff = (matchBuffer[i].x - matchBuffer[i].y*SECTORS/SECTORS+2*SECTORS-matchPhase)%SECTORS;
		if(								// reject perception ?
			matchBuffer[i].x!=-1 &&					// because not filled in
			!worldSectorUsed[matchBuffer[i].x]	&&	// or seen this world sector before
			phaseDiff<SIGMA_PHASE
		){
			worldSectorUsed[matchBuffer[i].x] = true;
			point[points++] = matchBuffer[i];
		}
	}

	if(points<2) return;

	linearRegress(point,points,regressionSlope,regressionB);

	// split points there
	int worldHeading = (int)((SECTORS/2 - regressionB)/regressionSlope);
	worldHeading = (worldHeading + SECTORS)%SECTORS;

	Alignment left[ALIGNMENT_BUFFER],right[ALIGNMENT_BUFFER];
	int leftPoints=0,rightPoints=0;
	for(i=0;i<points;i++) {
		if(point[i].x<worldHeading) 
			left[leftPoints++] = point[i];
		else
			right[rightPoints++] = point[i];
	}
	regressionDiffLeftRight = 0;

	double slopeLeft,slopeRight,bLeft,bRight;
	Vector2<int> newCenter,centerLeft,centerRight;
	newCenter = Vector2<int>(worldHeading,(int)(worldHeading * regressionSlope + regressionB));

	slopeLeft = regressionSlope;
	slopeRight = regressionSlope;
	averageSlope(left,leftPoints,slopeLeft);
	averageSlope(right,rightPoints,slopeRight);

	centerLeft = newCenter - Vector2<int>(1000,(int)(slopeLeft*1000));
	centerRight = newCenter + Vector2<int>(1000,(int)(slopeRight*1000));

	constructLine(centerLeft,newCenter,slopeLeft,bLeft);
	constructLine(newCenter,centerRight,slopeRight,bRight);

	regressionDiffLeftRight = (slopeLeft-slopeRight);
	//regressionSlope = (slopeLeft+slopeRight)/2;

	/*
	for(i=0;i<ALIGNMENT_BUFFER;i++) {
		phaseDiff = (alignmentBuffer[i].x - alignmentBuffer[i].y*SECTORS/SECTORS+2*SECTORS-matchPhase)%SECTORS;
		if(								// reject perception ?
			alignmentBuffer[i].x!=-1 &&					// because not filled in
			phaseDiff<SIGMA_PHASE
		){
			if(alignmentBuffer[i].x<worldHeading) 
				left[leftPoints++] = alignmentBuffer[i];
			else
				right[rightPoints++] = alignmentBuffer[i];
		}
	}


	linearRegress(left,leftPoints,slopeLeft,bLeft);
	linearRegress(right,rightPoints,slopeRight,bRight);

	// now ensure that left and right line touch each other by:
	// 1. computing mass points
	// 2. compute touch point of endings
	// 3. reestablish slope and b
	Vector2<int> centerLeft,centerRight;
	average(left,leftPoints,centerLeft);
	average(right,rightPoints,centerRight);

	Vector2<int> endLeft(worldHeading,(int)(worldHeading * slopeLeft + bLeft));
	Vector2<int> endRight(worldHeading,(int)(worldHeading * slopeRight + bRight));

	Vector2<int> newCenter = (endLeft+endRight)/2;
	newCenter = Vector2<int>(worldHeading,(int)(worldHeading * regressionSlope + regressionB));

	constructLine(centerLeft,newCenter,slopeLeft,bLeft);
	constructLine(newCenter,centerRight,slopeRight,bRight);

	regressionDiffLeftRight = (slopeLeft-slopeRight);
	*/

#ifdef SHOW_ESTIMATED_MAPPING
	int world,seen;
	for(world=0;world<SECTORS;world++) {
		seen = (int)(regressionSlope * world + regressionB+0.5);
		
		seen = (seen + SECTORS) % SECTORS;
		if(route[world][seen]>=0)
		route[world][seen] = -6;
/*
		if(world<worldHeading) 
			seen = (int)(slopeLeft * world + bLeft);
		else
			seen = (int)(slopeRight * world + bRight);
		seen = (seen + SECTORS) % SECTORS;

		route[world][seen] = (world<=worldHeading)?-5:-4;
*/
	}
#endif


}

In& operator>>(In& stream,PanoramaModel& panoramaModel) {
	stream >> panoramaModel.panoramaCenter;
	stream >> panoramaModel.panoramaRadius;
	stream >> panoramaModel.panoramaHistogramDistribution;
	return stream;
}

Out& operator<<(Out& stream, const PanoramaModel& panoramaModel) {
	stream << panoramaModel.panoramaCenter;
	stream << panoramaModel.panoramaRadius;
	stream << panoramaModel.panoramaHistogramDistribution;
	return stream;

}

