/**
*
*/

#ifndef __LocalAlignment_h_
#define __LocalAlignment_h_

#include "Representations/Perception/PanoramaLibrary.h"
#include "HistogramDistributionLibrary.h"
#include "Tools/Debugging/DebugImages.h"
#include "Representations/Perception/Image.h"
#include "Representations/Perception/LandmarksPercept.h"
#include "Representations/Perception/SLAMData.h"
#include "Tools/Debugging/Stopwatch.h"
#include "Representations/Perception/PanoramaParameters.h"

class LocalAlignment {
public:
	Vector2<int> path[MAX_SECTORS];	// x=world, y=seen
	int length;
	double signalToNoiseRatio;
	double rotationOffset;
	double rotationHead;
	double likelihood[MAX_SECTORS];


	LocalAlignment();
	void reset();

	LocalAlignment& operator=(const LocalAlignment& other) 
	{
		memcpy((void *)this, (const void *)&other, sizeof(LocalAlignment));
		return *this;
	}
};

class Alignment: public Vector2<int> {
public:
  Alignment& operator=(const Alignment& other)
  {
    x=other.x;y=other.y;slope=other.slope;
    return *this;
  }
	double slope;
};

class PanoramaModel {
public:
	PanoramaHistogramDistribution panoramaHistogramDistribution;
	const PanoramaHistogram &panoramaHistogram;
	RobotPose &robotPose;
	SLAMData &slamData;

	RobotPose estimatedPose;

	Pose2D panoramaCenter;
	int panoramaRadius;
	int panoramaSectors;

	LocalAlignment currentAlignment;
	double sumLikelihoods;
	long numLikelihoods;

	double correlation[MAX_SECTORS][MAX_SECTORS];
	int age[MAX_SECTORS][MAX_SECTORS];
	double matchMin,matchMax;
	int seenLeft,seenRight;//area of interest

	double distance[MAX_SECTORS][MAX_SECTORS];
	int route[MAX_SECTORS][MAX_SECTORS];
	Alignment matchBuffer[MATCH_BUFFER];
	int matchBufferPos;
	Vector2<int> alignmentBuffer[ALIGNMENT_BUFFER];
	int alignmentBufferPos;
	int matchPhase;// raw estimate
	double regressionSlope;
	double regressionB;
	double regressionDiffLeftRight;

	PanoramaModel(
			Pose2D panoramaCenter, int panoramaRadius,
			const LandmarksPercept &landmarksPercept,
			RobotPose &robotPose,SLAMData &slamData);
	int convertLocalToGlobalSector(double relativeSector);

	void reset();
	void learn();
	void match();
	void normalize();

	void computeLikelihoods();
	void computeShortestPath();
	void findDiagonal();
	void estimatePhase();
	void computeLinearRegression();// 2: compute big regression line, then split up in two regression lines left and right
	void computeIntersections();// 1: derive convex,concav properties of regression line
	void computePosition();
	void linearRegress(Alignment point[],int points,double &slope,double &b);

	void visualizeLikelihoods(Image &debugImage);
	void visualizeModel();
	void drawTransition(int sector, int offset, int size, unsigned char bottom, unsigned char top);

};


In& operator>>(In& stream,PanoramaModel& panoramaModel);
Out& operator<<(Out& stream, const PanoramaModel& panoramaModel);

/*
class WorldModel {
public:
	int panoramaModels;
	PanoramaModel panoramaModel[MAX_PANORAMAS];

	const PanoramaHistogram &panoramaHistogram;
	RobotPose &robotPose;

	WorldModel(
			const PanoramaHistogram &panoramaHistogram,
			RobotPose &robotPose);

	void reset();
	void learn();
	void match();

}
*/


#endif// __LocalAlignment_h_

/*
*/
