#include "ManualPositions.h"

ManualPositions::ManualPositions(const LandmarksPercept &landmarksPercept,RobotPose &robotPose,SLAMData &slamData) 
: landmarksPercept(landmarksPercept), robotPose(robotPose), slamData(slamData) 
{
	currentPosition = -1;
	positions = 0;

	char buf[255];
	
	InTextFile infile("manual.txt");
	if(infile.exists()) {
		while(!infile.eof()) {
			infile >> storedPosition[positions].frameNumber; 
			if(storedPosition[positions].frameNumber!=0) {
				infile >> storedPosition[positions].pose.translation.y;
				infile >> storedPosition[positions].pose.translation.x;
				infile >> storedPosition[positions].pose.rotation;
				infile >> buf;
				if(strcmpi(buf,"learn")==0) 
					storedPosition[positions].type = positionLearn;
				else
				if(strcmpi(buf,"test")==0) 
					storedPosition[positions].type = positionTest;
				else
				if(strcmpi(buf,"free")==0) 
					storedPosition[positions].type = positionFree;
				else
					OUTPUT(idText, text, "ManualPositions() error: unknown position type: "<<buf);
				infile >> storedPosition[positions].name;
				positions++;
				if(positions==MAX_MANUAL_POSITIONS) break;
			} else 
				infile >> endl;
		}
		OUTPUT(idText, text, "ManualPositions() loaded "<<positions <<" stored positions");
	} else {
		OUTPUT(idText, text, "ManualPositions() config/manual.txt not found!");
	}
}

void ManualPositions::execute() {
	int bestP=-1;
	int bestDist=10000000,dist;
	for(int p=0;p<positions;p++) {
		dist = landmarksPercept.frameNumber -storedPosition[p].frameNumber;
		if(dist>=0 && dist < bestDist) {
			bestP = p;
			bestDist = dist;
		}
	}
	if(bestP >=0 &&(storedPosition[bestP].type != positionFree )) {
			robotPose.translation = storedPosition[bestP].pose.translation;
			robotPose.rotation = storedPosition[bestP].pose.rotation;
			robotPose.setValidity(1.0);
	}
	if(bestP != currentPosition) {
		

		if(bestP >=0 &&(storedPosition[bestP].type != positionFree )) {
			OUTPUT(idText, text, "ManualPositions: assuming "<<storedPosition[bestP].name);

			robotPose.translation = storedPosition[bestP].pose.translation;
			robotPose.rotation = storedPosition[bestP].pose.rotation;
			robotPose.setValidity(1.0);

			switch(storedPosition[bestP].type) {
				case positionLearn:
					slamData.requestAction(SLAMData::recordPanorama);
					break;
				case positionTest:
					slamData.requestAction(SLAMData::localize);
					break;
				case positionFree:
					break;
			}
		} else {
			OUTPUT(idText, text, "ManualPositions: free position");
		}
		currentPosition = bestP;
	} 

}

