/**
*
*/

#ifndef __PanoramaLocator_h_
#define __PanoramaLocator_h_

#include "Modules/SelfLocator/SelfLocator.h"
#include "HistogramDistributionLibrary.h"
#include "LocalAlignment.h"
#include "GlobalAlignment.h"
#include "ManualPositions.h"

#define MAX_FRAMES 1000
#define MAX_SPOTS 10

/**
* Class for self-localization based on the panorama
*/
class PanoramaLocator : public SelfLocator
{
public:
	/** 
	* Constructor.
	* @param interfaces The paramters of the SelfLocator module.
	*/
	PanoramaLocator(const SelfLocatorInterfaces& interfaces);

	/**
	* Destructor.
	*/
	~PanoramaLocator() {}

	/** Executes the module */
	virtual void execute();

	void evaluateLocalization();

	void linearRegress(
		double headRotation,
		double headWidth,
		double offsetRotation,
		double offsetWidth,
		double &slope,double &b,double &mass);

	int learnCount;
	int localizationCount;

	int spotCount;
	PanoramaModel * spot[MAX_SPOTS];

	double kernel20[360];
	double bearingAccumulator[360];
	double snrSpotAverage[MAX_SPOTS];
	int snrSpotAge[MAX_SPOTS];

	int selectedSpot;
	RobotPose estimatedPose;
	Pose2D lastOdometryData;
	double remainderOdometryDeltaRotation;
	

	int storedFrames;
	double offset[MAX_FRAMES];
	double snr[MAX_FRAMES];
	double head[MAX_FRAMES];
	double reference0;

	Image RGBDebug;
	DECLARE_DEBUG_IMAGE(imageProcessorScanLines);
	
	void resetSpots();
	void appendSpot();
	void recordPanorama();
	void matchSpot();

	void updateBearingAccumulatorByOdometry();
	void updateBearingAccumulatorByAlignment();
	void decayBearingAccumulatorByTime(int halftime);

	void updateSpotAverageByAlignment(int halftime);

	void findStrongestBearing();
	void savePanorama(const char* filename);
	bool loadPanorama(const char* filename);
	void localize();
	void normalization();
	void visualize();
};

#endif// __PanoramaLocator_h_

/*
*/
