/**
* @file VariableLighting.h
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file VariableLighting.h
* 
*/

#ifndef __VariableLighting_h_
#define __VariableLighting_h_

#include "SpecialVision.h"
#include "Tools/SlidingAverageValue.h"
#include "Representations/Perception/ColorTable.h"
#include "Representations/Perception/ColorTable64.h"
#include "Representations/Perception/ColorTableTSL.h"

// over how many frames shall the Y median be averaged?
#define INTERPOLATED_FRAMES 5
// probability mass of median range considered to be normal before switching to the next setting
#define NORMALITY 0.9

class VariableSetting 
{
public:
	char name[255];
	int rangeLeft,rangeRight;
	CameraParameters::shutterSpeed shutter;
	CameraParameters::gain gain;
	ColorTable64 colorTable;

	VariableSetting();
};

/** A VariableLighting */
class VariableLighting : public SpecialVision
{
private:
	SlidingAverageValue slidingMedian;
	int lastDebugFrame;
	int timeToRecover;
	int distribution[257];
	VariableSetting settings[20];
	int numOfSettings;
	int currentSetting;
	int countTooBright;
	int countTooDark;
	bool writeColortableNextFrame;
	unsigned long frameNumber;
	bool learning;
	
public:
  /** Constructor 
  * @param interfaces The interfaces of the SpecialVision module */
  VariableLighting(const SpecialVisionInterfaces& interfaces);
  /** Executes the module */
  virtual void execute();

  void switchSetting(int delta);

  void reset();

};

#endif// __VariableLighting_h_

/*
* Change log :
* 
* $Log: VariableLighting.h,v $
*
*
*
*/
