/**
* @file OdometryCorrection.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file OdometryCorrection.cpp
* 
*/

#include "Modules/WalkingEngine/OdometryCorrection.h"
#include "Tools/Debugging/Debugging.h"

OdometryCorrection::OdometryCorrection(OdometryData& odometryData) 
:odometryData(odometryData)
{
//	OUTPUT(idText,text,"Creating OdometryCorrection " << endl);  
}

void OdometryCorrection::execute()
{
	Pose2D diff;
	
	diff.translation.x = odometryData.translation.x - referenceOdometry.translation.x;
	diff.translation.y = odometryData.translation.y - referenceOdometry.translation.y;
	diff.rotation = odometryData.rotation - referenceOdometry.rotation;
	//OUTPUT(idText,text,"OdometryCorrection x=" <<diff.translation.x<<" y="<<diff.translation.y<<" r="<<diff.rotation<< endl);  
	double correctionAngle =-22;//22 graden naar links per meter
	double correctionRotate = +120;//22 graden per 360 graden draaing

	Pose2D result;
	double length = sqrt(sqr(diff.translation.x)+sqr(diff.translation.y));
	result.translation.x = diff.translation.x;
//		cos(correctionAngle/(180/3.14))*diff.translation.x - 
//		sin(correctionAngle/(180/3.14))*diff.translation.y;
	result.translation.y = diff.translation.y;
//		cos(correctionAngle/(180/3.14))*diff.translation.y +
//		sin(correctionAngle/(180/3.14))*diff.translation.x;
	result.rotation = diff.rotation;

	// correction for (straight) walking
	result.rotation += (correctionAngle/(180/pi))*length/1000;//deg/m
	// correction for rotation
	//result.rotation += (correctionRotate/(180/pi))*(diff.rotation)/(2*pi);

	Pose2D correctedOdometry;
	correctedOdometry.translation.x = referenceOdometry.translation.x + result.translation.x;
	correctedOdometry.translation.y = referenceOdometry.translation.y + result.translation.y;
	correctedOdometry.rotation = referenceOdometry.rotation + result.rotation;

	odometryData.rotation = correctedOdometry.rotation;
	odometryData.translation = correctedOdometry.translation;

	referenceOdometry = odometryData;
}

/*
* Change log :
*
*/
