/** 
* @file  Platform/Aperios1.3.2/DDPHandler.h
*
* Handles the Dog Discovery Protocol (DDP) so that the robot
* is able to find out the ip's of his teammates.
*
* @author <A href=mailto:robocup@m-wachter.de>Michael Wachter</A>
*/

#ifndef	_DDPHandler_H_
#define	_DDPHandler_H_

#include "UDPEndpoint.h"
#include "Tools/Player.h"

/** 
  * @class DDPListEntry 
  * 
  * used to store the information of one discovered robot 
  */

class DDPListEntry
{
  public:
    IPAddress ip;  /**< The ip-address of the robot */
    Port port;     /**< port to send data to */
    unsigned long timeLastSeen; /**< Time when the last package from this robot was received */
};


/** @class DDPPackage
  * This class represents the data package which is send over the network
  */

class DDPPackage
{
   public:
      char teamIdentifier[16];  /**< a string discribing the team of the robot. 
								Data is only send to robots with the same teamIdentifier and teamColor */
      Player::teamColor teamColor; /**< TeamColor of the Robot */
      Player::playerNumber playerNumber; /**< PlayerNumber of the Robot */
};     

/** @class DDPHandler 
  *
  * This class handles the Dog-Discovery-Protocol DDP. Its used to find out the IP-addresses of
  * the teammates and stors them in a list. This list is then used by the UDPHandler for the 
  * team-communication
  *
  */
class DDPHandler : public UDPEndpoint
{
  public:
    DDPHandler();
    ~DDPHandler() {} ;
    void initDDPHandler();  /**< does stuff which can only be done after aperios completly runs */
    void sendDDPPackage();  /**< sends a ddp-package. This is periodically called by the UDP-Handler */
    virtual void onReceive(void* data, int size);  /**< called by the framework if a package is recived on the DDP-Port */

  public:
	  DDPListEntry list[Player::numOfPlayerNumbers-1]; /**< List of the other robots */
   
  private:
    DDPPackage thePackage; /**< Buffer for the DDPPackage to send **/
    unsigned long timeLastPackage; /**< Time when the last DDPPackage was send **/

};


#endif 

/*
 *
 * Changelog :
 *
 * $Log: DDPHandler.h,v $
 * Revision 1.3  2004/09/12 20:10:47  wachter
 * Documentation-fixes
 *
 * Revision 1.2  2004/09/08 12:41:43  wachter
 * - Fixed some warnings
 * - Fixed some documentation in network-code
 *
 * Revision 1.1.1.1  2004/05/22 17:23:25  cvsadm
 * created new repository GT2004_WM
 *
 * Revision 1.3  2004/05/14 14:12:08  wachter
 * - Added communication support for 5 robots
 * - rewrote parts of team-communication to be faster and more stable
 *
 * Revision 1.2  2004/01/09 15:44:30  wachter
 * Worked on with the Dog-Discovery-Protocol
 *
 * Revision 1.1  2003/12/21 19:27:02  wachter
 * Added classes for Sender/Receiver over TCP and UDP.
 * ( PLEASE DO NOT USE THIS NOW ! )
 *
 *
 */
