/**
* @file GameControllerUDPHandler.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
#include "GameControllerUDPHandler.h"

#include <stdio.h>
#include <iostream.h>

#include <tools/debugging/debugging.h>
#include "NetConfig.h"


GameControllerUDPHandler::GameControllerUDPHandler() :
UDPEndpoint(2048,2048)
{
    timeLastPackage = 0;
	packagesIgnoredByXabsl = 0;
	roboCupGameControlData = NULL;
}

void  GameControllerUDPHandler::setGameControl(GameControlData &myroboCupGameControlData){
  roboCupGameControlData = &myroboCupGameControlData;
}


int GameControllerUDPHandler::getOpponentTeamColor() {
	if(getPlayer().getTeamColor() == Player::blue)
		return UDP_TEAM_RED;
	else //if(getPlayer().getTeamColor() == Player::red) 
		return UDP_TEAM_BLUE;
	//else return -1;
}

void GameControllerUDPHandler::initGameControllerUDPHandler()
{
    bind(3838); //game controller port
	//roboCupGameControlData->data.state = ROBOCUP_STATE_INITIAL;
}

void GameControllerUDPHandler::onReceive(void* data, int size)
{
//    ASSERT(size == sizeof(DDPPackage));
 
    //INFO(sendDDPInfo,idText,text,"Incoming GameControllerPackage package from " << ipOfLastPackage);	
	
	int packageSize = *(int*)data;
	bool invalidPackage = false;
	bool incompletePackage = false;
	bool wrongGamePackage = false;

	if(packageSize > 0 ) {				
		
		int myTeamColor = -1;

		RoboCupGameControlUDPData* package; 
		if (size != sizeof(RoboCupGameControlUDPData)) {
			incompletePackage = true;

			INFO(sendDDPInfo,idText,text,"Ignoring Incomplete GameControllerPackage package from " << ipOfLastPackage);			
			INFO(sendDDPInfo,idText,text,"   data size=" << size << " but should be " << sizeof(RoboCupGameControlData));
		}
		else {			
			package = static_cast<RoboCupGameControlUDPData*>(data);
		}

		//convert all data from udp -> old gamecontrol stuff					
		// check version
		if (package->version != UDP_STRUCT_VERSION) {
			INFO(sendDDPInfo,idText,text,"OEPS WRONG PACKAGE");
			invalidPackage = true;
		}

		// check header workaround: strcmp doesn't do the job
		for(int i=0; i<4; i++) {
			if(package->header[i] != UDP_STRUCT_HEADER[i]) {
				invalidPackage = true;
				INFO(sendDDPInfo,idText,text,"Header UDP GameController Package is not valid ");
			}
		}

		char teamBlue[200],teamRed[200];
		sprintf(teamBlue,"%d",package->teams[UDP_TEAM_BLUE].teamNumber);
		sprintf(teamRed,"%d",package->teams[UDP_TEAM_RED].teamNumber);

		bool weAreBlue, weAreRed;

		weAreBlue = strcmp(teamBlue, getPlayer().getTeamName())==0;
		weAreRed = strcmp(teamRed, getPlayer().getTeamName())==0;

		// check if package is for this game
		if( !weAreRed && !weAreBlue) {
			wrongGamePackage = true;
			INFO(sendDDPInfo,idText,text,"Alien UDP GameController Package detected from " << ipOfLastPackage);
		}
		
		if(package != NULL && !invalidPackage && !wrongGamePackage && !incompletePackage) {
			//check team color
			if(weAreBlue) {
				roboCupGameControlData->data.teamColor = ROBOCUP_TEAMCOLOR_BLUE;
				myTeamColor = UDP_TEAM_BLUE;
				getPlayer().setTeamColor(Player::blue);
			}
			else if(weAreRed) {
				roboCupGameControlData->data.teamColor = ROBOCUP_TEAMCOLOR_RED;
				myTeamColor = UDP_TEAM_RED;
				getPlayer().setTeamColor(Player::red);				
			}
									
			//override manual teamcolor switch: preference for gamecontroller data
			if(weAreBlue && getPlayer().getTeamColor()== Player::red) {
				roboCupGameControlData->data.teamColor = ROBOCUP_TEAMCOLOR_BLUE;
				myTeamColor = UDP_TEAM_BLUE;
				getPlayer().setTeamColor(Player::blue);
			}
			else if(weAreRed && getPlayer().getTeamColor()== Player::blue) {
				roboCupGameControlData->data.teamColor = ROBOCUP_TEAMCOLOR_RED;
				myTeamColor = UDP_TEAM_RED;
				getPlayer().setTeamColor(Player::red);
			}

			//what's the score?			
			roboCupGameControlData->data.opponentScore = package->teams[getOpponentTeamColor()].score;
			roboCupGameControlData->data.ownScore = package->teams[myTeamColor].score;

			// do we have kickoff?
			switch(package->kickOffTeam) {
				case UDP_TEAM_BLUE : 
					if(myTeamColor == UDP_TEAM_BLUE) {
						roboCupGameControlData->data.kickoff = ROBOCUP_KICKOFF_OWN;
						INFO(sendDDPInfo,idText,text,"KICKOFF to: own team BLUE");
					}
					else if(myTeamColor == UDP_TEAM_RED) {
						roboCupGameControlData->data.kickoff = ROBOCUP_KICKOFF_OPPONENT;
						INFO(sendDDPInfo,idText,text,"KICKOFF: opponent team BLUE (we are RED)");
					}
					break;
				
				case UDP_TEAM_RED : 
					if(myTeamColor == UDP_TEAM_RED) {
						roboCupGameControlData->data.kickoff = ROBOCUP_KICKOFF_OWN;
						INFO(sendDDPInfo,idText,text,"KICKOFF: own team RED");
					}
					else if(myTeamColor == UDP_TEAM_BLUE) {
						roboCupGameControlData->data.kickoff = ROBOCUP_KICKOFF_OPPONENT;
						INFO(sendDDPInfo,idText,text,"KICKOFF: opponent team RED: (we are BLUE)");
					}
					break;																
			}
			
			// game states 
			switch(package->state) {			
				// game states
				case UDP_STATE_INITIAL :
					roboCupGameControlData->data.state = ROBOCUP_STATE_INITIAL; INFO(sendDDPInfo,idText,text,"STATE INITIAL");
					break;
				case UDP_STATE_READY :
					roboCupGameControlData->data.state = ROBOCUP_STATE_READY; INFO(sendDDPInfo,idText,text,"STATE READY");
					break;
				case UDP_STATE_SET :
					roboCupGameControlData->data.state = ROBOCUP_STATE_SET; INFO(sendDDPInfo,idText,text,"STATE SET");
					break;
				case UDP_STATE_PLAYING : 
					if(roboCupGameControlData->data.penalty == PENALTY_NONE) {
						roboCupGameControlData->data.state = ROBOCUP_STATE_PLAYING; INFO(sendDDPInfo,idText,text,"STATE PLAYING");
					}
					break;
				case UDP_STATE_FINISHED :
					roboCupGameControlData->data.state = ROBOCUP_STATE_FINISHED; INFO(sendDDPInfo,idText,text,"STATE FINISHED");
					break;			
			}									
			
			uint32 secsUntilUnPenalized = 0;

			//now check if we get penalized.. :(
			INFO(sendDDPInfo,idText,text,"Player " << getPlayer().getPlayerNumber());

			switch(package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].penalty) {
				// penalties are only legal when in penalized state; so we put aibo in penalized first
				case UDP_PENALTY_NONE :
					//roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					roboCupGameControlData->data.penalty = PENALTY_NONE;
					INFO(sendDDPInfo,idText,text,"PENALTY NONE");
					break;
				case UDP_PENALTY_BALL_HOLDING :
					roboCupGameControlData->data.penalty = PENALTY_BALL_HOLDING;
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					INFO(sendDDPInfo,idText,text,"PENALTY BALL HOLDING");
					break;
				case UDP_PENALTY_GOALIE_PUSHING :
					roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					roboCupGameControlData->data.penalty = PENALTY_KEEPER_CHARGE;
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					INFO(sendDDPInfo,idText,text,"PENALTY GOALIE PUSHING");
					break;
				case UDP_PENALTY_PLAYER_PUSHING :
					roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					roboCupGameControlData->data.penalty = PENALTY_FIELD_PLAYER_CHARGE;
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					INFO(sendDDPInfo,idText,text,"PENALTY PLAYER PUSHING");
					break;
				case UDP_PENALTY_ILLEGAL_DEFENDER :
					roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					roboCupGameControlData->data.penalty = PENALTY_ILLEGAL_DEFENDER;
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					INFO(sendDDPInfo,idText,text,"PENALTY ILLEGAL DEFENDER");
					break;
				case UDP_PENALTY_ILLEGAL_DEFENSE :
					roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					roboCupGameControlData->data.penalty = PENALTY_ILLEGAL_DEFENSE;
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					INFO(sendDDPInfo,idText,text,"PENALTY ILLEGAL DEFENSE");
					break;
				case UDP_PENALTY_OBSTRUCTION :
					roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					roboCupGameControlData->data.penalty = PENALTY_OBSTRUCTION;
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					INFO(sendDDPInfo,idText,text,"PENALTY OBSTRUCTION");
					break;
				case UDP_PENALTY_REQ_FOR_PICKUP :
					roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					roboCupGameControlData->data.penalty = PENALTY_REQ_FOR_PICKUP;
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					INFO(sendDDPInfo,idText,text,"PENALTY REQUEST PICKUP");
					break;
				case UDP_PENALTY_LEAVING : 
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					INFO(sendDDPInfo,idText,text,"PENALTY LEAVING (not implemented yet)");
					break; // TODO implement (needs to be added in behaviour?)
				case UDP_PENALTY_DAMAGE : 
					secsUntilUnPenalized = package->teams[myTeamColor].players[getPlayer().getPlayerNumber()].secsTillUnpenalised;
					roboCupGameControlData->data.state = ROBOCUP_STATE_PENALIZED;
					INFO(sendDDPInfo,idText,text,"PENALTY DAMAGE (not implemented yet)"); 
					break;  // TODO implement (needs to be added in behaviour?)		
			}			
						
			roboCupGameControlData->timeStamp = SystemCall::getCurrentSystemTime();			

		}
	}
	else {
		/*
		if(invalidPackage == true) 
			INFO(sendDDPInfo,idText,text,"invalid UDP package");
		else if(wrongGamePackage == true)
			INFO(sendDDPInfo,idText,text,"wrong UDP game package");
		else if(incompletePackage == true)
			INFO(sendDDPInfo,idText,text,"incomplete UDP package");
		else { 
			INFO(sendDDPInfo,idText,text,"No UDP GameController Packages received..");
		}*/
	}
}
