/**
* @file GameControllerUDPHandler.h
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/** 
* @file  Platform/Aperios1.3.2/DDPHandler.h
*
* Handles the Dog Discovery Protocol (DDP) so that the robot
* is able to find out the ip's of his teammates.
*
* @author <A href=mailto:robocup@m-wachter.de>Michael Wachter</A>
*/

#ifndef	_GameControllerUDPHandler_H_
#define	_GameControllerUDPHandler_H_

#include "UDPEndpoint.h"
#include "Tools/Player.h"
#include "../../Representations/RoboCup/GameControlData.h"
#include "../../Representations/RoboCup/RoboCupGameControlUDPData.h"

/** @class GameControllerUDPPackage
  * This class represents the data package which is send over the network
  */

/*
class GameControllerUDPPackage
{
public:
	RoboCupGameControlUDPData

      char teamIdentifier[16];  
								
      Player::teamColor teamColor;
      Player::playerNumber playerNumber;
};*/


/** @class GameControllerUDPHandler 
  *
  */
class GameControllerUDPHandler : public UDPEndpoint
{
  public:
    GameControllerUDPHandler();
    ~GameControllerUDPHandler() {} ;
    void initGameControllerUDPHandler();  /**< does stuff which can only be done after aperios completly runs */
    virtual void onReceive(void* data, int size);  /**< called by the framework if a package is recived on the DDP-Port */
    void setGameControl(GameControlData &roboCupGameControlData);
	int getOpponentTeamColor();
  public:	  
	  //GameControllerUDPPackage thePackage;
	  RoboCupGameControlUDPData thePackage;
	  GameControlData *roboCupGameControlData;
	  //NetReceiverBase* receiver; /**< the associated receiver for the gamecontroller package **/
   
  private:
    unsigned long timeLastPackage; /**< Time when the last GameControllerPackage was send **/
	uint32 myTeamColor;
	uint32 opponentTeamColor;
	int packagesIgnoredByXabsl;

};


#endif 

/*
 *
 * Changelog :
 *
 * $Log: DDPHandler.h,v $
 * Revision 1.3  2004/09/12 20:10:47  wachter
 * Documentation-fixes
 *
 * Revision 1.2  2004/09/08 12:41:43  wachter
 * - Fixed some warnings
 * - Fixed some documentation in network-code
 *
 * Revision 1.1.1.1  2004/05/22 17:23:25  cvsadm
 * created new repository GT2004_WM
 *
 * Revision 1.3  2004/05/14 14:12:08  wachter
 * - Added communication support for 5 robots
 * - rewrote parts of team-communication to be faster and more stable
 *
 * Revision 1.2  2004/01/09 15:44:30  wachter
 * Worked on with the Dog-Discovery-Protocol
 *
 * Revision 1.1  2003/12/21 19:27:02  wachter
 * Added classes for Sender/Receiver over TCP and UDP.
 * ( PLEASE DO NOT USE THIS NOW ! )
 *
 *
 */
