/** 
* @file  Platform/Aperios1.3.2/IPEndpoint.h
*
* @author <A href=mailto:robocup@m-wachter.de>Michael Wachter</A>
*/

#ifndef	_IPEndpoint_H_
#define	_IPEndpoint_H_

#include <ant.h>
#include <EndpointTypes.h>
#include <IPAddress.h>
#include "ProcessFramework.h"


/**
 * @class IPEndpoint
 * This class is used to encapsule ANT library entpoints. Its used as a base-class for
 * TCPEndpoint and UDPEndpoint.
 */

class IPEndpoint
{
   
   public:
     /** Constructor 
	   * @param sbs The size of the send-buffer. The send-buffer is shared memory between the process
	   *            and Aperios. You can only send data with this size at once.
	   * @param rbs The size of the receive-buffer. Its also shared memory and you can only receive this 
	   *            much data in one call of receiveCont 
	   */
	 IPEndpoint(int sbs = 16384, int rbs = 16384);
     ~IPEndpoint();

     /** called by the processFramework if someone connects to a listen port */
	 virtual void listenCont(antEnvMsg* msg) {};  

     /** called by the processFramework if data was send. */
     virtual void sendCont(antEnvMsg* msg) {};    

	 /** called by the processFramework if new data arrives */
     virtual void receiveCont(antEnvMsg* msg) {}; 

	 /** called by the processFreamwork after a close */
     virtual void closeCont(antEnvMsg* msg) {};   

     /** called by the processFramework after connecting */
     virtual void connectCont(antEnvMsg* msg){};  
    
 
   protected:
     antModuleRef endpoint;                    /**< ANT Library Endpoint */
     antSharedBuffer sendBuffer;               /**< ANT buffer for sending */
     byte*           sharedSendBuffer;         /**< Pointer to the data */
     int             sharedSendBufferSize;     /**< size of the sendBuffer */
     int             sharedReceiveBufferSize;  /**< size of the receiveBuffer */
     
     antSharedBuffer receiveBuffer;            /**< ANT buffer for receiving */
     byte*           sharedReceiveBuffer;      /**< pointer to the data received */

	 //*{
     /**
	  * Pointers to the entry-point selectors in the process-framework 
      * Need to be pointers because an IPEndpoint can be created before
      * the entry-points are created
      */

     int* listenContSelector;
     int* sendContSelector;
     int* receiveContSelector;
     int* closeContSelector;
     int* connectContSelector;
	 //*}

     antStackRef ipStackRef; /**< handle to comminicate to the ip-stack (ANT library */
     OID* myOID_; /**< Object id of this process */
     
};

Out& operator<<(Out& stream, const IPAddress& ipAddress);

#endif // _IPEndpoint_H_

/*
 * Change log :
 * 
 * $Log: IPEndpoint.h,v $
 * Revision 1.2  2004/09/12 20:10:47  wachter
 * Documentation-fixes
 *
 * Revision 1.1.1.1  2004/05/22 17:23:25  cvsadm
 * created new repository GT2004_WM
 *
 * Revision 1.7  2004/01/26 13:44:07  wachter
 * shared-memory-buffers now have variable sizes
 *
 * Revision 1.6  2004/01/09 13:57:23  wachter
 * Addes streaming-operator for Open-R IPAddress
 *
 * Revision 1.5  2004/01/03 16:18:24  wachter
 * debug-communication mostly working now
 *
 * Revision 1.4  2003/12/21 19:27:02  wachter
 * Added classes for Sender/Receiver over TCP and UDP.
 * ( PLEASE DO NOT USE THIS NOW ! )
 *
 * Revision 1.3  2003/12/03 14:21:52  wachter
 * Splitted IPEndpoint.*
 *
 * Revision 1.2  2003/12/02 20:52:41  wachter
 * Added IP-networking to aperios process-framework.
 *
 * Revision 1.1  2003/11/29 23:01:06  wachter
 * idea of an ip-implementation in the process-framework
 *
 */
