/** 
* @file  Platform/Aperios1.3.2/TCPHandler.h
*
* @author <A href=mailto:robocup@m-wachter.de>Michael Wachter</A>
*/

#ifndef	_TCPHandler_H_
#define	_TCPHandler_H_

class TCPHandler;

#include "TCPEndpoint.h"
#include "NetHandler.h"
#include "NetSender.h"
#include "NetReceiver.h"


/**
  * The associated sender first finds out the size of the data to send
  * and streams the data into a memory buffer. After that the sender calls
  * sendPackage in the TCPHandler and the TCPHandler takes care of sending
  * the data to the other host.
  *
  * the TCPHandler first sends the size of the data and after that the data
  * itself.
  *
  * If there is incomining data the TCPHandler first reads the size of the
  * incoming Data and after that all received data is collected in a memory
  * buffer until it is complete. Now the associated receiver is called to
  * stream the data from the buffer.
  */

class TCPHandler : public TCPEndpoint, public NetHandler
{
   public:
     TCPHandler(){}; 
     TCPHandler(NetSenderBase& theSender, NetReceiverBase& theReceiver);
     void setTransmitters(NetSenderBase& theSender, NetReceiverBase& theReceiver);
     virtual void onReceive(void* someData, int size);  /**< receives size, collects all data and calls the receiver after that */
     virtual void onConnect(void);  /**< sets sender active after the connection is established */
     virtual void onClose(int reason); /**< sets sender inactive after disconnecting */
     virtual void onSendingDone(void); /**< informs the sender that it can send again */
     virtual void sendPackage(int senderNumber,void* someData,int size); /**< called by the sender to send data */
     
   private:
     NetReceiverBase* receiver; /**< the associated receiver */
     NetSenderBase* sender; /**< the assiciated sender */

     void* receiveBuffer; /**< buffer where the data is stored until completly collected */
     int sizeOfDataToRead; /**< size of this data */
     bool sendingDone; 
};


#endif 

/*
 *
 * Changelog :
 *
 * $Log: TCPHandler.h,v $
 * Revision 1.2  2004/09/12 20:10:47  wachter
 * Documentation-fixes
 *
 * Revision 1.1.1.1  2004/05/22 17:23:40  cvsadm
 * created new repository GT2004_WM
 *
 * Revision 1.4  2004/01/20 14:21:41  wachter
 * - Added sender-number to NetSender
 * - worked on with Team-Communication
 *
 * Revision 1.3  2004/01/03 18:57:50  wachter
 * Debug-communication working now
 *
 * Revision 1.2  2004/01/03 16:18:25  wachter
 * debug-communication mostly working now
 *
 * Revision 1.1  2003/12/21 19:27:03  wachter
 * Added classes for Sender/Receiver over TCP and UDP.
 * ( PLEASE DO NOT USE THIS NOW ! )
 *
 *
 */
