/**
* @file GoalLinesPercept.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
 * @file GoalLinesPercept.cpp
 *
 * Implementation of class GoalLinesPercept.
 * @author <A href=mailto:juengel@informatik.hu-berlin.de>Matthias Juengel</A>
 */

#include "GoalLinesPercept.h"
#include "Tools/Math/Common.h"

void GoalLinesPercept::reset()
{
   numberOfPoints = 0;
}

void GoalLinesPercept::add(const Vector2<int>& point)
{
  if(numberOfPoints < maxNumberOfPoints){
    count[numberOfPoints]=0;
    points[numberOfPoints] = point;
    numberOfPoints++;
    
  }
}

void GoalLinesPercept::filterPoints(int diff){
  int i,j;
  int tempX,tempY;
  for(i=0;i<numberOfPoints;i++){
    for(j=i+1;j<numberOfPoints;j++){
      tempX=points[i].x-points[j].x;
      tempY=points[i].y-points[j].y;
      if((tempX*tempX+tempY*tempY)<(diff*diff)){
        count[j]=20;
      }
    }
  }
}

void GoalLinesPercept::shakeOut(int numberFrames){
  Vector2<int> TempPoints[maxNumberOfPoints]; /**< The points on lines. */
  int TempCount[maxNumberOfPoints];
  
  int tempNumber=0;
  int i;
    
  for(i=0;i<numberOfPoints;i++){
    if(count[i]<numberFrames){
      TempPoints[tempNumber]=points[i];
      TempCount[tempNumber]=count[i];
      tempNumber++;
    }
  }
  
  for(i=0;i<tempNumber && i<maxNumberOfPoints;i++){
     points[i]=TempPoints[i];
     count[i]=TempCount[i];
  }
  numberOfPoints=tempNumber;
 
}

void GoalLinesPercept::upCount(){
  for(int i=0;i<numberOfPoints;i++){
    count[i]++;
  }
}

In& operator>>(In& stream,GoalLinesPercept& GoalLinesPercept)
{
    stream >> GoalLinesPercept.numberOfPoints;
    for(int j = 0; j < GoalLinesPercept.numberOfPoints; j++)
      stream >> GoalLinesPercept.points[j].x >> GoalLinesPercept.points[j].y;
  
  return stream;
}
 
Out& operator<<(Out& stream, const GoalLinesPercept& GoalLinesPercept)
{
  stream << GoalLinesPercept.numberOfPoints;
    for(int j = 0; j < GoalLinesPercept.numberOfPoints; j++)
      stream << GoalLinesPercept.points[j].x << GoalLinesPercept.points[j].y;
  
    return stream;
}

/*

 */
