/**
* @file GoalLinesPercept.h
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
 * @file GoalLinesPercept.h
 * 
 * Declaration of class GoalLinesPercept
 * @author <A href=mailto:floris@ph.tn.tudelft.nl>Floris Mantz</A>
 */ 

#ifndef __GoalLinesPercept_h_
#define __GoalLinesPercept_h_


#include "Tools/Streams/InOut.h"
#include "Tools/Math/Vector2.h"

/**
* The class represents a percepted line on the field with its type and a Vector of points belonging to it.
*/
class GoalLinesPercept
{
  public:
    
    enum {maxNumberOfPoints = 1000}; /**< Specifies the maximum number of points per line type. */

    Vector2<int> points[maxNumberOfPoints]; /**< The points on lines. */
    int count[maxNumberOfPoints];
    int numberOfPoints; /**< The number of points per line type. */

    /**
     * Constructor.
     */
    GoalLinesPercept() {reset();}

    /**
     * The function empties the line percept.
     */
    void reset();

    //removes all points from numberFrames ago
    void shakeOut(int numberFrames);

    //increases count of every point 
    void upCount();

    //Filters 
    void filterPoints(int diff);


    /**
     * The function adds a new point to the lines percept.
     * @param type The line type of the point.
     * @param point The point.
     */
    void add(const Vector2<int>& point);
};

/**
 * Streaming operator that reads a LinesPercept from a stream.
 * @param stream The stream from which is read.
 * @param linesPercept The LinesPercept object.
 * @return The stream.
 */ 
In& operator>>(In& stream,GoalLinesPercept& goalLinesPercept);
 
/**
 * Streaming operator that writes a LinesPercept to a stream.
 * @param stream The stream to write on.
 * @param linesPercept The LinesPercept object.
 * @return The stream.
 */ 
Out& operator<<(Out& stream, const GoalLinesPercept& goalLinesPercept);


#endif //__GoalLinesPercept_h_

/*
 * Change log :

 */
