/**
* @file SLAMData.h
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
 * @file SLAMData.h
 *
 * The file contains the definition of the class SLAMData.
 *
 * @author <A href=mailto:roefer@tzi.de>Thomas Rfer</A>
 */

#ifndef __SLAMData_h_
#define __SLAMData_h_

#include "Tools/Streams/InOut.h"
#include "Tools/Math/Pose2D.h"
#include "Tools/Math/Vector3.h"
#include "Representations/Perception/PanoramaParameters.h"

/**
 * @class SLAMData
 * SLAM positions and phase information
 */
  
class SLAMData

{
   
  private:
           
  public:

		enum SLAMAction {
			offline,
			resetImageProcessor, 
			resetSelfLocator,
			standby, 

			loadClusters, 
			recordColors, 
			clusterColors, 
			saveClusters,

			clustersAvailable = saveClusters,

			loadPanorama,
			recordPanorama,
			savePanorama,

			localizationAvailable = savePanorama,

			localize,
			normalize
			};

		enum SLAMLocalizationMode {
			odometry,
			trainingSpot,
			probabilistic
			};

		enum SLAMBearingMode {
			unchangedBearing,
			trainingBearing,
			estimateBearing,
			estimateBearingSlow,
			estimateBearingNoDecay,
			setReference0
			};

			static const char* getActionName(int id)
			{
				switch (id)
				{
					case offline: return "offline";
					case resetImageProcessor: return "resetImageProcessor";
					case resetSelfLocator: return "resetSelfLocator";
					case standby: return "standby";

					case loadClusters: return "loadClusters";
					case recordColors: return "recordColors";
					case clusterColors: return "clusterColors";
					case saveClusters: return "saveClusters";

					case loadPanorama: return "loadPanorama";
					case recordPanorama: return "recordPanorama";
					case savePanorama: return "savePanorama";

					case localize: return "localize";
					case normalize: return "normalize";

					default: return "please edit SLAMData::getActionName";
				}
			}


		int slamModeOut;
		int lastSlamModeOut;

		int currentAction;
		int lastAction;

		int learningPoseX;
		int learningPoseY;
		int learningPosePhi;
		int learningPoseNumber;
		
		int localizationMode;
		int bearingMode;

		Pose2D learningPose;

		int actionIteration;

		Vector3<unsigned char> clusterColor[MAX_CLUSTERS];
    /**
     * Constructor.
     */
    SLAMData();

	void requestAction(SLAMAction action);
	void processingAndBusy();
	void actionCompleted(bool success);
	void updateFromXablsOutputSymbols();
};

/**
 * Streaming operator that reads a robot pose from a stream.
 * @param stream The stream from which is read.
 * @param TaskVisionData The robot pose object.
 * @return The stream.
 */ 
In& operator>>(In& stream,SLAMData& SLAMData);
 
/**
 * Streaming operator that writes a robot pose to a stream.
 * @param stream The stream to write on.
 * @param TaskVisionData The robot pose object.
 * @return The stream.
 */ 
Out& operator<<(Out& stream, const SLAMData& SLAMData);


#endif //__SLAMData_h_

