/**
* @file TaskVisionData.h
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
 * @file TaskVisionData.h
 *
 * The file contains the definition of the class TaskVisionData.
 *
 * @author <A href=mailto:roefer@tzi.de>Thomas Rfer</A>
 */

#ifndef __TaskVisionData_h_
#define __TaskVisionData_h_

#include "Tools/Streams/InOut.h"
#include "Tools/Math/Pose2D.h"



/**
 * @class TaskVisionData
 * A Pose2D with validity.
 */
  
class TaskVisionData

{
   
  private:
    
    double validityTask; /**< The validity of the robot pose. */
   
    int nrLines;
    double straal[2];
    double hoek[2];
    int type[2];
           
  public:
    int loadColorTable;
    int taskSolution1;

    enum{
      DT2004=0,
      POSITION=1,
      CLEAR=2,
      FIND_ORIENTATION=3,
      RETURN_GOAL=4,
      ODOMETRY=5,
      STRIKER=6};
    
    /**
     * Constructor.
     */
    TaskVisionData() {validityTask=0.2; taskSolution1=DT2004;}

       const TaskVisionData& operator=(const TaskVisionData& other)
    {
      validityTask = other.validityTask;
      return *this;
    }

         
    void setValidityTask(const double v) {validityTask = v;}
    
    void setHoek(const double Theta, int i) { hoek[i] = Theta;}
    void setStraal(const double Straal, int i){ straal[i]= Straal;}
    void setType(const int T, int i){ type[i] = T;}
    void setNrLines(const int nr) { nrLines = nr;} 
        
    void setTaskSolution1(int t){ taskSolution1=t;}

       /**
     * The function returns the validity of the current pose.
     * @return The validity.
     */
    const double& getValidityTask() const {return validityTask;}
    const double& getStraal(int i) const {return straal[i];}
    const double& getHoek(int i) const { return hoek[i];}
    const int& getType(int i) const { return type[i];}
    const int& getNrLines() const { return nrLines;}

};

/**
 * Streaming operator that reads a robot pose from a stream.
 * @param stream The stream from which is read.
 * @param TaskVisionData The robot pose object.
 * @return The stream.
 */ 
In& operator>>(In& stream,TaskVisionData& TaskVisionData);
 
/**
 * Streaming operator that writes a robot pose to a stream.
 * @param stream The stream to write on.
 * @param TaskVisionData The robot pose object.
 * @return The stream.
 */ 
Out& operator<<(Out& stream, const TaskVisionData& TaskVisionData);


#endif //__TaskVisionData_h_

