/**
* @file RoboCupGameControlUDPData.h
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/*
    Copyright (C) 2005  University Of New South Wales

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software source code file, to deal in the file without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the file, and to
    permit persons to whom the file is furnished to do so, subject to the
    following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the software source file.

    THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.
*/

/*******************************************************************************
*
* RoboCupGameControlUDPData.h
*
* Broadcasted data structure and constants
*
* willu@cse.unsw.edu.au
* shnl327@cse.unsw.edu.au
*
*******************************************************************************/

#ifndef _RoboCupGameControlUDPData_h_DEFINED
#define _RoboCupGameControlUDPData_h_DEFINED

// data structure version number
#define UDP_STRUCT_VERSION              4

// data structure header
#define UDP_STRUCT_HEADER               "RGme"

// team numbers
#define UDP_TEAM_BLUE                   0
#define UDP_TEAM_RED                    1

// game states
#define UDP_STATE_INITIAL               0
#define UDP_STATE_READY                 1
#define UDP_STATE_SET                   2
#define UDP_STATE_PLAYING               3
#define UDP_STATE_FINISHED              4

// penalties
#define UDP_PENALTY_NONE                0
#define UDP_PENALTY_BALL_HOLDING        1
#define UDP_PENALTY_GOALIE_PUSHING      2
#define UDP_PENALTY_PLAYER_PUSHING      3
#define UDP_PENALTY_ILLEGAL_DEFENDER    4
#define UDP_PENALTY_ILLEGAL_DEFENSE     5
#define UDP_PENALTY_OBSTRUCTION         6
#define UDP_PENALTY_REQ_FOR_PICKUP      7
#define UDP_PENALTY_LEAVING             8
#define UDP_PENALTY_DAMAGE              9    //NEW



// information that describes a player
struct RobotInfo {
    uint32 penalty;             // penalty state of the player
    uint32 secsTillUnpenalised; // estimate of time till unpenalised
};


// information that describes a team
struct TeamInfo {
    uint32 teamNumber;          // unique team number
    uint32 teamColour;          // colour of the team
    uint32 score;               // team's score
    RobotInfo players[4];       // the team's players
};


struct RoboCupGameControlUDPData {
    char   header[4];           // header to identify the structure
    uint32 version;             // version of the data structure
    uint32 state;               // state of the game
    uint32 firstHalf;           // 1 = game in first half, 0 otherwise
    uint32 kickOffTeam;         // the next team to kick off
    uint32 secsRemaining;       // estimate of number of seconds remaining in the half
    uint32 dropInTeam;          // team that caused last drop in
    uint32 dropInTime;          // number of seconds passed since the last drop inW
    TeamInfo teams[2];
};

#endif

