/**
* @file ColorTable64DlgBar.cpp
*
* File modified by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file ColorTable64DlgBar.cpp
*
* Implementation of class CColorTable64DlgBar
*
* @author <a href="mailto:juengel@informatik.hu-berlin.de">Matthias Jngel</a>
*/

#include "StdAfx.h"
#include "ColorTable64DlgBar.h"

#include "Visualization/DebugDrawingManager/DebugDrawingManager.h"
#include "Visualization/ImageMethods.h"
#include "Visualization/PaintMethodsWin32.h"

#include "RobotControlQueues.h"
#define HAVE_BOOLEAN
#include "Representations/Perception/JPEGImage.h"
#include "Platform/Sensors.h"
#include "Tools/Location.h"
#include "Modules/ImageProcessor/ImageProcessorTools/ColorCorrector.h"

CColorTable64DlgBar::CColorTable64DlgBar()
: CRobotControlDialogBar(IDD)
{
  //{{AFX_DATA_INIT(CColorTable64DlgBar)
  //}}AFX_DATA_INIT
}

CColorTable64DlgBar::~CColorTable64DlgBar()
{
}

bool CColorTable64DlgBar::handleMessage(InMessage& message)
{
  if (message.getMessageID() == idImage || message.getMessageID() == idJPEGImage)
  {
    if (debugImageID == Images::rawImage)
    {
      if(message.getMessageID() == idImage)
        message.bin >> rawImage[0];
      else
      {
        JPEGImage jpeg;
        message.bin >> jpeg;
        jpeg.toImage(rawImage[0]);
      }

	  //color correction, added by jurgo
	  if(rawImage[0].cameraInfo.resolutionWidth!=176) {//if not simulated
		Image correctedImage(rawImage[0]);
		ColorCorrector::correct(correctedImage);
		rawImage[0] = correctedImage;
	  }

      yPosSegmentedImage = rawImage[0].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1) + 1;
      for(int i = 0; i < 5; i++)
        xPosSegmentedImage[i] = xPosRawImage[i] = 120 + i * (rawImage[0].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) + 2);

	  if(m_recordCheck.GetCheck()) OnRecordNewImage();

      if (this->IsWindowVisible())
      {
        generateSegmentedImage();
        RedrawWindow(NULL, NULL, RDW_INVALIDATE);
        if ( m_autoremove.GetCheck() ) this->removeColorClassFromImage();
        if ( m_automedian.GetCheck() ) this->smoothImage();
      }


    }
    return true;
  }
  else if (message.getMessageID() == idDebugImage)
  {
    int id;
    message.bin >> id;
    if (id == debugImageID)
    {
      message.bin >> rawImage[0];

	  //color correction, added by jurgo
	  if(rawImage[0].cameraInfo.resolutionWidth!=176) {
	    Image correctedImage(rawImage[0]);
		ColorCorrector::correct(correctedImage);
		rawImage[0] = correctedImage;
	  }

	  
      yPosSegmentedImage = rawImage[0].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1) + 1;
      for(int i = 0; i < 5; i++)
        xPosSegmentedImage[i] = xPosRawImage[i] = 120 + i * (rawImage[0].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) + 2);

	  if(m_recordCheck.GetCheck()) OnRecordNewImage();

      if (this->IsWindowVisible())
      {
        generateSegmentedImage();
        RedrawWindow(NULL, NULL, RDW_INVALIDATE);
        if ( m_autoremove.GetCheck() ) this->removeColorClassFromImage();
        if ( m_automedian.GetCheck() ) this->smoothImage();
      }

    }
    return true;
  }
  else 
    return false;
}

void CColorTable64DlgBar::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CColorTable64DlgBar)
	DDX_Control(pDX, IDC_CT64_SELECTED_COLOR_ONLY, m_selectedColorOnlyCheck);
  DDX_Control(pDX, IDC_CT64_SEND_TO_ROBOT, m_sendButton);
  DDX_Control(pDX, IDC_CT64_RANGE_SLIDER, m_rangeSlider);
  DDX_Control(pDX, IDC_CT64_RANGE_STATIC, m_rangeStatic);
  DDX_Control(pDX, IDC_CT64_PEN_SLIDER, m_penSlider);
  DDX_Control(pDX, IDC_CT64_PEN_STATIC, m_penStatic);
  DDX_Control(pDX, IDC_CT64_COLOR_CLASS_COMBO, m_ct64ColorClassCombo);
	DDX_Control(pDX, IDC_CT64_AUTO_MEDIAN, m_automedian);
	DDX_Control(pDX, IDC_CT64_AUTO_REMOVE, m_autoremove);
	DDX_Control(pDX, IDC_CT64_MEDIAN_ALL_COLORS, m_medianAllColors);
	DDX_Control(pDX, IDC_CT64_MEDIAN_ALLOW_REASSIGN, m_medianAllowReassign);
  DDX_Control(pDX, IDC_CT64_MEDIAN_RANGE_SLIDER, m_medianRangeSlider);
  DDX_Control(pDX, IDC_CT64_MEDIAN_RANGE_STATIC, m_medianRangeStatic);
  DDX_Control(pDX, IDC_CT64_MEDIAN_DISTANCE_SLIDER, m_medianDistanceSlider);
  DDX_Control(pDX, IDC_CT64_MEDIAN_DISTANCE_STATIC, m_medianDistanceStatic);
  DDX_Control(pDX, IDC_CT64_MEDIAN_NEIGHBOUR_SLIDER, m_medianNeighbourSlider);
  DDX_Control(pDX, IDC_CT64_MEDIAN_NEIGHBOUR_STATIC, m_medianNeighbourStatic);
  DDX_Control(pDX, IDC_CT64_IMAGE_COMBO, m_ct64ImageCombo);
  DDX_Control(pDX, IDC_CT64_HIGH_RES, m_highRes);

  DDX_Control(pDX, IDC_CT64_RECORD_CHECK, m_recordCheck);
  DDX_Control(pDX, IDC_CT64_RECORD_AUTO_COLOR, m_recordAutoColor);
  DDX_Control(pDX, IDC_CT64_RECORD_DISTANCE, m_recordDistance);
  DDX_Control(pDX, IDC_CT64_RECORD_EXTENDED, m_recordExtended);

  //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CColorTable64DlgBar, CDynamicBarDlg)
//{{AFX_MSG_MAP(CColorTable64DlgBar)
ON_WM_PAINT()
ON_WM_SIZE()
ON_BN_CLICKED(IDC_CT64_CLEAR_COLOR_TABLE, OnCt64ClearColorTable)
ON_BN_CLICKED(IDC_CT64_CLEAR_CHANNEL, OnCt64ClearChannel)
ON_BN_CLICKED(IDC_CT64_OPEN, OnCt64Open)
ON_BN_CLICKED(IDC_CT64_SAVE, OnCt64Save)
ON_CBN_SELCHANGE(IDC_CT64_COLOR_CLASS_COMBO, OnSelchangeCt64ColorClassCombo)
ON_WM_LBUTTONDOWN()
ON_WM_RBUTTONDOWN()
ON_WM_HSCROLL()
ON_BN_CLICKED(IDC_CT64_UNDO, OnCt64Undo)
ON_BN_CLICKED(IDC_CT64_SEND_TO_ROBOT, OnCt64SendToRobot)
ON_BN_CLICKED(IDC_CT64_REMOVE, OnCt64Remove)
ON_BN_CLICKED(IDC_CT64_MEDIAN, OnCt64Median)
	ON_BN_CLICKED(IDC_CT64_CLEAR_CHANNEL_IN_IMAGE, OnClearChannelInImage)
	ON_BN_CLICKED(IDC_CT64_SELECTED_COLOR_ONLY, OnSelectedColorOnly)
ON_WM_TIMER()
ON_BN_CLICKED(IDC_CT64_SEND_TO_ROBOT, OnCt64SendToRobot)
ON_CBN_SELCHANGE(IDC_CT64_IMAGE_COMBO, OnSelchangeCt64ImageCombo)
ON_WM_MOUSEMOVE()
ON_BN_CLICKED(IDC_CT64_CAPTURE_1, OnCapture1)
ON_BN_CLICKED(IDC_CT64_CAPTURE_2, OnCapture2)
ON_BN_CLICKED(IDC_CT64_CAPTURE_3, OnCapture3)
ON_BN_CLICKED(IDC_CT64_CAPTURE_4, OnCapture4)
ON_BN_CLICKED(IDC_CT64_HIGH_RES, OnHighRes)

ON_BN_CLICKED(IDC_CT64_RECORD_RESET, OnRecordReset)
ON_BN_CLICKED(IDC_CT64_RECORD_CHECK, OnRecordCheck)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CColorTable64DlgBar::sendLocal()
{
  getQueues().toSimulated.allRobots.out.bin << colorTable;
  getQueues().toSimulated.allRobots.out.finishMessage(idColorTable64);

  getQueues().toGUI.out.bin << colorTable;
  getQueues().toGUI.out.finishMessage(idColorTable64);
}

void CColorTable64DlgBar::OnCt64SendToRobot() 
{
  getQueues().toPhysical.allRobots.out.bin << colorTable;
  getQueues().toPhysical.allRobots.out.finishMessage(idColorTable64);
}

BOOL CColorTable64DlgBar::OnInitDialog() 
{
  CDynamicBarDlg::OnInitDialog();

  maxGeneratedSegmentedImages = 5;
  
  selectedColorClass = orange;
  showSelectedColorClassOnly = false;
  // initialize the entries in the color selector
  int i;
  for(i = 0; i < numOfColors; i++)
  {
    colorClass currentColorClass = (colorClass)i;
    
    m_ct64ColorClassCombo.AddString(
      ColorClasses::getColorName(currentColorClass)
      );
  }
  m_ct64ColorClassCombo.SetCurSel(1);

  m_highRes.SetCheck(AfxGetApp()->GetProfileInt("ColorTable64Dlg", "HighRes", 0));
  useHighRes = (m_highRes.GetCheck() == 1);

  //initialize image combo
  debugImageID = (Images::ImageID)AfxGetApp()->GetProfileInt("ColorTable64Dlg", "DebugImageID", Images::rawImage);
  for(i = 1; i < Images::numberOfImageIDs; i++)
  {
    m_ct64ImageCombo.AddString(Images::getImageIDName((Images::ImageID)i));
  }
  m_ct64ImageCombo.SetCurSel(debugImageID - 1);
  
  // write something to the color table
  InBinaryFile fin(getLocation().getModelFilename("coltable.c64"));
  if (fin.exists())
  {
    fin >> colorTable;
  }
  else
  {
    colorTable.clear();
  }
  
  previousColorTable=colorTable; // normally obsolete init for undo
  undoColorTablePossible=FALSE; // this is a new color table
 
  repaintRawImage = true;
  repaintSegmentedImage = true;
  
  yPosRawImage = 0;
  
  yPosSegmentedImage = rawImage[0].cameraInfo.resolutionHeight + 1;
  
  for(i = 0; i < 5; i++)
  {
    ImageMethods::writeRobotControlLogoToImage(rawImage[i]);
    xPosSegmentedImage[i] = xPosRawImage[i] = 120 + i * (rawImage[0].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) + 2);
    imageCaptured[i] = (i == 0);
  }

  generateSegmentedImage();
  
  m_rangeSlider.SetRange(0, 30);
  setClickRangeSlider(1);
  m_penSlider.SetRange(0, 5);
  setPenSlider(0);
  
  m_medianRangeSlider.SetRange(0, 30);
  setMedianRangeSlider(0);
  m_medianDistanceSlider.SetRange(0, 30);
  setMedianDistanceSlider(2);
  m_medianNeighbourSlider.SetRange(3, 8);
  setMedianNeighbourSlider(7);

  for (i = 0; i < 5; i++)
  {
    xPosPreviewPixel[i] = 0;
    yPosPreviewPixel[i] = 0;
  }

  OnRecordReset();
  m_recordDistance.SetRange(1, 50);
  m_recordDistance.SetPos(20);
  m_recordCheck.SetCheck(1);
  maxGeneratedSegmentedImages = 1;
  m_recordAutoColor.SetCheck(1);

  return TRUE;
}

void CColorTable64DlgBar::OnPaint() 
{
  CPaintDC dc(this); // device context for painting
  
  for (int i = 0; i < 5; i++)
    if (imageCaptured[i])
    {
      ImageMethods::paintImage2CDCAsYUV(
        dc, rawImage[i],
        CRect(xPosRawImage[i], yPosRawImage, xPosRawImage[i] + rawImage[i].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1), yPosRawImage + rawImage[i].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1)), 
        useHighRes );
      ImageMethods::paintColorClassImage2CDC(
        dc, 
        segmentedImage[i], 
        CRect(xPosSegmentedImage[i], yPosSegmentedImage, xPosSegmentedImage[i] + segmentedImage[i].width, yPosSegmentedImage + segmentedImage[i].height) );

      // paint preview pixel
      RECT rect;
      rect.top = yPosSegmentedImage + segmentedImage[i].height + 1; //rect.bottom + 10;
      rect.left = xPosSegmentedImage[i];
      rect.bottom = rect.top + 50;
      rect.right = rect.left + 50;
      int Y,U,V;
      if (useHighRes)
      {
        Y = rawImage[i].getHighResY(xPosPreviewPixel[i], yPosPreviewPixel[i]);
        U = rawImage[i].image[yPosPreviewPixel[i] / 2][1][xPosPreviewPixel[i] / 2] - 128;
        V = rawImage[i].image[yPosPreviewPixel[i] / 2][2][xPosPreviewPixel[i] / 2] - 128;
      } else {
        Y = rawImage[i].image[yPosPreviewPixel[i]][0][xPosPreviewPixel[i]];
        U = rawImage[i].image[yPosPreviewPixel[i]][1][xPosPreviewPixel[i]] - 128;
        V = rawImage[i].image[yPosPreviewPixel[i]][2][xPosPreviewPixel[i]] - 128;
      }
      int B = (int)(Y + 1.140 * V);
      int G = (int)(Y - 0.394 * U - 0.581 * V);
      int R = (int)(Y + 2.028 * U);
      if(R < 0) R = 0; if(R > 255) R = 255;
      if(G < 0) G = 0; if(G > 255) G = 255;
      if(B < 0) B = 0; if(B > 255) B = 255;
      COLORREF color = RGB(R,G,B);
      dc.FillSolidRect(&rect, color);
      rect.left = rect.right + 1;
      rect.right = rect.left + 50;
      color = ColorClasses::colorClassToRGB((colorClass)segmentedImage[i].image[yPosPreviewPixel[i]][xPosPreviewPixel[i]]);
      dc.FillSolidRect(&rect, color);
    }
}

void CColorTable64DlgBar::OnSize(UINT nType, int cx, int cy) 
{
}

void CColorTable64DlgBar::OnCt64Undo() 
{
  if (this->undoColorTablePossible) 
  {
    this->colorTable=this->previousColorTable;
    this->undoColorTablePossible=FALSE;
    generateSegmentedImage();
    
    sendLocal();
    
    RedrawWindow(NULL, NULL, RDW_INVALIDATE);
  }
  if(this->undoRecordPossible) {
	  this->undoRecordPossible = false;
	  memcpy(record,backupRecord,sizeof(record));
  }
}


void CColorTable64DlgBar::OnCt64ClearColorTable() 
{
  this->previousColorTable=this->colorTable;
  this->undoColorTablePossible=TRUE;
  this->colorTable.clear();
  generateSegmentedImage();
  
  sendLocal();
  
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnCt64ClearChannel() 
{
  this->previousColorTable=this->colorTable;
  this->undoColorTablePossible=TRUE;
  this->colorTable.clearChannel(selectedColorClass);
  generateSegmentedImage();
  
  sendLocal();
  
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnCt64Open() 
{
  CString defaultPath = File::getGTDir();
  defaultPath += "/Config/";
  defaultPath.Replace('/','\\');
  CString pathName = 
    AfxGetApp()->GetProfileString("ColorTable64Dlg", "openPath", defaultPath);
  pathName += "*.c64";

  CFileDialog fileDialog(true, ".c64",pathName,
    OFN_HIDEREADONLY | OFN_EXPLORER | OFN_ENABLESIZING | OFN_NOCHANGEDIR | OFN_NONETWORKBUTTON
    , "color tables (*.c64)|*.c64|color matrix (*.matrix)|*.matrix|cmu color tables (*.tm)|*.tm|unsw color tables (*.cal)|*.cal||", this);

  if (fileDialog.DoModal()==IDOK)
  {
    CString fileName = fileDialog.GetFileName();
    CString pathAndFileName = fileDialog.GetPathName();
    pathName = 
      pathAndFileName.Left(
      pathAndFileName.GetLength() -
      fileName.GetLength()
      );
    AfxGetApp()->WriteProfileString("ColorTable64Dlg", "openPath", pathName);
  
    InBinaryFile fin(pathAndFileName);
    if (fin.exists())
    {
      if (this->undoColorTablePossible) // there have been actions
      {
        this->previousColorTable=this->colorTable;
      }
      if ( fileDialog.GetFileExt() == "c64" )
      {
	      fin >> colorTable;
		  }
      else if ( fileDialog.GetFileExt() == "matrix" )
      {
        // create jurgen color table matrix file (color class count per color class instead of color class per cube)
        for ( int y = 0; y < 64; y ++ ) {
          for ( int u = 0; u < 64; u ++ ) {
            for ( int v = 0; v < 64; v ++ ) {
				for(int c = 0; c<numOfColors;c++) {
					fin >> record[y][u][v][c];
				}
            }
          }
        }
      }
      else if ( fileDialog.GetFileExt() == "tm" )
      {
        // read cmu colortable - Y 4bit, U 6bit, V 6bit
        char header[19];
			  fin.read(&header, 19);
        colorClass colorclass;
        char idx;
        for ( int y = 0; y < 256; y = y + 16 ) {
          for ( int u = 0; u < 256; u = u + 4 ) {
            for ( int v = 0; v < 256; v = v + 4 ) {
              fin >> idx;
              switch ( idx ) {
                case 0:
                  colorclass = noColor;
                  break;
                case 1:
                  colorclass = orange;
                  break;
                case 2:
                  colorclass = green;
                  break;
                case 3:
                  colorclass = pink;
                  break;
                case 4:
                  colorclass = skyblue;
                  break;
                case 5:
                  colorclass = yellow;
                  break;
                case 6:
                  colorclass = blue;
                  break;
                case 7:
                  colorclass = red;
                  break;
                case 8:
                  colorclass = white;
                  break;
                default:
                  colorclass = noColor;
                  break;
						  }
              colorTable.addCuboidToColorClass(colorclass, y, u, v, y+15, u+3, v+3);
            }
          }
        }
		  }
      else if ( fileDialog.GetFileExt() == "cal" )
      {
        // read unsw colortable - Y 7bit, U 7bit, V 7bit
        colorClass colorclass;
        char idx;
        for ( int y = 0; y < 256; y = y + 2 ) {
          for ( int u = 0; u < 256; u = u + 2 ) {
            for ( int v = 0; v < 256; v = v + 2 ) {
              fin >> idx;
              switch ( idx ) {
                case 0:
                  colorclass = orange;
                  break;
                case 1:
                  colorclass = skyblue;
                  break;
                case 2: // green landmark
                  colorclass = green;
                  break;
                case 3:
                  colorclass = yellow;
                  break;
                case 4:
                  colorclass = pink;
                  break;
                case 5:
                  colorclass = blue;
                  break;
                case 6:
                  colorclass = red;
                  break;
                case 7: // green floor
                  colorclass = green;
                  break;
							  case 8: // white and nocolor
                  colorclass = white;
                  break;
              }
              colorTable.addCuboidToColorClass(colorclass, y, u, v, y+1, u+1, v+1);
            }
          }
        }
      }
      else
      {
        CString message;
        message.Format( "Unknown file-extension %s.", fileDialog.GetFileExt());
        AfxMessageBox( message, MB_OK);
      }
    }
    else
    {
      CString message;
      message.Format( "File %s not found.", pathAndFileName);
      AfxMessageBox( message, MB_OK);
    }
  }
  generateSegmentedImage();

  sendLocal();

  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnCt64Save() 
{
  CString defaultPath = File::getGTDir();
  defaultPath += "/Deployment/Colortables/";
  defaultPath.Replace('/','\\');
  CString pathName = 
    AfxGetApp()->GetProfileString("ColorTable64Dlg", "savePath", defaultPath);
  pathName += "*.c64";

  CFileDialog fileDialog(false, ".c64",pathName,
    OFN_HIDEREADONLY | OFN_EXPLORER | OFN_ENABLESIZING | OFN_NOCHANGEDIR | OFN_NONETWORKBUTTON
    , "color tables (*.c64)|*.c64|color matrix (*.matrix)|*.matrix|cmu color tables (*.tm)|*.tm|unsw color tables (*.cal)|*.cal||", this);

  if (fileDialog.DoModal()==IDOK)
  {
    CString fileName = fileDialog.GetFileName();
    CString pathAndFileName = fileDialog.GetPathName();
    CString pathName = 
      pathAndFileName.Left(
      pathAndFileName.GetLength() -
      fileName.GetLength()
      );
    AfxGetApp()->WriteProfileString("ColorTable64Dlg", "savePath", pathName);
  
    OutBinaryFile fout(pathAndFileName);
    if (fout.exists())
    {
      if ( fileDialog.GetFileExt() == "c64" )
      {
        fout << colorTable;
      }
      else if ( fileDialog.GetFileExt() == "tm" )
      {
        // create cmu colortable - Y 4bit, U 6bit, V 6bit
        char* header;
        header = "TMAP\nYUV8\n16 64 64\n";
        fout.write ( header, 19 );
        colorClass colorclass;
        char idx;
        for ( int y = 0; y < 256; y = y + 16 ) {
          for ( int u = 0; u < 256; u = u + 4 ) {
            for ( int v = 0; v < 256; v = v + 4 ) {
              colorclass = (colorClass) colorTable.getColorClass ( y, u, v );
              switch ( colorclass ) {
                case noColor:
                  idx = (char) 0;
                  break;
                case orange:
                  idx = (char) 1;
                  break;
                case green:
                  idx = (char) 2;
                  break;
                case pink:
                  idx = (char) 3;
                  break;
                case skyblue:
                  idx = (char) 4;
                  break;
                case yellow:
                  idx = (char) 5;
                  break;
                case blue:
                  idx = (char) 6;
                  break;
                case red:
                  idx = (char) 7;
                  break;
                case white:
                  idx = (char) 8;
                  break;
                default:
                  idx = (char) 0;
                  break;
              }
              fout << idx;
            }
          }
        }
      }
      else if ( fileDialog.GetFileExt() == "matrix" )
      {
        // create jurgen color table matrix file (color class count per color class instead of color class per cube)
        for ( int y = 0; y < 64; y ++ ) {
          for ( int u = 0; u < 64; u ++ ) {
            for ( int v = 0; v < 64; v ++ ) {
				for(int c = 0; c<numOfColors;c++) {
					fout << record[y][u][v][c];
				}
            }
          }
        }
      }
      else if ( fileDialog.GetFileExt() == "cal" )
      {
        // create unsw colortable - Y 7bit, U 7bit, V 7bit
        colorClass colorclass;
        char idx;
        for ( int y = 0; y < 256; y = y + 2 ) {
          for ( int u = 0; u < 256; u = u + 2 ) {
            for ( int v = 0; v < 256; v = v + 2 ) {
              colorclass = (colorClass) colorTable.getColorClass ( y, u, v );
              switch ( colorclass ) {
                case orange:
                  idx = (char) 0;
                  break;
                case skyblue:
                  idx = (char) 1;
                  break;
			  // green landmark = 2
                case yellow:
                  idx = (char) 3;
                  break;
                case pink:
                  idx = (char) 4;
                  break;
                case blue:
                  idx = (char) 5;
                  break;
                case red:
                  idx = (char) 6;
                  break;
                case green:
                  idx = (char) 7;
                  break;
                case white:
                case noColor:
                default:
                  idx = (char) 8;
                  break;
              }
              fout << idx;
            }
          }
        }
      }
      else
      {
        CString message;
        message.Format( "Unknown file-extension %s.", fileDialog.GetFileExt());
        AfxMessageBox( message, MB_OK);
      }
    }
    else
    {
      CString message;
      message.Format( "File %s not found.", pathAndFileName);
      AfxMessageBox( message, MB_OK);
    }
  }
}

void CColorTable64DlgBar::OnSelchangeCt64ColorClassCombo() 
{
  selectedColorClass = (colorClass)m_ct64ColorClassCombo.GetCurSel();
  generateSegmentedImage();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnSelchangeCt64ImageCombo() 
{
  debugImageID = m_ct64ImageCombo.GetCurSel() + 1;
  AfxGetApp()->WriteProfileInt("ColorTable64Dlg", "DebugImageID", debugImageID);
}

void CColorTable64DlgBar::OnLButtonDown(UINT nFlags, CPoint point) 
{
  bool somethingWasSelected = false;
  
  int selectedX = 0;
  int selectedY = 0;
  
  for (int c = 0; c < 5; c++)
    if (imageCaptured[c])
    {
      //is the selected Point in the raw image?
      if(
        point.x >= xPosRawImage[c] &&
        point.x < xPosRawImage[c] + rawImage[c].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) &&
        point.y >= yPosRawImage &&
        point.y < yPosRawImage + rawImage[c].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1)
        )
      {
        somethingWasSelected = true;
        selectedX = point.x - xPosRawImage[c];
        selectedY = point.y - yPosRawImage;
      }
  
      //is the selected Point in the segmented image?
      if(
        point.x >= xPosSegmentedImage[c] &&
        point.x < xPosSegmentedImage[c] + rawImage[c].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) &&
        point.y >= yPosSegmentedImage &&
        point.y < yPosSegmentedImage + rawImage[c].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1)
        )
      {
        somethingWasSelected = true;
        selectedX = point.x - xPosSegmentedImage[c];
        selectedY = point.y - yPosSegmentedImage;
      }
  
	  
	  if(m_recordCheck.GetCheck()&&somethingWasSelected) {
		 OnRecordClick(true,selectedX,selectedY,c);
	  } else 
      if (somethingWasSelected)
      {
        this->previousColorTable=this->colorTable;
        this->undoColorTablePossible=TRUE;
    
        for ( int i = selectedY - penSize; i <= selectedY + penSize; i++ ) {
          for ( int j = selectedX - penSize; j <= selectedX + penSize; j++ ) {
            if ( i >= 0 && 
                 i < rawImage[c].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1) && 
                 j >= 0 && 
                 j < rawImage[c].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) ) 
            {
              this->colorTable.addColorClass(
                selectedColorClass,
                (useHighRes ? rawImage[c].getHighResY(j, i) : rawImage[c].image[i][0][j]),
                rawImage[c].image[i / (useHighRes ? 2 : 1)][1][j / (useHighRes ? 2 : 1)],
                rawImage[c].image[i / (useHighRes ? 2 : 1)][2][j / (useHighRes ? 2 : 1)],
                clickRange
              );
            }
          }
        }
        generateSegmentedImage();
    
        sendLocal();

        RedrawWindow(NULL, NULL, RDW_INVALIDATE);
        break;
      }
    }
  
  CDynamicBarDlg::OnLButtonDown(nFlags, point);
}

void CColorTable64DlgBar::OnRButtonDown(UINT nFlags, CPoint point) 
{
  bool somethingWasSelected = false;
  
  int selectedX = 0;
  int selectedY = 0;
  
  for (int c = 0; c < 5; c++)
    if (imageCaptured[c])
    {
      //is the selected Point in the raw image?
      if(
        point.x >= xPosRawImage[c] &&
        point.x < xPosRawImage[c] + rawImage[c].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) &&
        point.y >= yPosRawImage &&
        point.y < yPosRawImage + rawImage[c].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1)
        )
      {
        somethingWasSelected = true;
        selectedX = point.x - xPosRawImage[c];
        selectedY = point.y - yPosRawImage;
      }
  
      //is the selected Point in the segmented image?
      if(
        point.x >= xPosSegmentedImage[c] &&
        point.x < xPosSegmentedImage[c] + rawImage[c].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) &&
        point.y >= yPosSegmentedImage &&
        point.y < yPosSegmentedImage + rawImage[c].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1)
        )
      {
        somethingWasSelected = true;
        selectedX = point.x - xPosSegmentedImage[c];
        selectedY = point.y - yPosSegmentedImage;
      }
  
	  if(m_recordCheck.GetCheck()&&somethingWasSelected) {
		 OnRecordClick(false,selectedX,selectedY,c);
	  } else 
      if (somethingWasSelected)
      {
        this->previousColorTable=this->colorTable;
        this->undoColorTablePossible=TRUE;
    
        for ( int i = selectedY - penSize; i <= selectedY + penSize; i++ ) {
          for ( int j = selectedX - penSize; j <= selectedX + penSize; j++ ) {
            if ( i >= 0 && 
                 i < rawImage[c].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1) && 
                 j >= 0 && 
                 j < rawImage[c].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) &&
                 (!showSelectedColorClassOnly || (colorClass)this->segmentedImage[0].image[i][j] == selectedColorClass))
            {
              this->colorTable.removeColorClass(
                (useHighRes ? rawImage[c].getHighResY(j, i) : rawImage[c].image[i][0][j]),
                rawImage[c].image[i / (useHighRes ? 2 : 1)][1][j / (useHighRes ? 2 : 1)],
                rawImage[c].image[i / (useHighRes ? 2 : 1)][2][j / (useHighRes ? 2 : 1)],
                clickRange
              );
            }
          }
        }
    
        generateSegmentedImage();
    
        sendLocal();
    
        RedrawWindow(NULL, NULL, RDW_INVALIDATE);
        break;
      }
    }
  
  CDynamicBarDlg::OnRButtonDown(nFlags, point);
}

void CColorTable64DlgBar::OnMouseMove(UINT nFlags, CPoint point) 
{
  bool somethingWasSelected = false;
  
  int selectedX = 0;
  int selectedY = 0;
  
  for (int c = 0; c < 5; c++)
    if (imageCaptured[c])
    {
      //is the selected Point in the raw image?
      if(
        point.x >= xPosRawImage[c] &&
        point.x < xPosRawImage[c] + rawImage[c].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) &&
        point.y >= yPosRawImage &&
        point.y < yPosRawImage + rawImage[c].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1)
        )
      {
        somethingWasSelected = true;
        selectedX = point.x - xPosRawImage[c];
        selectedY = point.y - yPosRawImage;
      }
  
      //is the selected Point in the segmented image?
      if(
        point.x >= xPosSegmentedImage[c] &&
        point.x < xPosSegmentedImage[c] + rawImage[c].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) &&
        point.y >= yPosSegmentedImage &&
        point.y < yPosSegmentedImage + rawImage[c].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1)
        )
      {
        somethingWasSelected = true;
        selectedX = point.x - xPosSegmentedImage[c];
        selectedY = point.y - yPosSegmentedImage;
      }
  
      if (somethingWasSelected)
      {
        xPosPreviewPixel[c] = selectedX;
        yPosPreviewPixel[c] = selectedY;
        RedrawWindow(NULL, NULL, RDW_INVALIDATE);
        break;
      }
    }
}

void CColorTable64DlgBar::OnCt64Remove()
{
  this->removeColorClassFromImage();
  sendLocal();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnCt64Median()
{
  this->smoothImage();
  sendLocal();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::smoothImage()
{
  HCURSOR lhCursor = AfxGetApp()->LoadStandardCursor(IDC_APPSTARTING);
  SetCursor(lhCursor);

  for ( int i = 1; i < rawImage[0].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1) - 1; i++ ) {
    for ( int j = 1; j < rawImage[0].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) - 1 - 1; j++ ) {

      int y1 = (useHighRes ? this->rawImage[0].getHighResY(j, i) : this->rawImage[0].image[i][0][j]);
      int u1 = this->rawImage[0].image[i / (useHighRes ? 2 : 1)][1][j / (useHighRes ? 2 : 1)];
      int v1 = this->rawImage[0].image[i / (useHighRes ? 2 : 1)][2][j / (useHighRes ? 2 : 1)];

      colorClass colorclass = (colorClass)this->segmentedImage[0].image[i][j];
      if ( (colorclass == noColor) || m_medianAllowReassign.GetCheck() ) {
        
        for ( int m = 0; m < numOfColors; m++ ) {
          colorClass c1 = (colorClass)m;
          
          if ( ( c1 != noColor ) && ( (c1 == selectedColorClass) || (m_medianAllColors.GetCheck()) ) ) {
            int counter = 0;
            double minDistance = 10000.0;
            for ( int k = i-1; k <= i+1; k++ ) {
              for ( int l = j-1; l <= j+1; l++ ) {
                if ( (k != i) || (l != j) ) {
                  colorClass c = (colorClass)this->segmentedImage[0].image[k][l];
                  if ( c1 == c ) {
                    counter++;
                    int y2 = (useHighRes ? this->rawImage[0].getHighResY(l, k) : this->rawImage[0].image[i][0][j]);;
                    int u2 = this->rawImage[0].image[k / (useHighRes ? 2 : 1)][1][l / (useHighRes ? 2 : 1)];
                    int v2 = this->rawImage[0].image[k / (useHighRes ? 2 : 1)][2][l / (useHighRes ? 2 : 1)];
                    double distance = sqrt ( sqr ((double)(y1-y2)) + sqr ((double)(u1-u2)) + sqr ((double)(v1-v2)) );
                    if ( distance < minDistance ) minDistance = distance;
                  }
                }
              }
            }
            if ( ( counter >= medianNeighbour ) && ( minDistance < medianDistance ) ) {
              this->colorTable.addColorClass( c1, y1, u1, v1, medianRange );
            } // if ( counter >= 7 )
          } // if ( c1 != noColor )
        } // for ( int m = 0; m < numOfColors; m++ )
      } // if ( colorclass == noColor )
    }
  }

  lhCursor = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
  SetCursor(lhCursor);
}

  
void CColorTable64DlgBar::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
  setClickRangeSlider(m_rangeSlider.GetPos());
  setPenSlider(m_penSlider.GetPos());
  setMedianRangeSlider(m_medianRangeSlider.GetPos());
  setMedianDistanceSlider(m_medianDistanceSlider.GetPos());
  setMedianNeighbourSlider(m_medianNeighbourSlider.GetPos());
  CDynamicBarDlg::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CColorTable64DlgBar::setClickRangeSlider(unsigned char newPosition)
{
  CString string;
  m_rangeSlider.SetPos(newPosition);
  clickRange = newPosition;
  string.Format("%d", newPosition);
  m_rangeStatic.SetWindowText(string);
}

void CColorTable64DlgBar::setPenSlider(unsigned char newPosition)
{
  CString string;
  m_penSlider.SetPos(newPosition);
  penSize = newPosition;
  string.Format("%d", newPosition * 2 + 1);
  m_penStatic.SetWindowText(string);
}

void CColorTable64DlgBar::setMedianRangeSlider(unsigned char newPosition)
{
  CString string;
  m_medianRangeSlider.SetPos(newPosition);
  medianRange = newPosition;
  string.Format("%d", newPosition);
  m_medianRangeStatic.SetWindowText(string);
}

void CColorTable64DlgBar::setMedianDistanceSlider(unsigned char newPosition)
{
  CString string;
  m_medianDistanceSlider.SetPos(newPosition);
  medianDistance = newPosition;
  string.Format("%d", newPosition);
  m_medianDistanceStatic.SetWindowText(string);
}

void CColorTable64DlgBar::setMedianNeighbourSlider(unsigned char newPosition)
{
  CString string;
  m_medianNeighbourSlider.SetPos(newPosition);
  medianNeighbour = newPosition;
  string.Format("%d", newPosition);
  m_medianNeighbourStatic.SetWindowText(string);
}

void CColorTable64DlgBar::OnClearChannelInImage() 
{
  removeColorClassFromImage();
}

void CColorTable64DlgBar::removeColorClassFromImage()
{
  this->previousColorTable=this->colorTable;
  this->undoColorTablePossible=TRUE;

  for(int x = 0; x < rawImage[0].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1); x++)
  {
    for(int y = 0; y < rawImage[0].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1); y++)
    {
      if(colorTable.getColorClass(
        (useHighRes ? rawImage[0].getHighResY(x, y) : rawImage[0].image[y][0][x]),
        rawImage[0].image[y / (useHighRes ? 2 : 1)][1][x / (useHighRes ? 2 : 1)],
        rawImage[0].image[y / (useHighRes ? 2 : 1)][2][x / (useHighRes ? 2 : 1)]) == selectedColorClass
        )
      {
        this->colorTable.removeColorClass(
          (useHighRes ? rawImage[0].getHighResY(x, y) : rawImage[0].image[y][0][x]),
          rawImage[0].image[y / (useHighRes ? 2 : 1)][1][x / (useHighRes ? 2 : 1)],
          rawImage[0].image[y / (useHighRes ? 2 : 1)][2][x / (useHighRes ? 2 : 1)],
          clickRange
        );
      }
    }
  }

  generateSegmentedImage();
  
  sendLocal();
    
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnSelectedColorOnly() 
{
	showSelectedColorClassOnly = (m_selectedColorOnlyCheck.GetCheck() == 1);
  generateSegmentedImage();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::generateSegmentedImage()
{
	for (int i = 0; i < maxGeneratedSegmentedImages; i++)
  {
    if (useHighRes)
    {
      if(showSelectedColorClassOnly)
        colorTable.generateHighResColorClassImage(rawImage[i], segmentedImage[i], selectedColorClass);
      else
        colorTable.generateHighResColorClassImage(rawImage[i], segmentedImage[i]);
    } else {
      if(showSelectedColorClassOnly)
        colorTable.generateColorClassImage(rawImage[i], segmentedImage[i], selectedColorClass);
      else
        colorTable.generateColorClassImage(rawImage[i], segmentedImage[i]);
    }
  }
}

void CColorTable64DlgBar::OnCapture1()
{
  rawImage[1] = rawImage[0];
  imageCaptured[1] = true;
  generateSegmentedImage();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnCapture2()
{
  rawImage[2] = rawImage[0];
  imageCaptured[2] = true;
  generateSegmentedImage();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnCapture3()
{
  rawImage[3] = rawImage[0];
  imageCaptured[3] = true;
  generateSegmentedImage();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnCapture4()
{
  rawImage[4] = rawImage[0];
  imageCaptured[4] = true;
  generateSegmentedImage();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnHighRes()
{
  AfxGetApp()->WriteProfileInt("ColorTable64Dlg", "HighRes", m_highRes.GetCheck() ? 1 : 0);
  useHighRes = (m_highRes.GetCheck() == 1);

  yPosSegmentedImage = rawImage[0].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1) + 1;
  for(int i = 0; i < 5; i++)
    xPosSegmentedImage[i] = xPosRawImage[i] = 120 + i * (rawImage[0].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1) + 2);

  generateSegmentedImage();
  RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}

void CColorTable64DlgBar::OnRecordReset() {
	memset(&record,0,sizeof(record));


	//init op green
			//update colortable
		for(int c1=0;c1<64;c1++)
		for(int c2=0;c2<64;c2++)
		for(int c3=0;c3<64;c3++) {
			colorClass winner = green;
				colorTable.addColorClass(winner,c1*4,c2*4,c3*4,1);//taking over..
		}


	undoRecordPossible=false;
}

/*
void CColorTable64DlgBar::OnRecordNewImage() {

	rawImage[1] = rawImage[0];
	imageCaptured[1] = true;
	rawImage[2] = rawImage[0];
	imageCaptured[2] = true;
	rawImage[3] = rawImage[0];
	imageCaptured[3] = true;

	//const int gauss[5] = {17,38,49,38,17};
	const int gauss[5] = {0,0,159,0,0};
	int w = rawImage[0].cameraInfo.resolutionWidth * (useHighRes ? 2 : 1);
	int h = rawImage[0].cameraInfo.resolutionHeight * (useHighRes ? 2 : 1);

	register int c,x,y,d,sum;

	//gauss filter (sigma=1.4), first pass: y dir, store in buffer 2
	for(c=0;c<3;c++) {	//color
		for(x=0;x<w;x++) {
			for(y=0;y<h-5;y++) {
				sum = 0;
				for(d=0;d<5;d++) {//convolution index
					sum += rawImage[0].image[y+d][c][x]*gauss[d];
				}
				rawImage[2].image[y+2][c][x]=sum/159;
			}
		}
	}

	//gauss filter (sigma=1.4), second pass: x dir, store in buffer 1
	for(c=0;c<3;c++) {	//color
		for(y=0;y<h;y++) {
			for(x=0;x<w-5;x++) {
				sum = 0;
				for(d=0;d<5;d++) {//convolution index
					sum += rawImage[2].image[y][c][x+d]*gauss[d];
				}
				rawImage[1].image[y][c][x+2]=sum/159;
			}
		}
	}

	// first derivative in x dir
	// first derivative in y dir
	// compute magnitude of gradient, merge color channels channels
	register int dx,dy;
	for(y=1;y<h-1;y++) {
		for(x=1;x<w-1;x++) {
			d=0;
			for(c=0;c<3;c++) {	//color
				dx=rawImage[1].image[y][c][x+1]-rawImage[1].image[y][c][x-1];
				dy=rawImage[1].image[y+1][c][x]-rawImage[1].image[y-1][c][x];
				d+=(int)(0.5+sqrt(dx*dx+dy*dy));
			}
			rawImage[2].image[y][0][x] = d;
			rawImage[2].image[y][1][x] = dx;
			rawImage[2].image[y][2][x] = dy;
		}
	}

	//clear corners, image 2
	for(c=0;c<3;c++) {
		for(y=1;y<h-1;y++) {
				rawImage[2].image[y][c][0] = 0;
				rawImage[2].image[y][c][w-1] = 0;
		}
		for(x=1;x<w-1;x++) {
				rawImage[2].image[0][c][x] = 0;
				rawImage[2].image[h-1][c][x] = 0;
		}
	}

	// compute histogram of possible edge points
	const int maxmagnitude=300;

	int hist[maxmagnitude];
	int i;
	for(i=0;i<maxmagnitude;i++) hist[i]=0;		//init hist

	for(y=1;y<h-1;y++) {						//fill histogram
		for(x=1;x<w-1;x++) {
			c = rawImage[2].image[y][0][x];
			if(c<0) c=0;
			if(c>=maxmagnitude) c=maxmagnitude-1;
			hist[c] ++;
		}
	}

	int totalhist=0,num;
	for(i=0;i<maxmagnitude;i++) totalhist += hist[i];		// sum histogram

	const double tHigh = 0.9;
	const double tLow = 0.6;

	int highcount = (int)(tHigh * totalhist);
	int lowcount = (int)(tLow * totalhist);

	int highthreshold=0;					
	int lowthreshold=0;

	num=hist[0];while((highthreshold<maxmagnitude-1)&&(num<highcount)) {						// select high quantile
		highthreshold++;
		num+=hist[highthreshold];
	}
	
	num=hist[0];while((lowthreshold<maxmagnitude-1)&&(num<lowcount)) {						// select low quantile
		lowthreshold++;
		num+=hist[lowthreshold];
	}

	getQueues().toGUI.out.text << "total edge candidates: "<<totalhist<<" highclass="<<highthreshold<<" lowclass="<<lowthreshold;
	getQueues().toGUI.out.text << " highcount="<<highcount<<" lowcount="<<lowcount;
	getQueues().toGUI.out.finishMessage(idText);
	
	// colorize
	for(y=0;y<h;y++) {
		for(x=0;x<w;x++) {
			if(rawImage[2].image[y][0][x]>highthreshold) {
				rawImage[2].image[y][0][x] = 100;
			} else {
				rawImage[2].image[y][0][x] = 0;
			}
			rawImage[2].image[y][1][x] = 0;
			rawImage[2].image[y][2][x] = 0;

		}
	}



}
*/

void CColorTable64DlgBar::OnRecordNewImage() {
	Image &real = rawImage[0];
	Image &temp = rawImage[3];
	Image &preview = rawImage[1];
	Image &gauss = rawImage[2];
	ColorClassImage &segmentation = segmentedImage[1];


	rawImage[1] = rawImage[0];
	imageCaptured[1] = true;
	rawImage[2] = rawImage[0];
//	imageCaptured[2] = true;
	rawImage[3] = rawImage[0];
//	imageCaptured[3] = true;

	const int convolution[5] = {17,38,49,38,17};
	//const int gauss[5] = {0,0,159,0,0};
	int w = rawImage[0].cameraInfo.resolutionWidth;
	int h = rawImage[0].cameraInfo.resolutionHeight;

	register int c,x,y,d,sum;

	//gauss filter (sigma=1.4), first pass: y dir, store in buffer 2
	for(c=0;c<3;c++) {	//color
		for(x=0;x<w;x++) {
			for(y=0;y<h-5;y++) {
				sum = 0;
				for(d=0;d<5;d++) {//convolution index
					sum += real.image[y+d][c][x]*convolution[d];
				}
				gauss.image[y+2][c][x]=sum/159;
			}
		}
	}

	segments.clear();
	OnRecordInitImages(w,h,real,gauss,temp,preview, segmentation);

	/*
	// te onbetrouwbaar.. helaas

	//auto segmentation?
	//1. color-class segmentation whole image
	//2. for each color-class, determine mean and variance, then test if blob sharp enough
	//3. add those blobs to the segments list
	OnRecordInitImages(w,h,real,gauss,temp,preview, segmentation);
	
	colorClass seg[300][300];
	for(x=0;x<w;x++) {	// step 1: segment whole image
		for(y=0;y<h;y++) {
			seg[x][y] = colorTable.getColorClass(real.image[y][0][x],real.image[y][1][x],real.image[y][2][x]);
		}
	}

	colorClass colors[] = {green,yellow,skyblue,pink,white};

	for(int cIndex=0;cIndex<5;cIndex++) {
		c = (int)colors[cIndex];
		int sumX=0,sumY=0,diffX=0,diffY=0,variance,meanX,meanY,count=0;

		int histX[300],histY[300];
		for(x=0;x<w;x++) histX[x]=0;
		for(y=0;y<w;y++) histY[y]=0;

		for(x=0;x<w;x++) {	
			for(y=0;y<h;y++) {

				if((int)seg[x][y]==c) {
					histX[x]++;
					histY[y]++;
					count++;
				}
			}
		}
		
		if(count) {
			meanX=0; while(sumX<count/2) { sumX += histX[meanX]; meanX++; }
			meanY=0; while(sumY<count/2) { sumY += histY[meanY]; meanY++; }

			OrderedList object;
			OnRecordFloodfill(meanX,meanY,object,w,h,real, gauss,temp,preview, segmentation);

			if((object.count()>recordObjectMinimumSize)&&(object.getColorClass()==c)) {
				// has minimum size and correct color
				int convexMinX[255],convexMinY[255],convexMaxX[255],convexMaxY[255];
				for(int i=0;i<255;i++) {
					convexMinX[i]=255;
					convexMaxX[i]=0;
					convexMinY[i]=255;
					convexMaxY[i]=0;
				}

				// determine convex hull
				object.backup();
				while(object.notEmpty()) {
					x = object.getX();
					y = object.getY();
					convexMinX[y]=min(convexMinX[y],x);
					convexMaxX[y]=max(convexMaxX[y],x);
					convexMinY[x]=min(convexMinY[x],y);
					convexMaxY[x]=max(convexMaxY[x],y);
					object.removeElement();
				}

				// determine hull volumne
				object.restore();
				int surfaceHull=0, surfaceObject=object.count();
				for(x=0;x<w;x++) {	
					for(y=0;y<h;y++) {
						if( ((x>=convexMinX[y]) && (x<= convexMaxX[y])) ||
							((y>=convexMinY[x]) && (y<= convexMaxY[x]))) {
							//segmentation.image[y][x] = c;
							surfaceHull++;
						} 
							//segmentation.image[y][x] = noColor;
					}
				}
				//getQueues().toGUI.out.text << " color "<<c<<"meanX "<<meanX<<" meanY "<<meanY<<" coverage:"<<((surfaceObject*100)/surfaceHull);
				//getQueues().toGUI.out.finishMessage(idText);
				if(surfaceObject>surfaceHull*recordObjectMinimumCoverage) {
					segments.appendList(object);
				}
			}


		}
		
	}

	OnRecordShow(segmentation,segments);
	*/
}


void CColorTable64DlgBar::OnRecordSeedPixel(int x,int y,int threshold) {
	if(rawImage[3].image[y][1][x]) return;
	if(rawImage[3].image[y][0][x]>threshold) {
		rawImage[3].image[y][1][x] = 1;	// edge
		OnRecordSeedPixel(x-1,y,threshold);
		OnRecordSeedPixel(x+1,y,threshold);
		OnRecordSeedPixel(x,y-1,threshold);
		OnRecordSeedPixel(x,y+1,threshold);
	} else {
		if(m_recordExtended.GetCheck()) 
			rawImage[3].image[y][1][x] = 1;	// edge point included

	}
}

int distance(int color1[],int color2[]) {
	int sum=0;
	if(color1[3]==0) return(0);
	if(color2[3]==0) return(0);
	for(int c=0;c<3;c++) sum+=sqr(color1[c]/color1[3]-color2[c]/color2[3]);
	return (int) sqrt((double) sum);
}

void CColorTable64DlgBar::OnRecordCheck() {
	maxGeneratedSegmentedImages = m_recordCheck.GetCheck()?1:5;
}


void CColorTable64DlgBar::OnRecordInitImages(int w,int h,Image &real,Image &gauss,Image &temp, Image &preview, ColorClassImage &segmentation) {
	for(register int x=0;x<w;x++) {
		for(register int y=0;y<h;y++) {
			temp.image[y][0][x]=0;
			temp.image[y][1][x]=128;
			temp.image[y][2][x]=128;

			preview.image[y][0][x] = real.image[y][0][x];
			preview.image[y][1][x] = 128;
			preview.image[y][2][x] = 128;

			if(useHighRes) {
				segmentation.image[y*2][x*2] = black;
				segmentation.image[y*2+1][x*2] = black;
				segmentation.image[y*2][x*2+1] = black;
				segmentation.image[y*2+1][x*2+1] = black;
			} else {
				segmentation.image[y][x] = black;
			}

		}
	}
}

void CColorTable64DlgBar::OnRecordFloodfill(int cx,int cy,OrderedList &object,int w,int h,Image &real,Image &gauss,Image &temp,Image &preview, ColorClassImage &segmentation) {
	int avgcolor[4];
	for(int c=0;c<3;c++) avgcolor[c] = gauss.image[cy][c][cx];	// our seed..
	avgcolor[3]=1;

	OrderedList opening,list;
	opening.appendElement(cx,cy);	

	int maxdist = m_recordDistance.GetPos();
	register int x,y;

	while(opening.notEmpty()) {
		// get the first element
		x = opening.getX();
		y = opening.getY();
		opening.removeElement();

		if((x<0)||(y<0)||(x>=w)||(y>=h)) continue;
		if(temp.image[y][0][x]) continue;

		temp.image[y][0][x]=200;

		int color[4]={0,0,0,1};
		for(c=0;c<3;c++) color[c] = gauss.image[y][c][x];	
		if(distance(color,avgcolor)>maxdist) continue;

		// add our color
		for(c=0;c<4;c++) avgcolor[c]+=color[c];

		// my mind to your mind..
		//for(c=0;c<3;c++) gauss.image[y][c][x] = avgcolor[c]/avgcolor[3];	
		list.appendElement(x,y);


		opening.appendElement(x,y-1);
		opening.appendElement(x,y+1);
		opening.appendElement(x-1,y);
		opening.appendElement(x+1,y);
	}

	int classCount[numOfColors];
	for(int i=0;i<numOfColors;i++) classCount[i]=0;

	list.backup();
	while(list.notEmpty()) {
		x = list.getX();
		y = list.getY();
		list.removeElement();

		char c = colorTable.colorClasses[real.image[y][0][x]/4][real.image[y][1][x]/4][real.image[y][2][x]/4];
		classCount[c]++;
		
			
		preview.image[y][0][x] = real.image[y][0][x];
		preview.image[y][1][x] = avgcolor[1]/avgcolor[3];
		preview.image[y][2][x] = avgcolor[2]/avgcolor[3];

	}

	list.restore();
	colorClass winner=noColor;
	if(m_recordAutoColor.GetCheck()) {
		for(i=1;i<numOfColors;i++) {
			if(classCount[i]>list.count()/2) winner = (colorClass)i;
		}
	} else winner =  selectedColorClass;

	//getQueues().toGUI.out.text << " and the winner is: "<<(int)winner<<" with "<<classCount[(int)winner];
	//getQueues().toGUI.out.finishMessage(idText);
	list.restore();
	while(list.notEmpty()) {
		object.appendElement(list.getX(),list.getY(),winner);
		list.removeElement();
	}
}

void CColorTable64DlgBar::OnRecordShow(ColorClassImage &segmentation,OrderedList &object) {
	segments.backup();
	while(object.notEmpty()) {
		int x = object.getX();
		int y = object.getY();
		colorClass c = object.getColorClass();
		object.removeElement();
		
		if(useHighRes) {
			segmentation.image[y*2][x*2] = c;
			segmentation.image[y*2+1][x*2] = c;
			segmentation.image[y*2][x*2+1] = c;
			segmentation.image[y*2+1][x*2+1] = c;
		} else {
			segmentation.image[y][x] = c;
		}

	}
	segments.restore();
}

void CColorTable64DlgBar::OnRecordClick(bool leftRight,int cx,int cy,int num) {
	int w = rawImage[0].cameraInfo.resolutionWidth;
	int h = rawImage[0].cameraInfo.resolutionHeight;

	if(useHighRes) {
		cx/=2; cy/=2;
	}

	Image &real = rawImage[0];
	Image &temp = rawImage[3];
	Image &preview = rawImage[1];
	Image &gauss = rawImage[2];
	ColorClassImage &segmentation = segmentedImage[1];

	if(leftRight) {

		OnRecordInitImages(w,h,real,gauss,temp,preview, segmentation);

		OnRecordFloodfill(cx,cy,segments,w,h,real, gauss,temp,preview, segmentation);
		OnRecordShow(segmentation,segments);
	} else {
		//rechts geklickt, add to record array
	    memcpy(backupRecord,record,sizeof(record));
		undoRecordPossible = true;


		// compute weights
		segments.restore();
		int weight[numOfColors];
		for(int i=0;i<numOfColors;i++) weight[i]=0;
		while(segments.notEmpty()) {
			weight[(int)segments.getColorClass()]++;
			segments.removeElement();
		}
		for(i=0;i<numOfColors;i++) 
			if(weight[i]) 
				weight[i]=(w*h*10)/weight[i];
				

		// store this in our record-array
		segments.restore();
		int x,y;
		int c;
		while(segments.notEmpty()) {
			x = segments.getX();
			y = segments.getY();
			c = (int)segments.getColorClass();
			segments.removeElement();

			record
				[real.image[y][0][x]/4]
				[real.image[y][1][x]/4]
				[real.image[y][2][x]/4]
				[c] += weight[c];
		}
		segments.clear();

		//update colortable
		for(int c1=0;c1<64;c1++)
		for(int c2=0;c2<64;c2++)
		for(int c3=0;c3<64;c3++) {
			colorClass winner = noColor;
			int count = 0;
			for(i=0;i<numOfColors;i++) count+=record[c1][c2][c3][i];
			for(i=0;i<numOfColors;i++) if(record[c1][c2][c3][i]>count/2) winner = (colorClass)i;
			if(count>0) {
				colorTable.addColorClass(winner,c1*4,c2*4,c3*4,1);//taking over..

			}
		}

        sendLocal();

	}
	RedrawWindow(NULL, NULL, RDW_INVALIDATE);
}


OrderedList::OrderedList() {
	head = 0;
	tail = 0;
	values = new short[3*OrderListMaxItems];
}

OrderedList::~OrderedList() {
	delete[] values;
}

int OrderedList::getX() {
	return(*(values+3*head));
}

int OrderedList::getY() {
	return(*(values+3*head+1));
}

colorClass OrderedList::getColorClass() {
	return(colorClass(*(values+3*head+2)));
}

void OrderedList::appendElement(int x,int y,int color) {
	if(((tail+1) % OrderListMaxItems)==head) return;
	*(values+3*tail+0) = x;
	*(values+3*tail+1) = y;
	*(values+3*tail+2) = color;
	tail = (tail+1) % OrderListMaxItems;
}

void OrderedList::removeElement() {
	head = (head+1) % OrderListMaxItems;
}

void OrderedList::appendList(OrderedList &source) {
	while(source.notEmpty()) {
		appendElement(source.getX(),source.getY(),source.getColorClass());
		source.removeElement();
	}
}

  

/*
* Change log :
* 
* $Log: ColorTable64DlgBar.cpp,v $
* Revision 1.3  2004/06/30 13:41:46  thomas
* improved usability (frequently readers know this phrase;-) ) of c64-dialog
* - storing config of debug-image and highres
*
* Revision 1.2  2004/05/27 10:08:09  thomas
* added model-specific locations
*
* Revision 1.1.1.1  2004/05/22 17:27:10  cvsadm
* created new repository GT2004_WM
*
* Revision 1.16  2004/05/03 12:31:12  thomas
* fix removing of selected color only works now as intended
*
* Revision 1.15  2004/04/23 12:53:22  risler
* imageProcessorGeneral should not be default for ColorTableToolBar, was accidentally checked in from DDDGO stuff
*
* Revision 1.14  2004/04/17 12:10:20  risler
* bugfix: add color class did not work in low resolution
*
* Revision 1.13  2004/04/07 13:00:46  risler
* ddd checkin after go04 - second part
*
* Revision 1.6  2004/04/06 13:19:35  risler
* cleaned up and improved high resolution image support
*
* Revision 1.5  2004/04/05 14:58:34  risler
* colortable dialog now can show both resolutions
*
* Revision 1.4  2004/04/02 14:46:19  risler
* added segmented high resolution image
* colortable dialog supports high resolution images
*
* Revision 1.3  2004/03/30 08:12:48  risler
* bugfix multiple image positions
*
* Revision 1.2  2004/03/29 13:32:26  risler
* imageProcessorGeneral as default image
*
* Revision 1.1.1.1  2004/03/29 08:28:44  Administrator
* initial transfer from tamara
*
* Revision 1.12  2004/03/28 20:30:19  risler
* corrected YUV conversion for pixel preview
*
* Revision 1.11  2004/03/28 18:39:35  risler
* colortable64 dialog multiple image support
*
* Revision 1.10  2004/03/28 14:16:40  risler
* left pixel preview corrected
*
* Revision 1.9  2004/03/18 15:03:50  risler
* added pixel preview
*
* Revision 1.8  2004/03/18 10:35:36  risler
* simplified image selection
*
* Revision 1.7  2004/03/17 17:24:47  risler
* ColorTable64Dlg can segment debug images
*
* Revision 1.6  2004/01/09 15:56:14  thomas
* added: support for importing cmu/unsw ct's and exporting unsw ct's
*
* Revision 1.5  2004/01/04 16:36:41  juengel
* adaptation for larger images
*
* Revision 1.4  2003/12/15 11:46:14  juengel
* Introduced CameraInfo
*
* Revision 1.3  2003/12/09 19:49:24  loetzsch
* Renamed some of the main queues of RobotControl.
*
* Added possibility to send messages to specific simulated or physical robots.
*
* Revision 1.2  2003/12/09 16:25:29  jhoffman
* - addColorClass( , , , , ) now works like addColorClass( , , , , , range)
* - 176x144 --> 144 x ...Width
*
* Revision 1.1  2003/10/07 10:09:37  cvsadm
* Created GT2004 (M.J.)
*
* Revision 1.4  2003/09/26 15:28:23  juengel
* Renamed DataTypes to representations.
*
* Revision 1.3  2003/09/26 11:40:12  juengel
* - sorted tools
* - clean-up in DataTypes
*
* Revision 1.2  2003/07/07 07:38:04  roefer
* JPEG support added
*
* Revision 1.1.1.1  2003/07/02 09:40:25  cvsadm
* created new repository for the competitions in Padova from the 
* tamara CVS (Tuesday 2:00 pm)
*
* removed unused solutions
*
* Revision 1.14  2003/05/26 13:09:49  thomas
* added: option to save color table in cmu-format
*
* Revision 1.13  2003/05/11 23:46:32  dueffert
* Depend now works with RobotControl too
*
* Revision 1.12  2003/05/11 17:03:00  risler
* added location.cfg
*
* Revision 1.11  2003/04/16 17:46:46  juengel
* clean up in ColorTable64DlgBar
*
* Revision 1.10  2003/04/15 15:52:07  risler
* DDD GO 2003 code integrated
*
* Revision 1.10  2003/04/08 21:34:43  Charlie
* bugfix
*
* Revision 1.9  2003/04/07 21:13:09  Charlie
* added more functionality to ct64 dialog
*
* Revision 1.9  2003/04/14 16:17:07  loetzsch
* ATH after GermanOpen CVS merge
*
* improved dialog
*
* Revision 1.3  2003/04/10 12:08:01  juengel
* Activated showSelectedColorClassOnly
*
* Revision 1.2  2003/04/10 07:57:37  juengel
* no message
*
* Revision 1.8  2003/04/01 14:00:42  juengel
* color table is sent to gui
*
* Revision 1.7  2003/03/31 17:02:55  osterhues
* Disabled sendLocal() at init because it has overridden color table selection from coltable.* file
*
* Revision 1.6  2003/03/05 17:09:27  loetzsch
* redesign of the queues and debug key tables in RobotControl
*
* Revision 1.5  2002/10/04 10:22:16  loetzsch
* Invalidate and RedrawWindow are only called in the handleMessage
* function when the dialog is visible
*
* Revision 1.4  2002/09/20 23:28:34  juengel
* Moved instance of DebugDrawingManager to RobotControlMainFrame.
*
* Revision 1.3  2002/09/18 16:35:13  loetzsch
* made GT2003 compilable,
* rechanged message ids
*
* Revision 1.2  2002/09/17 23:55:23  loetzsch
* - unraveled several datatypes
* - changed the WATCH macro
* - completed the process restructuring
*
* Revision 1.1  2002/09/10 15:49:04  cvsadm
* Created new project GT2003 (M.L.)
* - Cleaned up the /Src/DataTypes directory
* - Removed challenge related source code
*
* Revision 1.4  2002/08/22 15:20:10  loetzsch
* debug messages to the local processes are now sent in a better synchronized way.
*
* Revision 1.3  2002/08/07 10:50:21  loetzsch
* - removed direct access to the dialogs
* - faster starting of RobotControl application and main window
* - general clean up
*
* Revision 1.2  2002/07/23 13:43:36  loetzsch
* - new streaming classes
* - removed many #include statements
* - 5 instead of 3 debug queues in RobotControl
* - exchanged StaticQueue with MessageQueue
* - new debug message handling
* - empty constructors in bars / dialogs
* - access to debugkeytables and queues via RobotControlQueues.h and RobotControlDebugKeyTables.h
* - general clean up
*
* Revision 1.1.1.1  2002/05/10 12:40:20  cvsadm
* Moved GT2002 Project from ute to tamara.
*
* Revision 1.23  2002/05/04 12:22:55  juengel
* Moved global instance of the ColorTable64 for segmented image views from CRobotControlApp to DebugDrawingManager.cpp
*
* Revision 1.22  2002/05/04 11:40:40  juengel
* Added << endl after colorTable.
*
* Revision 1.21  2002/04/23 17:45:17  loetzsch
* - splitted debugKeyTable into debugKeyTableForRobot and debugKeyTableForLocalProcesses
* - removed Modules Toolbar
* - removed access for dialogs and toolbars to solutionRequest.
* - changed access for dialogs and toolbars to debug queues
* - removed the instance of SolutionRequest in CRobotControlApp
* - only the log player, local processes and the robot put messages into queueToRobotControl
*
* Revision 1.20  2002/04/16 15:46:36  dueffert
* no message
*
* Revision 1.2  2002/04/12 02:06:06  dueffert
* we do not want <<endl after coltable64, it has its own stopsign
*
* Revision 1.19  2002/03/22 16:56:39  loetzsch
* added a send to robot button to the color table 64 tool
*
* Revision 1.18  2002/02/12 16:34:49  risler
* finished MofTester
*
* Revision 1.17  2002/02/11 00:53:33  loetzsch
* ::Paint Methoden in ::paint umbenannt
*
* Revision 1.16  2002/02/07 16:27:12  loetzsch
* colortables are sent twice on every change of the color table
* for synchronization with the processes
*
* Revision 1.15  2002/01/26 18:09:20  juengel
* DebugDrawingManager umstrukturiert.
*
* Revision 1.14  2002/01/22 14:55:48  juengel
* Save Dialog speichert nun und ffnet nicht mehr.
*
* Revision 1.13  2002/01/19 21:36:26  risler
* added HeadMotionTester, HeadControlSelector
*
* Revision 1.12  2002/01/15 13:50:34  bach
* Undo button in color table tool added (Joscha)
*
* Revision 1.11  2002/01/08 15:50:10  juengel
* Zeichnungsmethoden fr Image und colorClassImage  knnen jetzt skalieren.
*
* Revision 1.10  2002/01/04 14:35:18  juengel
* no message
*
* Revision 1.9  2001/12/22 20:35:35  loetzsch
* removed ColorTable::segmentImage, replaced by
* ColorTable64::generateColorClassImage
*
* Revision 1.8  2001/12/22 08:25:30  roefer
* Enhanced constructor of class File, removed second one
*
* Revision 1.7  2001/12/21 12:38:37  juengel
* Jetzt kann auch in das segmented Image geklickt werden.
* Immer wenn sich die Farbtabelle ndert, wird sie in die QueueToRobotControl gestellt.
*
* Revision 1.6  2001/12/20 19:19:52  juengel
* Slider eingebaut
*
* Revision 1.5  2001/12/20 14:36:34  juengel
* Methoden zum Zeichnen der ColorTable zeichnen jetzt in Bitmaps und nicht mehr in CDCs
*
* Revision 1.4  2001/12/19 14:04:47  loetzsch
* In ColorTables Pointer gegen Referenzen ausgetauscht,
* Funktion segmentImage hinzugefgt und implementiert
*
* Revision 1.3  2001/12/19 10:45:21  juengel
* An der Anzeige gefrickelt.
*
* Revision 1.2  2001/12/17 19:22:58  juengel
* Grundfunktionalitt hergestellt.
*
* Revision 1.1  2001/12/15 19:26:13  juengel
* ColorTable, ColorTable64, ColorTable64DlgBar hinzugefgt.
*
*
*/
