/**
* @file StatisticsDlgBar.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file StatisticsDlgBar.cpp
* 
* Implementation of class CStatisticsDlgBar.
*
* @author Jurgen Sturm, Amsterdam
*/

#include "StdAfx.h"
#include "StatisticsDlgBar.h"

#include "RobotControlQueues.h"
#include "RobotControlDebugKeyTables.h"



BEGIN_MESSAGE_MAP(CStatisticsDlgBar, CDynamicBarDlg)
//{{AFX_MSG_MAP(CStatisticsDlgBar)
ON_WM_PAINT()
ON_WM_DESTROY()
ON_WM_LBUTTONDOWN()
ON_BN_CLICKED(IDC_STATISTICS_GENERAL_RESET, OnResetButton)
ON_BN_CLICKED(IDC_STATISTICS_GENERAL_TIMER, OnTimerButton)
ON_CBN_SELCHANGE(IDC_STATISTICS_POSITION_SOURCE, OnReferenceSourceChange)
ON_CBN_SELCHANGE(IDC_STATISTICS_POSITION_PREDEFINED, OnPredefinedPositionChange)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CStatisticsDlgBar::CStatisticsDlgBar()
: CRobotControlDialogBar(IDD)
{
	//{{AFX_DATA_INIT(CStatisticsDlgBar)
	//}}AFX_DATA_INIT
	captureCount = 0;
	fSmall.CreateFont(8,0,0,0,0,0,0,0,0,0,0,0,0,"Small Fonts");
	fError.CreateFont(24,0,0,0,FW_BOLD,0,0,0,0,0,0,0,0,"Georgia");
	fNormal.CreateFont(12,0,0,0,0,0,0,0,0,0,0,0,0,"Verdana");
	fGiant.CreateFont(250,0,0,0,0,0,0,0,0,0,0,0,0,"Verdana");
	fBig.CreateFont(50,0,0,0,0,0,0,0,0,0,0,0,0,"Verdana");
	brushBlue.CreateSolidBrush(RGB(196,196,255));
	brushOrange.CreateSolidBrush(RGB(255,165,0));
	brushLime.CreateSolidBrush(RGB(196,255,196));
	brushGreen.CreateSolidBrush(RGB(128,255,128));
	brushRed.CreateSolidBrush(RGB(255,196,196));
	brushGrey.CreateSolidBrush(RGB(128,128,128));
	for(int j=0;j<6;j++) {
		brushFade[5-j].CreateSolidBrush(RGB(155+j*20,155+j*20,255));
	}
	selectedFrame = -1;
	selectedType = (PerceptType)-1;
	activePosition = -2;
}

CStatisticsDlgBar::~CStatisticsDlgBar()
{
}

void CStatisticsDlgBar::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CStatisticsDlgBar)
  DDX_Control(pDX, IDC_STATISTICS_GENERAL_FRAMES, m_generalFrames);
  DDX_Control(pDX, IDC_STATISTICS_GENERAL_RESET, m_generalReset);
  DDX_Control(pDX, IDC_STATISTICS_GENERAL_SECONDS, m_generalSeconds);
  DDX_Control(pDX, IDC_STATISTICS_GENERAL_TIMER, m_generalTimer);
  DDX_Control(pDX, IDC_STATISTICS_POSITION_TRUE_X, m_positionTrueX);
  DDX_Control(pDX, IDC_STATISTICS_POSITION_TRUE_Y, m_positionTrueY);
  DDX_Control(pDX, IDC_STATISTICS_POSITION_TRUE_PHI, m_positionTruePhi);
  DDX_Control(pDX, IDC_STATISTICS_POSITION_WITHIN_DIST, m_positionWithinDist);
  DDX_Control(pDX, IDC_STATISTICS_POSITION_SOURCE, m_positionSource);
  DDX_Control(pDX, IDC_STATISTICS_POSITION_PREDEFINED, m_positionPredefined);
  DDX_Control(pDX, IDC_STATISTICS_HISTORY, m_History);
  DDX_Control(pDX, IDC_STATISTICS_HISTORY_ESTIMATED, m_History_Estimated);
  DDX_Control(pDX, IDC_STATISTICS_HISTORY_TRUE, m_History_True);
  DDX_Control(pDX, IDC_STATISTICS_HISTORY_ESTIMATED, m_History_Estimated);
  DDX_Control(pDX, IDC_STATISTICS_HISTORY_TRUE, m_History_True);
  DDX_Control(pDX, IDC_STATISTICS_HISTORY_ESTIMATED_RELATIVE, m_History_EstimatedRelative);
  DDX_Control(pDX, IDC_STATISTICS_HISTORY_TRUE_RELATIVE, m_History_TrueRelative);

  //}}AFX_DATA_MAP
}

BOOL CStatisticsDlgBar::OnInitDialog() 
{
  CDynamicBarDlg::OnInitDialog();
  OnResetButton();
  
//  getQueues().toGUI.out.text << "Hoho Statistics Dialog created. Hello World.";
//  getQueues().toGUI.out.finishMessage(idText);
  
  return TRUE;
}

void CStatisticsDlgBar::OnResetButton()
{
	showSummary();
	buffer = MainBuffer();
	RedrawWindow(NULL,NULL,RDW_INVALIDATE);
	activePosition = -2;//disabled
	kidnapMode = false;
}

void CStatisticsDlgBar::OnTimerButton()
{
	buffer = MainBuffer();
	RedrawWindow(NULL,NULL,RDW_INVALIDATE);
	kidnapMode = false;
	activePosition = -1;//wait for first percept
	for(int i=0;i<11;i++) {
		predefinedPositionsFound[i]=0;
		startingFrameAtPosition[i]=0;
	}
}


bool CStatisticsDlgBar::handleCommand(UINT command)
{
	return false;
}



void CStatisticsDlgBar::updateUI(CCmdUI* pCmdUI)
{
}

void CStatisticsDlgBar::showSummary() {
	if(activePosition==-2) return;
	buffer.spreadTrueData();//before the buffer is completly flushed in the meantime, spread data
	double distBound = getDouble(m_positionWithinDist);
	buffer.evaluate(distBound);
	buffer.updateStatistics();

	SingleFrame *frame=buffer.getLatest();

	getQueues().toGUI.out.text << "===== Summary =====";
	getQueues().toGUI.out.finishMessage(idText);

	int tp,tn,fn,fp;
	tp=tn=fn=fp=0;
	for(int type=1;type<maxPerceptType-1;type++) {
		tp += frame->landmark[type].statistics.TruePositives;
		fp += frame->landmark[type].statistics.FalsePositives;
		tn += frame->landmark[type].statistics.TrueNegatives;
		fn += frame->landmark[type].statistics.FalseNegatives;
	}
	getQueues().toGUI.out.text << "Flags: ";
	getQueues().toGUI.out.finishMessage(idText);

	if(tp+fn) {
		getQueues().toGUI.out.text << "  precision visible flags: "<< (tp*100)/(tp+fn) <<"% ("<<fn<<" FN : "<<tp<<" TP)";
		getQueues().toGUI.out.finishMessage(idText);
	}
	
	if(tn+fp) {
		getQueues().toGUI.out.text << "  precision invisible flags: "<< ((tn)*100)/(fp+tn)<<"% ("<<fp<<" FP : "<<tn<<" TN)";
		getQueues().toGUI.out.finishMessage(idText);
	}
	
	tp = frame->landmark[goalYellow].statistics.TruePositives;
	fp = frame->landmark[goalYellow].statistics.FalsePositives;
	tn = frame->landmark[goalYellow].statistics.TrueNegatives;
	fn = frame->landmark[goalYellow].statistics.FalseNegatives;

	getQueues().toGUI.out.text << "Yellow goal: ";
	getQueues().toGUI.out.finishMessage(idText);

	if(tp+fn) {
		getQueues().toGUI.out.text << "  precision visible goals: "<< (tp*100)/(tp+fn) <<"% ("<<fn<<" FN : "<<tp<<" TP)";
		getQueues().toGUI.out.finishMessage(idText);
	}
	
	if(tn+fp) {
		getQueues().toGUI.out.text << "  precision invisible goals "<< ((tn)*100)/(tn+fp)<<"% ("<<fp<<" FP : "<<tn<<" TN)";
		getQueues().toGUI.out.finishMessage(idText);
	}
	
	tp = frame->landmark[goalBlue].statistics.TruePositives;
	fp = frame->landmark[goalBlue].statistics.FalsePositives;
	tn = frame->landmark[goalBlue].statistics.TrueNegatives;
	fn = frame->landmark[goalBlue].statistics.FalseNegatives;

	getQueues().toGUI.out.text << "Blue goal: ";
	getQueues().toGUI.out.finishMessage(idText);

	if(tp+fn) {
		getQueues().toGUI.out.text << "  precision visible goals: "<< (tp*100)/(tp+fn) <<"% ("<<fn<<" FN : "<<tp<<" TP)";
		getQueues().toGUI.out.finishMessage(idText);
	}
	
	if(tn+fp) {
		getQueues().toGUI.out.text << "  precision invisible goals "<< ((tn)*100)/(tn+fp)<<"% ("<<fp<<" FP : "<<tn<<" TN)";
		getQueues().toGUI.out.finishMessage(idText);
	}

	getQueues().toGUI.out.text << "Selflocator: ";
	getQueues().toGUI.out.finishMessage(idText);

	if(frame->locator.statistics.TotalCount) {
		getQueues().toGUI.out.text << "  average error: " << 
			(frame->locator.statistics.SumErrorDist/frame->locator.statistics.TotalCount)<<" mm, " <<
			(frame->locator.statistics.SumErrorPhi/frame->locator.statistics.TotalCount*180/M_PI)<<" deg" ;
		getQueues().toGUI.out.finishMessage(idText);
	}

	for(int i=0;i<10;i++) {
		
		if(predefinedPositionsFound[i]==-1) {
			getQueues().toGUI.out.text << "Kidnap recovery time "<<(i+1)<<": failed";
		} else {
			getQueues().toGUI.out.text << "Kidnap recovery time "<<(i+1)<<": after "<<
				(predefinedPositionsFound[i]-startingFrameAtPosition[i])<<" frames";
		}
		getQueues().toGUI.out.finishMessage(idText);

	}

	m_generalSeconds.SetWindowText("sequence finished");
}


bool CStatisticsDlgBar::handleMessage(InMessage& message)
{
	if((message.getMessageID()!=idPercepts)
		&&(message.getMessageID()!=idWorldState)
		&&(message.getMessageID()!=idOracledWorldState)) 
		return false;

	if(message.getMessageID()==idPercepts) 
	{
		handleMessagePercepts(message);
	}
	if(message.getMessageID()==idWorldState)
	{
		handleMessageWorldState(message);
	}
	if(message.getMessageID()==idOracledWorldState)
	{
		handleMessageOracledWorldState(message);
	}



	// 1. determine whether in kidnap-mode
	// 2. when switching to kidnap-mode, increment activePosition
	// 3. when kidnap-mode is ending, store current frame number in startingFrameAtPosition[activePosition]
	// 4. when not in kidnap-mode, and position is within bounds, store current frame in predefinedPositionsFound[activePosition]
	double distBound = getDouble(m_positionWithinDist);
	buffer.spreadTrueData();//before the buffer is completly flushed in the meantime, spread data
	buffer.evaluate(distBound);
	buffer.updateStatistics();

	bool newKidnapMode = buffer.getLatest()->framesSinceLastLandmark > KIDNAP_DETECTION_FRAMES;

	if(newKidnapMode)
		buffer.getLatest()->setKidnapped();

	if(!kidnapMode && newKidnapMode && activePosition>=-1) {
		activePosition++;
		startingFrameAtPosition[activePosition]=buffer.latestFrame;
		if(activePosition > m_positionPredefined.GetCount()) {
			activePosition = -1; // disabled;
			//finish
			showSummary();
		}
		else {
			m_positionPredefined.SetCurSel(activePosition+1);
			OnPredefinedPositionChange();
			for(int i=0;i<KIDNAP_DETECTION_FRAMES;i++) {
				SingleFrame *s = buffer.getSingleFrame(buffer.latestFrame-i);
				if(s!= NULL)
					s->setKidnapped();
			}
		}

	}

	if(kidnapMode && !newKidnapMode) {
		startingFrameAtPosition[activePosition] = buffer.latestFrame;
	}

	kidnapMode = newKidnapMode;
	if(	!kidnapMode && 
		(buffer.getLatest()->locator.withinRangeValid && buffer.getLatest()->locator.withinRange) &&
		(predefinedPositionsFound[activePosition]==0)
		) 
	{//recovered ffrom kidnap
		predefinedPositionsFound[activePosition]=buffer.latestFrame;
		getQueues().toGUI.out.text << "Kidnap recovery time "<<(activePosition)<<": after "<<(predefinedPositionsFound[activePosition]-startingFrameAtPosition[activePosition])<<" frames";
		getQueues().toGUI.out.finishMessage(idText);
	}

	if(!(predefinedPositionsFound[activePosition]==0))
		buffer.getLatest()->setRecovered();

	CString s;
	if(activePosition>=0) {
		if(kidnapMode) 
			s.Format("waiting for position %d since %d frames",activePosition,buffer.latestFrame-startingFrameAtPosition[activePosition]);
		else
			s.Format("recording in position %d since %d frames",activePosition,buffer.latestFrame-startingFrameAtPosition[activePosition]);
	} else s.Format("");

	m_generalSeconds.SetWindowText(s);

	s.Format("%d",buffer.latestFrame);
	m_generalFrames.SetWindowText(s);

//	RedrawWindow(NULL,NULL,RDW_INVALIDATE);

	return true;
}


void CStatisticsDlgBar::OnReferenceSourceChange() 
{
}

void CStatisticsDlgBar::OnPredefinedPositionChange() 
{
	int index =m_positionPredefined.GetCurSel();
	if(index>0) {
		CString s;
		Pose2D pose;
		pose.translate(predefinedPositions[index][0],predefinedPositions[index][1]);
		pose.rotate(double(predefinedPositions[index][2])/180*M_PI);
		pose.translate(-100,0);	// one robotlength backwards

		s.Format("%d",int(pose.translation.x));
		m_positionTrueX.SetWindowText(s);
		s.Format("%d",int(pose.translation.y));
		m_positionTrueY.SetWindowText(s);
		s.Format("%d",int(pose.rotation/M_PI*180));
		m_positionTruePhi.SetWindowText(s);
	}
}

void CStatisticsDlgBar::OnLButtonDown(UINT nFlags, CPoint point) {
	int x = (point.x-ox);
	int y = (point.y-oy);
	if(x>=0 && y>=0) {
		x /= ow;
		y /= oh;
		if (x>=0 && x<ocolumns && y>=0 && y<maxPerceptType+1) {
			selectedType = (PerceptType) (y-1);
			selectedFrame = framePaintedAt[ocolumns-1-x];
		}
	}
	RedrawWindow(NULL,NULL,RDW_INVALIDATE);
}


void CStatisticsDlgBar::OnConfigurationLoad(CString sectionName)
{
  m_positionTrueX.SetWindowText(AfxGetApp()->GetProfileString(sectionName + "\\Statistics","TrueX","0"));
  m_positionTrueY.SetWindowText(AfxGetApp()->GetProfileString(sectionName + "\\Statistics","TrueY","0"));
  m_positionTruePhi.SetWindowText(AfxGetApp()->GetProfileString(sectionName + "\\Statistics","TruePhi","0"));
  m_positionWithinDist.SetWindowText(AfxGetApp()->GetProfileString(sectionName + "\\Statistics","WithinDist","200"));

  m_positionSource.SelectString(-1,
		  AfxGetApp()->GetProfileString(sectionName + "\\Statistics", "referenceSource", "manual"));
}


void CStatisticsDlgBar::OnConfigurationSave(CString sectionName)
{
  AfxGetApp()->WriteProfileString(sectionName + "\\Statistics","TrueX",getString(m_positionTrueX));
  AfxGetApp()->WriteProfileString(sectionName + "\\Statistics","TrueY",getString(m_positionTrueY));
  AfxGetApp()->WriteProfileString(sectionName + "\\Statistics","TruePhi",getString(m_positionTruePhi));
  AfxGetApp()->WriteProfileString(sectionName + "\\Statistics","WithinDist",getString(m_positionWithinDist));

  CString referenceSelection;
  m_positionSource.GetLBText(m_positionSource.GetCurSel(),referenceSelection);
  AfxGetApp()->WriteProfileString(
		sectionName + "\\Statistics", "referenceSource", referenceSelection);
}

void CStatisticsDlgBar::setStaticInt(CStatic &control, int i) 
{
	CString s="";
	s.Format("%d",i);
	control.SetWindowText(s);
}

void CStatisticsDlgBar::setStaticDouble(CStatic &control, double i) 
{
	CString s="";
	s.Format("%.2f",i);
	control.SetWindowText(s);
}

double CStatisticsDlgBar::getDouble(CEdit &control) 
{
	char s[255];
	control.GetWindowText(s,255);
	return(atof(s));

}

CString CStatisticsDlgBar::getString(CEdit &control) 
{
	char s[255];
	control.GetWindowText(s,255);
	CString a="";
	a+=s;
	return(a);

}

// ************************************************************************** Message parsing & paint
void CStatisticsDlgBar::handleMessagePercepts(InMessage& message) {
	LandmarksPercept landmarksPercept;
	BallPercept ballPercept;
	PlayersPercept playersPercept;
	ObstaclesPercept obstaclesPercept;
	PSDPercept psdPercept;
	CollisionPercept collisionPercept;
	LinesPercept linesPercept;
	EdgesPercept edgesPercept;
	CameraMatrix cameraMatrix;
	CameraInfo cameraInfo;

	message.bin >> RECEIVE_PERCEPTS(cameraMatrix, cameraInfo, ballPercept,
		landmarksPercept,linesPercept,edgesPercept,playersPercept,obstaclesPercept, psdPercept, collisionPercept);

	int frame = cameraMatrix.frameNumber = ballPercept.frameNumber;
	if(frame==0) return;

	// 1. store the camera data
	buffer.setCameraData(frame,cameraMatrix,cameraInfo);

	bool flagLandmarkSeen[maxPerceptType];
	int i;
	for(i=0;i<maxPerceptType;i++)
		flagLandmarkSeen[i]=false;

	for(i=0;i<landmarksPercept.numberOfFlags;i++) {
		Pose2D relativePosition = Pose2D();
		PerceptType type;
		
		relativePosition
			.rotate(landmarksPercept.flags[i].angle)
			.translate(landmarksPercept.flags[i].distance,0);

		switch(landmarksPercept.flags[i].type) {
		    case Flag::pinkAboveYellow:
				type = flagPinkYellow;
				break;
			case Flag::pinkAboveSkyblue:
				type = flagPinkBlue;
				break;
			case Flag::yellowAbovePink:
				type = flagYellowPink;
				break;
			case Flag::skyblueAbovePink:
				type = flagBluePink;
				break;
		}

		// 2. store seen flags
		buffer.setSeenLandmark(frame,type,relativePosition);
		flagLandmarkSeen[type]=true;
		
	}


	for(i=0;i<landmarksPercept.numberOfGoals;i++) {
		Pose2D relativePosition = Pose2D();
		PerceptType type;

		relativePosition
			.rotate(landmarksPercept.goals[i].angle)
			.translate(landmarksPercept.goals[i].distance,0);

		switch(landmarksPercept.goals[i].color) {
		case yellow:
				type = goalYellow;
				break;
		case skyblue:
				type = goalBlue;
				break;
		}

		// 3. store seen goals
		buffer.setSeenLandmark(frame,type,relativePosition);
		flagLandmarkSeen[type]=true;
		//log("yep goal seen");
	}

	// 4. store unseen landmarks
	for(i=0;i<maxPerceptType;i++)
		if(!flagLandmarkSeen[i])
			buffer.setLandmarkNotSeen(frame,(PerceptType)i);

	// 5. calculate true positions
	for(i=0;i<maxPerceptType;i++)
		buffer.setLandmarkTruePosition(frame,(PerceptType)i,getPlayer());

	if(m_positionSource.GetCurSel()==0) 
		handleMessageManualWorldState(frame);		// share our wisdom

}

void CStatisticsDlgBar::handleMessageWorldState(InMessage& message) {
	Player player;
	BallModel ballModel;
	PlayerPoseCollection playerPoseCollection;
	ObstaclesModel obstaclesModel;
	RobotState robotState;
	RobotPose robotPose;
	CameraMatrix cameraMatrix;
	CameraInfo cameraInfo;

	player = getPlayer();

    message.bin >> RECEIVE_WORLDSTATE(robotPose,
	      ballModel, playerPoseCollection, obstaclesModel, robotState, cameraMatrix, cameraInfo);

	if(robotPose.frameNumber==0) return;

	// store camera data
	buffer.setCameraData(robotPose.frameNumber,cameraMatrix,cameraInfo);

	// store estimated position
	buffer.setLocatorPosition(robotPose.frameNumber,robotPose);

	if(m_positionSource.GetCurSel()==0) // manual
		handleMessageManualWorldState(robotPose.frameNumber);		// share our wisdom

	if(m_positionSource.GetCurSel()==2) // selflocator gives "true" position
		buffer.setTrueRobotPosition(robotPose.frameNumber,robotPose);

	//getQueues().toGUI.out.text << "state="<<robotState.getState() <<" button "<<robotState.getButtonPressed(BodyPercept.Switches::backBack);
	//getQueues().toGUI.out.finishMessage(idText);
}



void CStatisticsDlgBar::handleMessageOracledWorldState(InMessage& message) {
	if(m_positionSource.GetCurSel()==1) {
		Player player;
		RobotPose oracledRobotPose;
		BallModel oracledBallModel;
		PlayerPoseCollection oracledPlayerPoseCollection;
		ObstaclesModel obstaclesModel;
		RobotState robotState;
		CameraMatrix cameraMatrix;
		CameraInfo cameraInfo;

		message.bin >> RECEIVE_WORLDSTATE(oracledRobotPose,oracledBallModel,oracledPlayerPoseCollection,obstaclesModel,robotState, cameraMatrix, cameraInfo);

		if(oracledRobotPose.frameNumber==0) return;

		// store camera data (why not)
		buffer.setCameraData(oracledRobotPose.frameNumber,cameraMatrix,cameraInfo);

		// store estimated position
		buffer.setTrueRobotPosition(oracledRobotPose.frameNumber,oracledRobotPose);
	}
}

void CStatisticsDlgBar::handleMessageManualWorldState(unsigned long frameNumber) {
  double tX,tY,tPhi;
  tX = getDouble(m_positionTrueX);
  tY = getDouble(m_positionTrueY);
  tPhi = getDouble(m_positionTruePhi)/180*M_PI;
  Pose2D truePosition;

  truePosition.translate(tX,tY).rotate(tPhi);

  if(buffer.getLatest()->framesSinceLastLandmark<KIDNAP_DETECTION_FRAMES) 
	buffer.setTrueRobotPosition(frameNumber,truePosition);
  else 
	buffer.setKidnapped(frameNumber);
}


void CStatisticsDlgBar::OnPaint() {
	CPaintDC dc( this ); // Device context for painting
	dc.SelectObject(&fSmall);
	dc.Rectangle(ox-30,oy-16,ox+10*ow+5+43,oy+7*oh+10);
	dc.SetBkMode(TRANSPARENT);

	/*
	bool infoscreen = true;
	if(MEASURE_LENGTH-timestatus < 4) {
		dc.SelectObject(&fGiant);
		dc.SetTextColor(RGB(255,0,0));
		CString s;
		s.Format("%d",MEASURE_LENGTH-timestatus);
		dc.TextOut(ox-20,oy-50,s);
		dc.SelectObject(&fBig);
		dc.SetTextColor(RGB(255,0,0));
		dc.TextOut(ox+100,oy,"STOP");
	} else 
	if(MEASURE_BLIND-timestatus < 4 && timestatus<MEASURE_BLIND) {
		dc.SelectObject(&fGiant);
		dc.SetTextColor(RGB(255,0,0));
		CString s;
		s.Format("%d",MEASURE_BLIND-timestatus);
		dc.TextOut(ox-20,oy-50,s);
		dc.SelectObject(&fBig);
		dc.SetTextColor(RGB(255,0,0));
		dc.TextOut(ox+100,oy,"START");
	} else
	if(timestatus<5 && m_positionPredefined.GetCurSel()>=1 && timestatus>=0) {
		dc.SelectObject(&fBig);
		dc.SetTextColor(RGB(0,0,0));
		CString s;
		s.Format("Position %d",m_positionPredefined.GetCurSel());
		dc.TextOut(ox-10,oy,s);
		dc.SetTextColor(RGB(255,0,0));
		m_positionPredefined.GetLBText(m_positionPredefined.GetCurSel(),s);
		CString s2,s3;
		if(s.Find(" - ")>=0) {
			s2 = s.Mid(3,s.Find(" - ")-3);
			s3 = s.Mid(s.Find(" - ")+3);
		} else {
			s2 = "";
			s3=s.Mid(3);
		}
		dc.TextOut(ox-10,oy+50,s2);
		dc.TextOut(ox-10,oy+100,s3);
	} else infoscreen = false;
	
	if(infoscreen) {
		buffer.spreadTrueData();//before the buffer is completly flushed in the meantime, spread data
		buffer.evaluate(distBound);
		buffer.updateStatistics();

	}
	else {
	*/
		// good moment for some evaluation
		paintBackground(dc);
		paintSelectedFrame(dc);
		paintPercepts(dc);
		paintStatistics(dc);
	//}
	

	buffer.removeChangedFlag();
}

void CStatisticsDlgBar::paintBackground(CPaintDC &dc) {
    
	dc.SelectObject(&fSmall);

	// draw the bitmap
	CBitmap BmpField;
	BmpField.LoadBitmap(IDB_STATISTICS);
	CDC MemDCField;
	MemDCField.CreateCompatibleDC(&dc);
	CBitmap *BmpPrevious = MemDCField.SelectObject(&BmpField);
	dc.BitBlt(ox-23, oy+2, 21,7*oh, &MemDCField, 0, 0, SRCCOPY);
	dc.SelectObject(BmpPrevious);

	CString sframe;
	int framesPainted=0;
	unsigned long lastFrame=0;
	int dx1,dy1;

	dc.SelectObject(brushBlue);
	dc.Rectangle(ox,oy+2+oh+oh*selectedType,ox+(ocolumns-1)*oh+50,oy+2*oh-2+oh*selectedType);
	dc.SelectStockObject(WHITE_BRUSH);


	// first loop: draw frame numbers & selection area
	framesPainted=0;
	for(int i=0; (i<MAX_FRAMES) && (framesPainted<ocolumns);i++) {
		bool painted = false;
		int frameNumber = buffer.latestFrame-i;
		if(frameNumber<0) break;

		SingleFrame * frame = buffer.getSingleFrame(frameNumber,false);

		if((frame!=NULL) && (!frame->isEmpty)){

			dx1=ox+(ocolumns-framesPainted-1)*ow;
			dy1=oy-10;

			
			if(frameNumber==selectedFrame) {
				// this landmark has been selected
				dy1=oy+oh*selectedType;
				dc.SelectObject(brushBlue);
				dc.Ellipse(dx1-5,dy1-5+oh,dx1+ow+5,dy1+2*oh+5);
				dc.SelectStockObject(WHITE_BRUSH);
			}
			

			dy1=oy-10;
			if(frame->hasBeenChanged()) {
				dc.SelectStockObject(WHITE_PEN);
				dc.SelectObject(brushFade[5]);
				dc.Rectangle(dx1+3,dy1-2,dx1+ow+3,dy1+10);
				dc.SelectStockObject(BLACK_PEN);
			}

			sframe.Format("%d",frameNumber);
			dc.TextOut(dx1+5,dy1,sframe);
			framePaintedAt[framesPainted] = buffer.latestFrame-i;
			framesPainted++;

		}
	}
}

void CStatisticsDlgBar::paintSelectedFrame(CPaintDC &dc) {
	CString sframe;

	SingleFrame * frame = buffer.getSingleFrame(selectedFrame,false);
	if(frame==NULL) return;
	if(frame->isEmpty) return;

	if(selectedType==-1) {
		// locator has been selected

		// estimated position
		if(frame->locator.estimatedPerceptValid)
			sframe.Format("est. position: x=%5.0f y=%5.0f phi=%.0f",
					frame->locator.estimatedPosition.translation.x,
					frame->locator.estimatedPosition.translation.y,
					frame->locator.estimatedPosition.rotation*180/M_PI);
		else 
			sframe = "estimated position: not valid";
		m_History_Estimated.SetWindowText(sframe);

		// true position
		if(frame->trueData.validRobotPosition)
			sframe.Format("true position: x=%5.0f y=%5.0f phi=%.0f",
					frame->trueData.trueRobotPosition.translation.x,
					frame->trueData.trueRobotPosition.translation.y,
					frame->trueData.trueRobotPosition.rotation*180/M_PI);
		else 
			sframe = "true position: not valid";
		m_History_True.SetWindowText(sframe);

		// both valid: calculate difference
		if((frame->locator.estimatedPerceptValid)&&(frame->trueData.validRobotPosition)) {
				double dist = sqrt(
					sqr(
						frame->locator.estimatedPosition.translation.x-
						frame->trueData.trueRobotPosition.translation.x)
					+
					sqr(
						frame->locator.estimatedPosition.translation.y-
						frame->trueData.trueRobotPosition.translation.y)
				);
				double phi = 
						((frame->locator.estimatedPosition.rotation-
						frame->trueData.trueRobotPosition.rotation))*180/M_PI;
				while (phi>180) phi -=360;
				while (phi<-180) phi +=360;
				sframe.Format("error %.0f mm, %.0f deg",dist,phi);
				m_History_EstimatedRelative.SetWindowText(sframe);

				dist = frame->locator.statistics.SumErrorDist/frame->locator.statistics.TotalCount;
				phi = frame->locator.statistics.SumErrorPhi/frame->locator.statistics.TotalCount;

				sframe.Format("average err %.0f mm, %.0f deg",dist,phi);
				m_History_TrueRelative.SetWindowText(sframe);
		} else {
			sframe = "";
			m_History_EstimatedRelative.SetWindowText(sframe);
			m_History_TrueRelative.SetWindowText(sframe);
		}

	} else {
		// flag or goal has been selected

		// true position
		if(frame->landmark[selectedType].trueAbsolutePositionValid)
			sframe.Format("true position: x=%5.0f y=%5.0f phi=%.0f",
					frame->landmark[selectedType].trueAbsolute.translation.x,
					frame->landmark[selectedType].trueAbsolute.translation.y,
					frame->landmark[selectedType].trueAbsolute.rotation*180/M_PI);
		else 
			sframe = "true position: not valid";
		m_History_True.SetWindowText(sframe);

		sframe = "";
		m_History_Estimated.SetWindowText(sframe);

		// relative estimated position
		if(frame->landmark[selectedType].trueVisibility==visibleYes || frame->landmark[selectedType].trueVisibility==visiblePartially) {
			if(frame->landmark[selectedType].seenVisibility == visibleYes) 
				sframe.Format("relative seen: x=%5.0f y=%5.0f phi=%.0f",
						frame->landmark[selectedType].seenRelative.translation.x,
						frame->landmark[selectedType].seenRelative.translation.y,
						frame->landmark[selectedType].seenRelative.rotation*180/M_PI);
			else 
				sframe = "landmark not seen";

		} else 
			sframe = "";
		m_History_EstimatedRelative.SetWindowText(sframe);

		switch(frame->landmark[selectedType].trueVisibility) {
		case visibleYes: 
			sframe = (frame->landmark[selectedType].trueVisibility==visibleYes)?
				"object visible":"object visible but not detected (FALSE NEGATIVE)";
			break;
		case visibleNo: 
			sframe = (frame->landmark[selectedType].trueVisibility==visibleYes)?
				"object invisible but detected (FALSE POSITIVE)":"object invisible";
			break;
		case visiblePartially: 
			sframe = (frame->landmark[selectedType].trueVisibility==visibleYes)?
				"object partially visible":"object partially visible";
			break;
		case visibleUnknown: 
			sframe = "visibility information not available";
			break;
		default:
			sframe = "??";
		}
		m_History_TrueRelative.SetWindowText(sframe);

	}
	// paint selection rectangle (again)..
	dc.SelectObject(brushBlue);
	dc.SelectStockObject(NULL_PEN);
	dc.Rectangle(ox+1,oy+2+oh+oh*selectedType+1,ox+(ocolumns-1)*oh+50,oy+2*oh-2+oh*selectedType);
	dc.SelectStockObject(WHITE_BRUSH);
	dc.SelectStockObject(BLACK_PEN);
}

void CStatisticsDlgBar::paintPercepts(CPaintDC &dc) {
	// second loop: paint events & percepts
	dc.SelectObject(&fError);
	int framesPainted=0;
	framesPainted=0;
	for(int i=0; (i<MAX_FRAMES) && (framesPainted<ocolumns);i++) {
		bool painted = false;
		int frameNumber = buffer.latestFrame-i;
		if(frameNumber<0) break;

		SingleFrame * frame = buffer.getSingleFrame(frameNumber,false);

		if((frame!=NULL) && (!frame->isEmpty)){

			int dx1=ox+(ocolumns-framesPainted-1)*ow;
			int dy1=oy;

			// locator
			if(frame->locator.estimatedPerceptValid) {
				if(frame->locator.withinRange) {
					dc.SelectObject(brushLime);
					dc.Ellipse(dx1+5,dy1+5,dx1+15,dy1+15);
				} else {
					dc.SelectObject(brushRed);
					dc.Ellipse(dx1+5,dy1+5,dx1+15,dy1+15);
				}
			}

			// landmarks
			for(int type=0;type<maxPerceptType;type++) {
				dy1=oy+oh*type+oh;
				bool error=false;

				Visibility seen = frame->landmark[type].seenVisibility;
				Visibility real = frame->landmark[type].trueVisibility;

				switch(real) {
					case visibleYes:
						// green background
						dc.SelectObject(brushGreen);
						dc.SelectStockObject(NULL_PEN);
						dc.Rectangle(dx1,dy1+10,dx1+ow+1,dy1+oh-5);
						dc.SelectStockObject(WHITE_BRUSH);
						dc.SelectStockObject(BLACK_PEN);
						if(seen==visibleYes) {
							// true positive
							dc.SelectObject(brushLime);
							dc.Ellipse(dx1+5,dy1+5,dx1+15,dy1+15);
						} else if(seen=visibleNo) {
							// false negative
							error=true;
						}
						break;
					case visiblePartially:
					case visibleUnknown:
						// no background
						if(seen==visibleYes) {
							dc.SelectObject(brushLime);
							dc.Ellipse(dx1+5,dy1+5,dx1+15,dy1+15);
						}
						break;
					case visibleNo:
						// grey background
						dc.SelectObject(brushGrey);
						dc.SelectStockObject(NULL_PEN);
						dc.Rectangle(dx1,dy1+10,dx1+ow+1,dy1+oh-5);
						dc.SelectStockObject(WHITE_BRUSH);
						dc.SelectStockObject(BLACK_PEN);
						if(seen==visibleYes) {
							// false positive
							dc.SelectObject(brushLime);
							dc.Ellipse(dx1+5,dy1+5,dx1+15,dy1+15);
							error = true;
						} else {
							// true negative
						}
						break;
				}
				if(error) {
					dc.SetTextColor(RGB(255,0,0));
					dc.TextOut(dx1+7,dy1-2,"!");
				}

			}
			framesPainted++;

		}
	}
}

void CStatisticsDlgBar::paintStatistics(CPaintDC &dc) {
	int dx1=ox+ow*ocolumns+5;
	int dy1=oy-15;

	dc.SelectObject(&fSmall);
	dc.SetTextColor(RGB(0,0,0));
	//dc.TextOut(dx1,dy1,"RECALL /");
	//dc.TextOut(dx1,dy1+8,"PRECISION");
	dc.TextOut(dx1,dy1,"TP::FP");
	dc.TextOut(dx1,dy1+8,"TN::FN");

	dc.SelectObject(&fNormal);
	dc.TextOut(dx1+27,dy1,"%");

	CString s1,s2,s3;
	// recall = truepositives / (truepos + falsenegative)
	// precision = truepositives/tp+fn)
	SingleFrame * frame = buffer.getLatest();

	//locator stats..
	dy1 = oy+2;
	if(frame->locator.statistics.TotalCount>0) {
		dc.SelectObject(&fNormal);
		s3.Format("%d",100*frame->locator.statistics.WithinRange/frame->locator.statistics.TotalCount);
		dc.TextOut(dx1+17,dy1+2,s3);
	}

	// landmarks..
	for(int type=0;type<maxPerceptType;type++) {
		dy1=oy+oh+oh*type+2;

		int tp,fp,tn,fn,c;
		tp = frame->landmark[type].statistics.TruePositives;
		fp = frame->landmark[type].statistics.FalsePositives;
		tn = frame->landmark[type].statistics.TrueNegatives;
		fn = frame->landmark[type].statistics.FalseNegatives;
		c = frame->landmark[type].statistics.TotalCount;

		dc.SelectObject(&fSmall);
		s1.Format("%d::%d",tp,fp);
		s2.Format("%d::%d",tn,fn);

		dc.TextOut(dx1,dy1,s1);
		dc.TextOut(dx1,dy1+8,s2);

		if(tp+fn>0 && tn+fp>0) {// (tp+fn)/(tp) + (tn+fp)/tn
			dc.SelectObject(&fNormal);
			s3.Format("%d",((tp)*50)/(tp+fn)+((tn)*50)/(tn+fp));// fehler 50:50
			dc.TextOut(dx1+17,dy1+2,s3);
		}
		
	}
}


// ************************************************************************** class TrueData
TrueData::TrueData() {
	validCamera = false;
	validRobotPosition = false;
	kidnapped = false;
	changed = false;
	validError = false;
}

void TrueData::setCameraData(CameraMatrix &matrix,CameraInfo &info) {
	cameraMatrix = matrix;
	cameraInfo = info;
	validCamera = true;
	changed = true;
}

void TrueData::setTrueRobotPosition(Pose2D &truePosition) {
	trueRobotPosition = truePosition;
	validRobotPosition = true;
	changed = true;
}

void TrueData::setKidnapped() {
	changed = true;
	validRobotPosition = false;
	kidnapped = true;//do not spread data to this frame
}

void TrueData::setRecovered() {
	changed = true;
	validError = true;
}



// ************************************************************************** class LandmarkStatistics
LandmarkStatistics::LandmarkStatistics() {
	TruePositives = 0;
	TrueNegatives = 0;
	FalsePositives = 0;
	FalseNegatives = 0;
	TotalCount = 0;
	framesSinceLastLandmark = KIDNAP_DETECTION_FRAMES+1;	// if we don't know better, assume we are kidnapped
}

// ************************************************************************** class LocatorStatistics
LocatorStatistics::LocatorStatistics() {
	WithinRange = 0;
	TotalCount = 0;
	SumErrorDist = 0;
	SumErrorPhi = 0;
}


// ************************************************************************** class LandmarkPercept
LandmarkPercept::LandmarkPercept() {
	changed = false;
	seenPerceptValid = false;
	trueAbsolutePositionValid = false;
	seenVisibility = visibleUnknown;
	trueVisibility = visibleUnknown;
	statistics = LandmarkStatistics();
}

void LandmarkPercept::setSeenLandmark(Pose2D &relativePosition) {
	seenPerceptValid = true;
	seenVisibility = visibleYes;
	seenRelative = relativePosition;
	changed = true;
}

void LandmarkPercept::setLandmarkNotSeen() {
	seenPerceptValid = true;
	seenVisibility = visibleNo;
	seenRelative = Pose2D();
	changed = true;
}

void LandmarkPercept::setLandmarkTruePosition(PerceptType atype,Pose2D &truePosition) {
	changed = true;
	trueAbsolutePositionValid = true;
	trueAbsolute = truePosition;
	type = atype;
}

void LandmarkPercept::updateVisibility(TrueData &data) {
	if(!changed) return;
	if(!trueAbsolutePositionValid) return;
	if(!data.validCamera) return;

	changed = true;
	trueRelative = trueAbsolute - data.trueRobotPosition;

	if((type == flagPinkBlue)||(type == flagBluePink)||(type == flagYellowPink)||(type == flagPinkYellow))
			trueVisibility = checkVisibility(
				trueRelative,
				Vector2<double>(-150,-50),Vector2<double>(+150,+500),
				data.cameraMatrix,data.cameraInfo);

	if((type == goalBlue)||(type == goalYellow))
			trueVisibility  = checkVisibility(
				trueRelative,
				Vector2<double>(-400,0),Vector2<double>(+400,+300),
				data.cameraMatrix,data.cameraInfo);

}

/*
looks a bit clumbsy, but this is how it works:

  relativePosition is a 2D relative position wrt the robot on the field
  lowerLeft and upperRight span a window in the YZ plane
  
  Keep in mind that YZ is always the orthongonal plane (X is more or less the viewing direction,
  so we extend the seen position in the YZ plane.)

  Could have been done nicer, though..

*/
Visibility LandmarkPercept::checkVisibility(
	Pose2D &relativePosition, 
	Vector2<double> lowerLeft, Vector2<double> upperRight, 
	CameraMatrix &cameraMatrix, CameraInfo &cameraInfo) {

	int visible[2];

	for(int insideOutside=0;insideOutside<2;insideOutside++) {
		visible[insideOutside]=0;

		if(checkVisibility(
			Vector3<int>(
				(int)(relativePosition.translation.x),
				(int)(relativePosition.translation.y + lowerLeft.x),
				(int)(lowerLeft.y)),insideOutside,
			cameraMatrix,cameraInfo)) visible[insideOutside]++;

		if(checkVisibility(
			Vector3<int>(
				(int)(relativePosition.translation.x),
				(int)(relativePosition.translation.y + lowerLeft.x),
				(int)(upperRight.y)),insideOutside,
			cameraMatrix,cameraInfo)) visible[insideOutside]++;

		if(checkVisibility(
			Vector3<int>(
				(int)(relativePosition.translation.x),
				(int)(relativePosition.translation.y + upperRight.x),
				(int)(upperRight.y)),insideOutside,
			cameraMatrix,cameraInfo)) visible[insideOutside]++;

		if(checkVisibility(
			Vector3<int>(
				(int)(relativePosition.translation.x),
				(int)(relativePosition.translation.y + upperRight.x),
				(int)(lowerLeft.y)),insideOutside,
			cameraMatrix,cameraInfo)) visible[insideOutside]++;
	}

	if(relativePosition.rotation>M_PI) relativePosition.rotation-=M_PI;
	if(relativePosition.rotation<M_PI) relativePosition.rotation+=M_PI;
	/*
	if(relativePosition.translation.x<0 && (relativePosition.rotation<-3/4*M_PI || relativePosition.rotation>3/4*M_PI)) {	// behind us!!
		visible[0]=0;
		visible[1]=0;
	}*/
	
//	getQueues().toGUI.out.text << "relativeX: "<<relativePosition.translation.x<<" relativeY:"<<relativePosition.translation.y<<"in:"<<visible[0]<<"out:"<<visible[1];
//	getQueues().toGUI.out.finishMessage(idText);


	if(visible[0]==4) return(visibleYes);
	if(visible[1]==0) return(visibleNo);

	return(visiblePartially);
	


}

bool LandmarkPercept::checkVisibility(
	Vector3<int> &relativePositionInt, int testInsideOutside,
	CameraMatrix &cameraMatrix, CameraInfo &cameraInfo) {

	Vector2<int> pixelPosition;

	Geometry::calculatePointInImage(relativePositionInt,cameraMatrix,cameraInfo,pixelPosition);

	bool result=false;
	if(testInsideOutside==0) {
		result = (Geometry::isPointInsideRectangle(
				Vector2<int>(CAMERA_PIXEL_BORDER,CAMERA_PIXEL_BORDER),
				Vector2<int>(cameraInfo.resolutionWidth-CAMERA_PIXEL_BORDER,cameraInfo.resolutionHeight-CAMERA_PIXEL_BORDER),
				pixelPosition));
	} else {
		result = (Geometry::isPointInsideRectangle(
				Vector2<int>(-CAMERA_PIXEL_BORDER*5,-CAMERA_PIXEL_BORDER*5),
				Vector2<int>(cameraInfo.resolutionWidth+CAMERA_PIXEL_BORDER*5,cameraInfo.resolutionHeight+CAMERA_PIXEL_BORDER*5),
				pixelPosition));
	}

//	getQueues().toGUI.out.text << "testing:"<<pixelPosition.x<<"=x y="<<pixelPosition.y<<" res.width"<<cameraInfo.resolutionWidth<<" insideoutside="<<testInsideOutside<<"result:"<<result;
//	getQueues().toGUI.out.finishMessage(idText);

	return(result);
}



void LandmarkPercept::updateStatistics(LandmarkPercept &previousPercept) {
//	if((!previousPercept.changed)&&(!changed)) 
//		return;

	statistics = previousPercept.statistics;
	changed = true;
	if(trueVisibility==visibleYes) {
		// should be visible
		if(seenVisibility==visibleYes) {
			statistics.TruePositives++;
			statistics.TotalCount++;
		} else if(seenVisibility==visibleNo) {
			statistics.FalseNegatives++;
			statistics.TotalCount++;
		}
	} else if(trueVisibility==visibleNo) {
		// should not be visible
		if(seenVisibility==visibleNo) {
			statistics.TrueNegatives++;
			statistics.TotalCount++;
		} else if(seenVisibility==visibleYes) {
			statistics.FalsePositives++;
			statistics.TotalCount++;
		}
		
	}
	if(seenVisibility==visibleYes)
		statistics.framesSinceLastLandmark=0;
	else
		statistics.framesSinceLastLandmark++;
}


// ************************************************************************** class LocatorPercept
LocatorPercept::LocatorPercept() {
	changed = false;
	estimatedPerceptValid = false;
	withinRangeValid = false;
	validError = false;
	statistics = LocatorStatistics();
}

void LocatorPercept::setEstimatedPosition(Pose2D &position) {
	changed = true;
	estimatedPerceptValid = true;
	estimatedPosition = position;
}

void LocatorPercept::updatePrecision(TrueData &data,double distBound) {
	if(!changed) return;
	if(!estimatedPerceptValid) return;
	if(!data.validRobotPosition) return;	// so koennen wir nicht arbeiten..

	double dist = sqrt(
			sqr(estimatedPosition.translation.x - data.trueRobotPosition.translation.x)
		+	sqr(estimatedPosition.translation.y - data.trueRobotPosition.translation.y));
	
	this->withinRange = (dist < distBound);
	this->withinRangeValid = true;
	this->errDist = dist;
	this->errPhi = (2*M_PI + estimatedPosition.rotation - data.trueRobotPosition.rotation);
	this->validError = data.validError;
	while(this->errPhi>M_PI) this->errPhi -= 2*M_PI;
	this->errPhi = fabs(this->errPhi);
	changed = true;
}

void LocatorPercept::updateStatistics(LocatorPercept &previousPercept) {
//	if((!previousPercept.changed)&&(!changed)) 
//		return;

	changed = true;

	statistics = previousPercept.statistics;
	if(withinRangeValid && validError) {
		statistics.TotalCount++;
		if(withinRange)
			statistics.WithinRange++;
		statistics.SumErrorDist += errDist;
		statistics.SumErrorPhi += errPhi;
	}
}
// ************************************************************************** class SingleFrame
SingleFrame::SingleFrame() {
	changed = false;
	isEmpty = true;
	frame = 0;
	framesSinceLastLandmark = KIDNAP_DETECTION_FRAMES+1;

	trueData = TrueData();
	locator = LocatorPercept();
	for(int i=0;i<maxPerceptType;i++) {
		landmark[i] = LandmarkPercept();
	}
}

void SingleFrame::spreadTrueData(SingleFrame &previousFrame) {
	if(trueData.kidnapped) return;//nothing to do

	// grab data from previous frame if neccessary
	if(!trueData.validCamera) {
		if(trueData.validCamera = previousFrame.trueData.validCamera) {
			trueData.setCameraData(previousFrame.trueData.cameraMatrix,previousFrame.trueData.cameraInfo);
		}
	}

	if(!trueData.validRobotPosition) {
		if(trueData.validRobotPosition = previousFrame.trueData.validRobotPosition) {
			trueData.setTrueRobotPosition(previousFrame.trueData.trueRobotPosition);
		}
	}
}


void SingleFrame::updateStatistics(SingleFrame &previousFrame) {
	/*
	if(!trueData.validRobotPosition && !kidnapped) {
		if(trueData.validRobotPosition = previousFrame.trueData.validRobotPosition) {
			trueData.setTrueRobotPosition(previousFrame.trueData.trueRobotPosition);
		}
	}
	*/

	// update locator statistics
	locator.updateStatistics(previousFrame.locator);

	framesSinceLastLandmark = -1;
	// and landmark statistics
	for(int i=0;i<maxPerceptType;i++) {
		landmark[i].updateStatistics(previousFrame.landmark[i]);
		if((framesSinceLastLandmark ==-1) || (framesSinceLastLandmark>landmark[i].statistics.framesSinceLastLandmark))
			framesSinceLastLandmark = landmark[i].statistics.framesSinceLastLandmark;
	}

}

void SingleFrame::evaluate(double distBound) {
	if(isEmpty) return;
	if(!hasBeenChanged() ) return;
	if(!trueData.validRobotPosition) return;

	// update locator statistics
	locator.updatePrecision(trueData,distBound);

	// and landmark statistics
	for(int i=0;i<maxPerceptType;i++) 
		landmark[i].updateVisibility(trueData);

}

bool SingleFrame::hasBeenChanged() {
	bool result=false;
	result |= trueData.changed;
	result |= locator.changed ;
	for(int i=0;i<maxPerceptType;i++) 
		result |= landmark[i].changed;
	result |= changed;
	return(result);
	
}


void SingleFrame::removeChangedFlag() {
	// clean up
	trueData.changed = false;
	locator.changed = false;
	for(int i=0;i<maxPerceptType;i++) 
		landmark[i].changed = false;
	changed = false;
}


void SingleFrame::setSeenLandmark(PerceptType type,Pose2D &relativePosition) {
	isEmpty = false;
	landmark[(int)type].setSeenLandmark(relativePosition);
}

void SingleFrame::setLandmarkNotSeen(PerceptType type) {
	isEmpty = false;
	landmark[(int)type].setLandmarkNotSeen();
}

void SingleFrame::setLandmarkTruePosition(PerceptType type, Player player) {
	// calulate true position of landmark, depending on team color
	int flip = player.getTeamColor() == Player::blue ? -1 : 1;
	Pose2D truePosition;
	switch(type) {
		case flagBluePink:
			truePosition = Pose2D(xPosFrontFlags*flip,yPosLeftFlags*flip);
			break;
		case flagPinkBlue:
			truePosition = Pose2D(xPosFrontFlags*flip,yPosRightFlags*flip);
			break;
		case flagYellowPink:
			truePosition = Pose2D(xPosBackFlags*flip,yPosLeftFlags*flip);
			break;
		case flagPinkYellow:
			truePosition = Pose2D(xPosBackFlags*flip,yPosRightFlags*flip);
			break;
		case goalBlue:
			truePosition = Pose2D(xPosOpponentGoal*flip,yPosCenterGoal*flip);
			break;
		case goalYellow:
			truePosition = Pose2D(xPosOwnGoal*flip,yPosCenterGoal*flip);
			break;
	}
	landmark[(int)type].setLandmarkTruePosition(type,truePosition);
}

void SingleFrame::setCameraData(CameraMatrix &matrix,CameraInfo &info) {
	isEmpty = false;
	trueData.setCameraData(matrix,info);
}

void SingleFrame::setLocatorPosition(Pose2D &estimatedRobotPosition) {
	isEmpty = false;
	locator.setEstimatedPosition(estimatedRobotPosition);
}

void SingleFrame::setTrueRobotPosition(Pose2D &trueRobotPosition) {
	isEmpty = false;
	trueData.setTrueRobotPosition(trueRobotPosition);
}

void SingleFrame::setKidnapped() {
	isEmpty = false;
	trueData.setKidnapped();

}

void SingleFrame::setRecovered() {
	isEmpty = false;
	trueData.setRecovered();

}


// ************************************************************************** class MainBuffer
MainBuffer::MainBuffer() {
	latestFrame = 0;
	latestIndex = 0;
	for(int i = 0;i<MAX_FRAMES;i++)
			frame[i] = SingleFrame();
//	log("MainBuffer: created");
}

SingleFrame * MainBuffer::getSingleFrame(unsigned long frameNumber,bool forceExistance) {
	if((MAX_FRAMES+frameNumber<=latestFrame ) ||
		(frameNumber>=latestFrame + MAX_FRAMES)) {
		if(!forceExistance) return(NULL);
		// flush the whole buffer
		for(int i = 0;i<MAX_FRAMES;i++)
			frame[i] = SingleFrame();
		latestFrame = frameNumber;
		latestIndex = 0;
		return(&frame[latestIndex]);
	} else if(frameNumber>latestFrame) {
		if(!forceExistance) return(NULL);
		// shift the buffer
		for(unsigned int i = latestFrame+1; i<=frameNumber;i++) {
			// row, row, row the boat
			latestIndex = (latestIndex+1)%MAX_FRAMES;
			latestFrame++;
			frame[latestIndex] = SingleFrame();
		}
		//getQueues().toGUI.out.text << "created frame "<<frameNumber;
		//getQueues().toGUI.out.finishMessage(idText);

		return(&frame[latestIndex]);
	} else {
		// already in the buffer
		return(&frame[(latestIndex-(latestFrame-frameNumber)+MAX_FRAMES)%MAX_FRAMES]);
	}
}

SingleFrame * MainBuffer::getLatest() {
	return(&frame[latestIndex]);
}

void MainBuffer::spreadTrueData() {
	for(int i=2;i<=MAX_FRAMES;i++) {
		frame[(i+latestIndex) % MAX_FRAMES]
			.spreadTrueData(
				frame[(i+latestIndex-1) % MAX_FRAMES]);
	}
}

void MainBuffer::updateStatistics() {
	for(int i=2;i<=MAX_FRAMES;i++) {
		frame[(i+latestIndex) % MAX_FRAMES]
			.updateStatistics(
				frame[(i+latestIndex-1) % MAX_FRAMES]);
	}
}

void MainBuffer::evaluate(double distBound) {
	for(int i=1;i<=MAX_FRAMES;i++)
		frame[(i+latestIndex) % MAX_FRAMES].evaluate(distBound);
}

void MainBuffer::removeChangedFlag() {
	for(int i=1;i<=MAX_FRAMES;i++)
		frame[(i+latestIndex) % MAX_FRAMES].removeChangedFlag();
}

void MainBuffer::setSeenLandmark(unsigned long frame, PerceptType type,Pose2D &relativePosition) {
	getSingleFrame(frame)
		->setSeenLandmark(type,relativePosition);
}

void MainBuffer::setLandmarkNotSeen(unsigned long frame, PerceptType type) {
	getSingleFrame(frame)
		->setLandmarkNotSeen(type);
}

void MainBuffer::setCameraData(unsigned long frame, CameraMatrix &matrix,CameraInfo &info) {
	getSingleFrame(frame)
		->setCameraData(matrix,info);
}

void MainBuffer::setLocatorPosition(unsigned long frame, Pose2D &estimatedRobotPosition) {
	getSingleFrame(frame)
		->setLocatorPosition(estimatedRobotPosition);

}

void MainBuffer::setTrueRobotPosition(unsigned long frame, Pose2D &trueRobotPosition) {
	getSingleFrame(frame)
		->setTrueRobotPosition(trueRobotPosition);
}

void MainBuffer::setLandmarkTruePosition(unsigned long frame, PerceptType type, Player player) {
	getSingleFrame(frame)
		->setLandmarkTruePosition(type,player);
}

void MainBuffer::setKidnapped(unsigned long frame) {
	getSingleFrame(frame)
		->setKidnapped();
}

void log(CString s) {
  getQueues().toGUI.out.text << s;
  getQueues().toGUI.out.finishMessage(idText);
}


/*
* Change Log:
*
* $Log$
*
*/
