/**
* @file WLANDeployDialog.cpp
*
* File created by the members of the Dutch Aibo Team 2005
*
* @author Jurgen Sturm, Silvain van Weers, Stefan Leijnen, Nico Assink, Niek Wijngaards 
*/
/**
* @file WLANDeployDialog.cpp
* 
* Implementation of WLAN new software deployment dialog in the RobotControl project.
*
* @author Jurgen Sturm
*/

#include "StdAfx.h"

#include "WLANDeployDialog.h"
#include "RobotControl.h"
#include "Tools/Player.h"
#include "Tools/CPing/CPing.h"

#include "Tools/Streams/InStreams.h"

CWLANDeployDialog::CWLANDeployDialog(CWnd* pParent /*=NULL*/)
: CDialog(CWLANDeployDialog::IDD, pParent)
{
  //{{AFX_DATA_INIT(CWLANDeployDialog)

  //}}AFX_DATA_INIT

}


void CWLANDeployDialog::DoDataExchange(CDataExchange* pDX)
{

  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CWLANDeployDialog)
  DDX_Control(pDX, IDC_WLAN_DEPLOY_WLAN_COLTABLE, m_WLANUpdatesColortable);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_WLAN_BINARIES, m_WLANUpdatesBinaries);


  DDX_Control(pDX, IDC_WLAN_DEPLOY_MS, m_MS);

  DDX_Text(pDX, IDC_WLAN_DEPLOY_AIBO1, m_Aibo[0]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_AIBO2, m_Aibo[1]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_AIBO3, m_Aibo[2]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_AIBO4, m_Aibo[3]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_AIBO5, m_Aibo[4]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_AIBO6, m_Aibo[5]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_AIBO7, m_Aibo[6]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_AIBO8, m_Aibo[7]);

  DDX_Text(pDX, IDC_WLAN_DEPLOY_STICK1, m_Stick[0]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_STICK2, m_Stick[1]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_STICK3, m_Stick[2]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_STICK4, m_Stick[3]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_STICK5, m_Stick[4]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_STICK6, m_Stick[5]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_STICK7, m_Stick[6]);
  DDX_Text(pDX, IDC_WLAN_DEPLOY_STICK8, m_Stick[7]);

  DDX_Control(pDX, IDC_WLAN_DEPLOY_BINARIES1, m_Binaries[0]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_BINARIES2, m_Binaries[1]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_BINARIES3, m_Binaries[2]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_BINARIES4, m_Binaries[3]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_BINARIES5, m_Binaries[4]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_BINARIES6, m_Binaries[5]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_BINARIES7, m_Binaries[6]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_BINARIES8, m_Binaries[7]);

  DDX_Control(pDX, IDC_WLAN_DEPLOY_COLORTABLE1, m_Colortable[0]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_COLORTABLE2, m_Colortable[1]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_COLORTABLE3, m_Colortable[2]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_COLORTABLE4, m_Colortable[3]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_COLORTABLE5, m_Colortable[4]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_COLORTABLE6, m_Colortable[5]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_COLORTABLE7, m_Colortable[6]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_COLORTABLE8, m_Colortable[7]);

  DDX_Control(pDX, IDC_WLAN_DEPLOY_PING1, m_Ping[0]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_PING2, m_Ping[1]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_PING3, m_Ping[2]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_PING4, m_Ping[3]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_PING5, m_Ping[4]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_PING6, m_Ping[5]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_PING7, m_Ping[6]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_PING8, m_Ping[7]);

  DDX_Control(pDX, IDC_WLAN_DEPLOY_INDICATOR1, m_Indicator[0]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_INDICATOR2, m_Indicator[1]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_INDICATOR3, m_Indicator[2]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_INDICATOR4, m_Indicator[3]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_INDICATOR5, m_Indicator[4]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_INDICATOR6, m_Indicator[5]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_INDICATOR7, m_Indicator[6]);
  DDX_Control(pDX, IDC_WLAN_DEPLOY_INDICATOR8, m_Indicator[7]);

  //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CWLANDeployDialog, CDialog)
//{{AFX_MSG_MAP(CWLANDeployDialog)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_PINGALL, OnPingAll)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_REFRESH, OnRefresh)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_REFRESHALL, OnRefreshAll)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_SNAPSHOT, OnSnapshot)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_STICK1, OnCopyStick0)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_STICK2, OnCopyStick1)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_STICK3, OnCopyStick2)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_STICK4, OnCopyStick3)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_STICK5, OnCopyStick4)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_STICK6, OnCopyStick5)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_STICK7, OnCopyStick6)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_STICK8, OnCopyStick7)

ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_WLAN1, OnCopyWLAN0)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_WLAN2, OnCopyWLAN1)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_WLAN3, OnCopyWLAN2)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_WLAN4, OnCopyWLAN3)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_WLAN5, OnCopyWLAN4)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_WLAN6, OnCopyWLAN5)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_WLAN7, OnCopyWLAN6)
ON_BN_CLICKED(IDC_WLAN_DEPLOY_UPD_WLAN8, OnCopyWLAN7)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CWLANDeployDialog::OnInitDialog() 
{
  CDialog::OnInitDialog();

  m_MS.SelectString(-1,
		  AfxGetApp()->GetProfileString("WLANDeployDialog", "memorydrive", "G:"));

  m_IP[0]=IPToStr(config->ipRed[0]);
  m_IP[1]=IPToStr(config->ipRed[1]);
  m_IP[2]=IPToStr(config->ipRed[2]);
  m_IP[3]=IPToStr(config->ipRed[3]);
  m_IP[4]=IPToStr(config->ipBlue[0]);
  m_IP[5]=IPToStr(config->ipBlue[1]);
  m_IP[6]=IPToStr(config->ipBlue[2]);
  m_IP[7]=IPToStr(config->ipBlue[3]);

  m_Aibo[0]=AfxGetApp()->GetProfileString("WLANDeployDialog", "aibo0", "aibo0");
  m_Aibo[1]=AfxGetApp()->GetProfileString("WLANDeployDialog", "aibo1", "aibo1");
  m_Aibo[2]=AfxGetApp()->GetProfileString("WLANDeployDialog", "aibo2", "aibo2");
  m_Aibo[3]=AfxGetApp()->GetProfileString("WLANDeployDialog", "aibo3", "aibo3");
  m_Aibo[4]=AfxGetApp()->GetProfileString("WLANDeployDialog", "aibo4", "aibo4");
  m_Aibo[5]=AfxGetApp()->GetProfileString("WLANDeployDialog", "aibo5", "aibo5");
  m_Aibo[6]=AfxGetApp()->GetProfileString("WLANDeployDialog", "aibo6", "aibo6");
  m_Aibo[7]=AfxGetApp()->GetProfileString("WLANDeployDialog", "aibo7", "aibo7");

  m_Stick[0]=AfxGetApp()->GetProfileString("WLANDeployDialog", "stick0", "stick0");
  m_Stick[1]=AfxGetApp()->GetProfileString("WLANDeployDialog", "stick1", "stick1");
  m_Stick[2]=AfxGetApp()->GetProfileString("WLANDeployDialog", "stick2", "stick2");
  m_Stick[3]=AfxGetApp()->GetProfileString("WLANDeployDialog", "stick3", "stick3");
  m_Stick[4]=AfxGetApp()->GetProfileString("WLANDeployDialog", "stick4", "stick4");
  m_Stick[5]=AfxGetApp()->GetProfileString("WLANDeployDialog", "stick5", "stick5");
  m_Stick[6]=AfxGetApp()->GetProfileString("WLANDeployDialog", "stick6", "stick6");
  m_Stick[7]=AfxGetApp()->GetProfileString("WLANDeployDialog", "stick7", "stick7");

  m_WLANUpdatesColortable.SetCheck(AfxGetApp()->GetProfileInt("WLANDeployDialog","wlanupdate",0)==0);
  m_WLANUpdatesBinaries.SetCheck(AfxGetApp()->GetProfileInt("WLANDeployDialog","wlanupdate",0)==1);

  refreshBinaryCombo();
  refreshColortableCombo();

  selectIndicator(-1);

  UpdateData(false);
  return TRUE;
}

void CWLANDeployDialog::selectIndicator(int number) {
	int i;
	for(i=0;i<8;i++)
		if(i!=number) m_Indicator[i].ShowWindow(SW_HIDE);
		else m_Indicator[i].ShowWindow(SW_SHOWNORMAL);
}


void CWLANDeployDialog::refreshBinaryCombo() {
   CFileFind finder;
   CString defaultPath = File::getGTDir();
   defaultPath += "/Deployment/Binaries/*.tgz";
   defaultPath.Replace('/','\\');
   int i;
   for(i=0;i<8;i++) m_Binaries[i].Clear();

   BOOL bWorking = finder.FindFile(defaultPath);
   while (bWorking)
   {
      bWorking = finder.FindNextFile();
//      AfxMessageBox(finder.GetFileName());
      for(i=0;i<8;i++) 
	      m_Binaries[i].AddString(finder.GetFileName());
   }

   char buf2[255];
   for(i=0;i<8;i++) {
	CString name;

	name.Format("bin%d",i);
    strcpy(buf2,name);
      m_Binaries[i].SelectString(-1,
		  AfxGetApp()->GetProfileString("WLANDeployDialog", buf2, ""));

   }


   UpdateData(false);
}

void CWLANDeployDialog::refreshColortableCombo() {
   CFileFind finder;
   CString defaultPath = File::getGTDir();
   defaultPath += "/Deployment/Colortables/*";
   defaultPath.Replace('/','\\');
   int i;
   for(i=0;i<8;i++) m_Colortable[i].Clear();

   BOOL bWorking = finder.FindFile(defaultPath);
   while (bWorking)
   {
      bWorking = finder.FindNextFile();
//      AfxMessageBox(finder.GetFileName());
	  if(finder.GetFileName().Left(1).Compare("."))
      for(i=0;i<8;i++) 
	      m_Colortable[i].AddString(finder.GetFileName());
   }
   char buf2[255];
   for(i=0;i<8;i++) {
	CString name;

	name.Format("col%d",i);
    strcpy(buf2,name);
      m_Colortable[i].SelectString(-1,
		  AfxGetApp()->GetProfileString("WLANDeployDialog", buf2, ""));
   }

   UpdateData(false);
}

void CWLANDeployDialog::OnCopyStick0() {
	copyStick(0);
}
void CWLANDeployDialog::OnCopyStick1() {
	copyStick(1);
}
void CWLANDeployDialog::OnCopyStick2() {
	copyStick(2);
}
void CWLANDeployDialog::OnCopyStick3() {
	copyStick(3);
}
void CWLANDeployDialog::OnCopyStick4() {
	copyStick(4);
}
void CWLANDeployDialog::OnCopyStick5() {
	copyStick(5);
}
void CWLANDeployDialog::OnCopyStick6() {
	copyStick(6);
}
void CWLANDeployDialog::OnCopyStick7() {
	copyStick(7);
}
void CWLANDeployDialog::copyStick(int number) {
  prepareImage(number);

  CString fBin,fCol;
  m_Binaries[number].GetLBText(m_Binaries[number].GetCurSel(),fBin);
  m_Colortable[number].GetLBText(m_Colortable[number].GetCurSel(),fCol);

  CString memorydrive;
  m_MS.GetLBText(m_MS.GetCurSel(),memorydrive);

  _flushall(); // just for safety. see system()
  system(
    CString("bash ")
    + File::getGTDir() + "/Make/copyfiles-prepare-and-deliver.bash "+fBin+" "+fCol+" "+memorydrive
	
  );

}

void CWLANDeployDialog::prepareImage(int number) {
  selectIndicator(number);

	CString fBin,fCol;
	m_Binaries[number].GetLBText(m_Binaries[number].GetCurSel(),fBin);
	m_Colortable[number].GetLBText(m_Colortable[number].GetCurSel(),fCol);

  CString defaultPath = File::getGTDir();
  defaultPath += "/Deployment/Config/";
  defaultPath.Replace('/','\\');

  OutTextRawFile(defaultPath+"player.cfg")
    << "teamColor " << Player::getTeamColorName(number<4?Player::red:Player::blue) << "\r\n"
    << "playerNumber " << Player::getPlayerNumberName((Player::playerNumber)(number%4)) << "\r\n"
    << "teamIdentifier " << (number<4 ? config->teamIdentifierRed : config->teamIdentifierBlue) << "\r\n";

  CString netmask;
  CString gateway;
  CString essid,wepkey,apmode,channel;

  gateway = "";//no gateway

  if(number<4) {
	  essid = config->essidRed;
	  wepkey = config->wepkeyRed;
	  netmask = IPToStr(config->netmaskRed);
	  apmode.Format("%d", config->apmodeRed);
	  channel.Format("%d",  config->channelRed);
  } else {
	  essid = config->essidBlue;
	  wepkey = config->wepkeyBlue;
	  netmask = IPToStr(config->netmaskRed);

	  apmode.Format("%d", config->apmodeBlue);
	  channel.Format("%d",  config->channelBlue);
  }



  OutTextRawFile(defaultPath+"wlanconf.txt")
    << "HOSTNAME=AIBO"  << (m_Aibo[number])      << "\r\n"
    << "ETHER_IP="      << (m_IP[number])      << "\r\n"
    << "ETHER_NETMASK=" << (netmask) << "\r\n"
    << "IP_GATEWAY="    << (gateway)      << "\r\n"
    << "ESSID="         << essid            << "\r\n"
    << "WEPENABLE="     << (wepkey!="")     << "\r\n"
    << "WEPKEY="        << wepkey           << "\r\n"
    << "APMODE="        << apmode           << "\r\n"
    << "CHANNEL="       << channel          << "\r\n";

  OutTextRawFile(defaultPath+"stickinf.txt")
    << "Slot........... "  << (number)      << "\r\n"
    << "Memorystick.... "  << (m_Stick[number])      << "\r\n"
    << "Aibo........... "      << (m_Aibo[number])      << "\r\n"
	<< "" << "\r\n"
	<< "Technical data:" << "\r\n"
    << "IP............. " << m_IP[number] << "\r\n"
    << "CODEBASE....... " << fBin            << "\r\n"
    << "COLORTABLE..... " << fCol     << "\r\n"
    << "TEAMCOLOR...... " << Player::getTeamColorName(number<4?Player::red:Player::blue) << "\r\n"
    << "PLAYER......... " << Player::getPlayerNumberName((Player::playerNumber)(number%4)) << "\r\n";

}


void CWLANDeployDialog::OnCopyWLAN0() {
	copyWLAN(0);
}
void CWLANDeployDialog::OnCopyWLAN1() {
	copyWLAN(1);
}
void CWLANDeployDialog::OnCopyWLAN2() {
	copyWLAN(2);
}
void CWLANDeployDialog::OnCopyWLAN3() {
	copyWLAN(3);
}
void CWLANDeployDialog::OnCopyWLAN4() {
	copyWLAN(4);
}
void CWLANDeployDialog::OnCopyWLAN5() {
	copyWLAN(5);
}
void CWLANDeployDialog::OnCopyWLAN6() {
	copyWLAN(6);
}
void CWLANDeployDialog::OnCopyWLAN7() {
	copyWLAN(7);
}
void CWLANDeployDialog::copyWLAN(int number) {
  prepareImage(number);

  CString fBin,fCol;
  m_Binaries[number].GetLBText(m_Binaries[number].GetCurSel(),fBin);
  m_Colortable[number].GetLBText(m_Colortable[number].GetCurSel(),fCol);

  CString update="";

  if(m_WLANUpdatesColortable.GetCheck())
	  update="update";

  _flushall(); // just for safety. see system()
  system(
    CString("bash ")
    + File::getGTDir() + "/Make/copyfiles-prepare-and-ftp.bash "+fBin+" "+fCol+" "+m_IP[number]+" "+update
	
  );

}

void CWLANDeployDialog::OnOK() 
{ 
  UpdateData(true);
  if(m_WLANUpdatesColortable.GetCheck())
	AfxGetApp()->WriteProfileInt("WLANDeployDialog","wlanupdate",0);
  if(m_WLANUpdatesBinaries.GetCheck())
	AfxGetApp()->WriteProfileInt("WLANDeployDialog","wlanupdate",1);


  AfxGetApp()->WriteProfileString("WLANDeployDialog", "aibo0", m_Aibo[0]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "aibo1", m_Aibo[1]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "aibo2", m_Aibo[2]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "aibo3", m_Aibo[3]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "aibo4", m_Aibo[4]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "aibo5", m_Aibo[5]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "aibo6", m_Aibo[6]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "aibo7", m_Aibo[7]);
  
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "stick0", m_Stick[0]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "stick1", m_Stick[1]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "stick2", m_Stick[2]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "stick3", m_Stick[3]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "stick4", m_Stick[4]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "stick5", m_Stick[5]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "stick6", m_Stick[6]);
  AfxGetApp()->WriteProfileString("WLANDeployDialog", "stick7", m_Stick[7]);

  char buf[255];
  char buf2[255];
  CString bin[8];
  int i=0;
  for(i=0;i<8;i++) {
	m_Binaries[i].GetLBText(m_Binaries[i].GetCurSel(),buf);
	CString name;
	name.Format("bin%d",i);
    strcpy(buf2,name);
	AfxGetApp()->WriteProfileString(
		"WLANDeployDialog", buf2, buf);
  }
  for(i=0;i<8;i++) {
	m_Colortable[i].GetLBText(m_Colortable[i].GetCurSel(),buf);
	CString name;
	name.Format("col%d",i);
    strcpy(buf2,name);
	AfxGetApp()->WriteProfileString(
		"WLANDeployDialog", buf2, buf);

  }

  CString memorydrive;
  m_MS.GetLBText(m_MS.GetCurSel(),memorydrive);
  AfxGetApp()->WriteProfileString(
		"WLANDeployDialog", "memorydrive", memorydrive);


  CDialog::OnOK();
}

void CWLANDeployDialog::OnRefreshAll() 
{
	int i;
	CPing ping;
	char mybuf[200];
	int flag[8];
	for(i=0;i<8;i++) {
		setPingIcon(i,1);
        strcpy(mybuf,m_IP[i]);
		//strcpy(mybuf,"192.168.0.114");
		if(ping.Ping(mybuf)) {
			setPingIcon(i,2);//green
			flag[i]=1;
		    _flushall(); // just for safety. see system()
	        system(
				CString("bash ")
				+ File::getGTDir() + "/Make/copyfiles-retrieve-stickinfo.bash "+m_IP[i]);
  		    CString fname=File::getGTDir();
			fname +="\\Make\\stickinf.txt";
			updateSlotFromFile(fname,i);
		} else {
			setPingIcon(i,0);//red
			flag[i]=0;
		}
	}
	UpdateData(false);

}

void CWLANDeployDialog::OnRefresh() 
{
	selectIndicator(-1);
  CString memorydrive;
  m_MS.GetLBText(m_MS.GetCurSel(),memorydrive);

  updateSlotFromFile(memorydrive+"\\stickinf.txt",-1);
}

void CWLANDeployDialog::updateSlotFromFile(CString fname,int forcedNumber) {

	
  InTextFile infile(fname);
  if(!infile.exists()) return; // Cannot open the file
  
  char sbuf[255];

  char aiboname[255],stickname[255],bin[255],col[255];
  int number;

  infile >> sbuf >> number;	// Slot.....: 112
  infile >> sbuf >> stickname; // Stickname:
  infile >> sbuf >> aiboname; // aiboname;
  infile >> sbuf >> sbuf; // technical data
  infile >> sbuf >> sbuf; // ip
  infile >> sbuf >> bin; // binaries
  infile >> sbuf >> col; // colortable;

  if((number>=0)&&(number<=7)) {
    selectIndicator(number);
	if(forcedNumber>=0) number=forcedNumber;
    m_Aibo[number]=aiboname;
    m_Stick[number]=stickname;
    m_Binaries[number].SelectString(-1,bin);
    m_Colortable[number].SelectString(-1,col);
  }

  UpdateData(false);
}
 
void CWLANDeployDialog::OnSnapshot() 
{
  CString defaultPath = File::getGTDir();
  defaultPath += "/Deployment/Binaries/";
  defaultPath.Replace('/','\\');
  CString pathName = 
    AfxGetApp()->GetProfileString("WLANDeployDialog", "savePath", defaultPath);
  pathName += "*.tgz";

  CFileDialog fileDialog(false, ".tgz",pathName,
    OFN_HIDEREADONLY | OFN_EXPLORER | OFN_ENABLESIZING | OFN_NOCHANGEDIR | OFN_NONETWORKBUTTON
    , "aibo software package (*.tgz)|*.tgz||", this);

  if (fileDialog.DoModal()==IDOK)
  {
    CString fileName = fileDialog.GetFileName();
    CString pathAndFileName = fileDialog.GetPathName();
    CString pathName = 
      pathAndFileName.Left(
      pathAndFileName.GetLength() -
      fileName.GetLength()
      );
	  _flushall(); // just for safety. see system()
	  system(
	    CString("bash ")
	    + File::getGTDir() + "/Make/copyfiles-package.bash "+fileName);

    AfxGetApp()->WriteProfileString("WLANDeployDialog", "savePath", pathName);

	refreshBinaryCombo();
	refreshColortableCombo();

  }
}
 

void CWLANDeployDialog::OnPingAll() 
{
	int i;
	CPing ping;
	char mybuf[200];
	char mybuf2[200];
	char mybuf3[200];
	int flag[8];
	for(i=0;i<8;i++) {
		setPingIcon(i,1);
        strcpy(mybuf,m_IP[i]);
		//strcpy(mybuf,"192.168.0.114");
		if(ping.Ping(mybuf)) {
			setPingIcon(i,2);//green
			flag[i]=1;

		} else {
			setPingIcon(i,0);//red
			flag[i]=0;
		}
	}
    ping.readOutARP();
	for(i=0;i<8;i++) 
	if(flag[i]){
       strcpy(mybuf,m_IP[i]);
       ping.WIN32GetMAC(mybuf,mybuf2);
	   getAiboName(mybuf2,mybuf3);
	   if(mybuf3[0]!='\0') 
	       m_Aibo[i]=mybuf3;

	}

	UpdateData(false);
}

int CWLANDeployDialog::getAiboName(const char* mac, char* name) 
{
  CString aiboname;
  aiboname= File::getGTDir();
  aiboname+="/Deployment/Config/aibonames.txt";
  InTextFile infile(aiboname);
  if (!infile.exists()) return 0;

  char mac_in[255];

  while(!infile.eof()) {
    infile >> mac_in;

    if(strcmp(mac_in, mac) == 0) { // Parse the MAC Address
		infile >> name;
		return 1;
    }
  }
  return 0;
}
 

void CWLANDeployDialog::setWLANConfiguration(CRobotControlWLANConfiguration* externConfig)
{
  config=externConfig;
}

CString CWLANDeployDialog::IPToStr(unsigned long ip)
{
  CString s; s.Format(
    "%d.%d.%d.%d",
    (ip>>24), ((ip>>16)&0xff), ((ip>>8)&0xff), (ip&0xff)
  );
  return s;
}

void CWLANDeployDialog::setPingIcon(int number,int status) 
{
	switch(status) {
	case 0:
		m_Ping[number].SetBitmap(LoadBitmap(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_PING_RED)));
		break;
	case 1:
		m_Ping[number].SetBitmap(LoadBitmap(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_PING_YELLOW)));
		break;
	case 2:
		m_Ping[number].SetBitmap(LoadBitmap(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_PING_GREEN)));
		break;
    }
	m_Ping[number].RedrawWindow();

}



/*
* Change log :
* 
* $Log: WLANDeployDialog.cpp,v $
*
*/

