/**
 * @file CollectedBeliefs.cpp
 *
 * Implementation of class CollectedBeliefs.
 */

#include "CollectedBeliefs.h"



/** Implementation of SingleBeliefs*/

SingleBeliefs::SingleBeliefs()
{
  int i;
  for (i = 0; i < RateableOptions::numOfOptions; ++i)
    ratedOptions[i].reset(1);
  timeStamp = SystemCall::getCurrentSystemTime();
  for (i = 0; i < RateableOptions::numOfOptionClasses; ++i)
    lastBestOpt[i] = RateableOptions::getClassMember((RateableOptions::OptionClassID)(i+RateableOptions::numOfOptions),0);
}

SingleBeliefs::~SingleBeliefs()
{
}

RateableOptions::OptionID SingleBeliefs::getBestOptionInOptionClass(RateableOptions::OptionClassID id)
{
	RateableOptions::OptionID maxopt = lastBestOpt[(int)(id)-RateableOptions::numOfOptions];
  int maxval = ratedOptions[(int)(maxopt)];
  int ct = RateableOptions::getOptionClassChangeThreshold(id);

  for (int i = 0; i < RateableOptions::getNumberOfClassMembers(id); ++i)
	  if (ratedOptions[(int)(RateableOptions::getClassMember(id,i))] >= (maxval+ct))
		{
		  maxval = ratedOptions[(int)(RateableOptions::getClassMember(id,i))];
			maxopt = RateableOptions::getClassMember(id,i);
		}

  lastBestOpt[(int)(id)-RateableOptions::numOfOptions] = maxopt;
  return maxopt;
}


In& operator>>(In& stream,SingleBeliefs& singleBeliefs)
{
  int tmp = 0;
  for (int i = 0; i < RateableOptions::numOfOptions; ++i){
    stream.read(&tmp,sizeof(int));
    singleBeliefs.ratedOptions[i] = tmp;
  }
  return stream;
}


Out& operator<<(Out& stream,SingleBeliefs& singleBeliefs)
{
  int tmp = 0;
  for (int i = 0; i < RateableOptions::numOfOptions; ++i){
    tmp = singleBeliefs.ratedOptions[i];
    stream.write(&tmp,sizeof(int));
  }
  return stream;
}






/** Implementation of CollectedBeliefs*/

CollectedBeliefs::CollectedBeliefs(const BehaviorControlInterfaces& interfaces)
: BehaviorControlInterfaces(interfaces)
{
  for (int i=0; i < Player::numOfPlayerNumbers; ++i) {
    for (int j=0; j < RateableOptions::numOfOptions; ++j) {
      singleBeliefs[i].ratedOptions[j].reset(RateableOptions::getSlidingAverageRange(j));
    }
    singleBeliefs[i].timeStamp = SystemCall::getCurrentSystemTime();
  }

}

CollectedBeliefs::~CollectedBeliefs()
{
}

void CollectedBeliefs::update()
{
	// update the singleBeliefs ...
	// the BehaviorControlInterfaces should provide the needed information...
	// btw: the "own" SingleBeliefs is
	// singleBeliefs[(int)(getPlayer().getPlayerRole())]


  // due to some problems with aging based on timestamps from the other dogs
  // we use the timestamp of the last change in a singleBeliefs structure...
  //bool somethingChanged;

  for (int i=0; i<Player::numOfPlayerNumbers-1; ++i)
  {
    if ((teamMessageCollection[i].isActual()) &&
        (teamMessageCollection[i].playerNumberOfSender != Player::undefinedPlayerNumber))
    {
      SingleBeliefs& actSingleBeliefs =
				singleBeliefs[(int)(teamMessageCollection[i].playerNumberOfSender)];
      /*
      somethingChanged = false;
			for (int j=0; j<RateableOptions::numOfOptions; ++j)
        if (actSingleBeliefs.ratedOptions[j] != 
              teamMessageCollection[i].behaviorTeamMessage.ratedOptions[j])
        {
				  actSingleBeliefs.ratedOptions[j] =
				    teamMessageCollection[i].behaviorTeamMessage.ratedOptions[j];
          somethingChanged = true;
        }
      if (somethingChanged)
			  actSingleBeliefs.timeStamp = SystemCall::getCurrentSystemTime();
        */
			for (int j=0; j<RateableOptions::numOfOptions; ++j)
				  actSingleBeliefs.ratedOptions[j] =
				    teamMessageCollection[i].behaviorTeamMessage.ratedOptions[j];
		  actSingleBeliefs.timeStamp = SystemCall::getCurrentSystemTime();
    }
  }
  
}

void CollectedBeliefs::broadcast()
{
  // broadcast your singleBeliefs
	// maybe use the outgoingBehaviorTeamMessage property for this
  int tmpno = (int)(getPlayer().getPlayerNumber());
  if (tmpno != Player::undefinedPlayerNumber)
  {
    for (int j=0; j<RateableOptions::numOfOptions; ++j)
      outgoingBehaviorTeamMessage.ratedOptions[j] = 
        singleBeliefs[tmpno].ratedOptions[j].getLastValue();

	  INFO(sendOptionRatings,idSingleBeliefs,bin,singleBeliefs[tmpno]);
  }
}

void CollectedBeliefs::trigger()
{
  for (int i=0; i < Player::numOfPlayerNumbers; ++i)
  {
    for (int j=0; j<RateableOptions::numOfOptions; ++j)
      singleBeliefs[i].ratedOptions[j].trigger();
  }
}



In& operator>>(In& stream,CollectedBeliefs& collectedBeliefs)
{
  stream.read(&collectedBeliefs,sizeof(CollectedBeliefs));
  return stream;
}


Out& operator<<(Out& stream, const CollectedBeliefs& collectedBeliefs)
{
  stream.write(&collectedBeliefs,sizeof(CollectedBeliefs));
  return stream;
}


/*
 * Change log :
 *
 * $Log: CollectedBeliefs.cpp,v $
 * Revision 1.3  2004/07/22 22:38:34  kerdels
 * added DTT used by Open Challenge, RIP and Xabsl-Options will follow
 *
 *
 */
