/**
 * @file CollectedBeliefs.h
 * 
 * Declaration of class CollectedBeliefs
 * @author <A href=mailto:jochen@kerdels.de>Jochen Kerdels</A>
 */ 

#ifndef __CollectedBeliefs_h_
#define __CollectedBeliefs_h_

#include "Modules/BehaviorControl/BehaviorControl.h"
#include "Tools/Player.h"
#include "Representations/Cognition/BallModel.h"
#include "Representations/Cognition/RobotPose.h"
#include "RateableOptions.h"
#include "Tools/SlidingAverageValue.h"


/**
 *  The class contains information on how the robot senses his
 *  enviroment.
 */
class SingleBeliefs
{
public:
  /** Constructor */
  SingleBeliefs();

  /** Destructor */
  ~SingleBeliefs();

  /** Rating on all "options" */
  SlidingAverageValue ratedOptions[RateableOptions::numOfOptions];

  /** The time when the package was sent */
  unsigned long timeStamp;

  /** the timeStamps from the other SingleBeliefs
	  * can be used to detect single sided Communication failure
		* based upon suggestion from Jrn (see IRC Log 6.12.2003)
		*
		* zunchst lasse ich das mal raus... mal sehen, ob wir das
		* wirklich brauchen
	  */
  //unsigned long lastUpdateFromTeamMember[3];

	RateableOptions::OptionID getBestOptionInOptionClass(RateableOptions::OptionClassID id);

private:

  RateableOptions::OptionID lastBestOpt[RateableOptions::numOfOptionClasses];

};

/**
 * Streaming operator that reads a SingleBeliefs from a stream.
 * @param stream The stream from which is read.
 * @param singleBeliefs The SingleBeliefs object.
 * @return The stream.
 */ 
In& operator>>(In& stream,SingleBeliefs& singleBeliefs);

/**
 * Streaming operator that writes a SingleBeliefs to a stream.
 * @param stream The stream to write on.
 * @param singleBeliefs The SingleBeliefs object.
 * @return The stream.
 */ 
Out& operator<<(Out& stream,SingleBeliefs& singleBeliefs);



/**
 *  This class contains the PlayerPoses of all detected robots.
 */
class CollectedBeliefs : public BehaviorControlInterfaces
{
public:
  /** Constructor */
  CollectedBeliefs(const BehaviorControlInterfaces& interfaces);

  /** Destructor */
  ~CollectedBeliefs();

  /** 
   * The four SingleBeliefs of the team members.
   * To be adressed by (int)Player::playerNumber
   */
  SingleBeliefs singleBeliefs[Player::numOfPlayerNumbers];

  /**
   *  This method updates recently received SingleBeliefs.
	 *  btw: the "own" SingleBeliefs is singleBeliefs[(int)(getPlayer().getPlayerRole())]
	 */
	void update();


  /**
   *  This method broadcasts the own SingleBeliefs to the other robots.
	 *  btw: the "own" SingleBeliefs is singleBeliefs[(int)(getPlayer().getPlayerRole())]
	 */
	void broadcast();

  /**
   *  This method triggers all SingleBeliefs.rateOptions
	 */
	void trigger();


};

/**
 * Streaming operator that reads a CollectedBeliefs from a stream.
 * @param stream The stream from which is read.
 * @param collectedBeliefs The CollectedBeliefs object.
 * @return The stream.
 */ 
In& operator>>(In& stream,CollectedBeliefs& collectedBeliefs);
 
/**
 * Streaming operator that writes a CollectedBeliefs to a stream.
 * @param stream The stream to write on.
 * @param collectedBeliefs The CollectedBeliefs object.
 * @return The stream.
 */ 
Out& operator<<(Out& stream, const CollectedBeliefs& collectedBeliefs);


#endif //__CollectedBeliefs_h_

/*
 * Change log :
 * 
 * $Log: CollectedBeliefs.h,v $
 * Revision 1.3  2004/07/22 22:38:34  kerdels
 * added DTT used by Open Challenge, RIP and Xabsl-Options will follow
 *
 *
 *
 */
