/**
* @file OptionRating.h
* 
* This file contains a generic class for Option Rating.
* @author <A href=mailto:jochen@kerdels.de>Jochen Kerdels</A>
*/

#ifndef __OptionRating_h_
#define __OptionRating_h_

#include "Modules/BehaviorControl/BehaviorControl.h"
#include "CollectedBeliefs.h"
#include "ChooserRaterInterfaces.h"

/**
* @class OptionRating 
*
* The Base class for option rating
*
* It is the task of option rating to rate all avaible options.
*/
class OptionRating :  public BehaviorControlInterfaces
{
protected:
  
  CollectedBeliefs& collectedBeliefs;
  
  ChooserRaterInterfaces& chooserRaterInterfaces;
  
public:
  
/**
* Constructor.
* @param interfaces The parameters of the OptionRating module.
* @param collectedBeliefs The collected-Beliefs to work on.
* @param chooserRaterInterfaces We want to talk to the Tactic-Chooser.
  */
  OptionRating(const BehaviorControlInterfaces& interfaces,
    CollectedBeliefs& collectedBeliefs,
    ChooserRaterInterfaces& chooserRaterInterfaces
    )
    : BehaviorControlInterfaces(interfaces),
		  collectedBeliefs(collectedBeliefs),
      chooserRaterInterfaces(chooserRaterInterfaces)
  {}
  
  /** Destructor */
  virtual ~OptionRating() {}
  
  
  /** This method should be implemented by descendants */
  virtual void rateOptions() = 0;
  
};

#endif // __OptionRating_h_

/*
* Change log :
* $Log: OptionRating.h,v $
* Revision 1.3  2004/07/22 22:38:34  kerdels
* added DTT used by Open Challenge, RIP and Xabsl-Options will follow
*
*
*/
