#include "RateableOptions.h"

RateableOptions::OptionInfo  RateableOptions::optionInfos[] = {
  // options
  //
  { noOption,                      "NoOption",                       0, 0, notype       ,1  ,0 }, 
  { doNothing,                     "DoNothing",                      0, 0, special      ,1  ,0 },
  { stand,                         "Stand",                          0, 0, gotoPos      ,1  ,0 },
  { keepOption,                    "KeepOption",                     0, 0, notype       ,1  ,0 },
  { intro,                         "Intro",                          0, 0, notype       ,1  ,0 },  
  { extro,                         "Extro",                          0, 0, notype       ,1  ,0 },  
  { finished,                      "Finished",                       0, 0, notype       ,1  ,0 },  
  { gotoBitePos1,                  "GotoBitePos1",                   0, 0, notype       ,1  ,0 },  
  { gotoBitePos2,                  "GotoBitePos2",                   0, 0, notype       ,1  ,0 },  
  { gotoBitePos3,                  "GotoBitePos3",                   0, 0, notype       ,1  ,0 },  
  { gotoBitePos4,                  "GotoBitePos4",                   0, 0, notype       ,1  ,0 },  
  { bitePos1,                      "BitePos1",                       0, 0, notype       ,1  ,0 },  
  { bitePos2,                      "BitePos2",                       0, 0, notype       ,1  ,0 },  
  { bitePos3,                      "BitePos3",                       0, 0, notype       ,1  ,0 },  
  { bitePos4,                      "BitePos4",                       0, 0, notype       ,1  ,0 },  
  { movePos1,                      "MovePos1",                       0, 0, notype       ,1  ,0 },  
  { movePos2,                      "MovePos2",                       0, 0, notype       ,1  ,0 },  
  { movePos3,                      "MovePos3",                       0, 0, notype       ,1  ,0 },  
  { movePos4,                      "MovePos4",                       0, 0, notype       ,1  ,0 },  
  { gotoBridge,                    "GotoBridge",                     0, 0, notype       ,1  ,0 },  
  { climbBridge,                   "ClimbBridge",                    0, 0, notype       ,1  ,0 },  
  { moveBridge,                    "MoveBridge",                     0, 0, notype       ,1  ,0 },  
  { waitForBiteDogs,               "waitForBiteDogs",                0, 0, notype       ,1  ,0 },  

  // we need at least one option class
  { doSomething,           "DoSomething",          1, 10, notype ,0  ,0 },  
  // end marker
  //
  { -1, NULL, 0, 0, notype ,0  ,0 }
};


int RateableOptions::getNumberOfClassMembers(OptionClassID id)
{
  for (OptionInfo *o=optionInfos; o->id != -1; o++) {
    if (o->id == id)
      return o->numOfSubOptions;
  }  
  return 0;
}

int RateableOptions::getOptionClassChangeThreshold(OptionClassID id)
{
  for (OptionInfo *o=optionInfos; o->id != -1; o++) {
    if (o->id == id)
      return o->changeThreshold;
  }  
  return 0;
}

RateableOptions::OptionID RateableOptions::getClassMember(OptionClassID id, int idx)
{
  switch (id) {
    case doSomething:
      switch (idx) {
        case 0   : return stand;
      }
      break;
  }

  return noOption;
}


/**
 * returns names for option and option class ids
 */
const char* RateableOptions::getOptionName(int id)
{
  for (OptionInfo *o=optionInfos; o->id != -1; o++) {
    if (o->id == id)
      return o->name;
  }
  return "please edit the RateableOptions::optionInfos array";
}


/**
 * returns ids for option and option class names
 */
int RateableOptions::getOptionID(const char* name)
{
  for (OptionInfo *o=optionInfos; o->id != -1; o++) {
    if (!strcmp(name, o->name))
      return o->id;
  }
  return -1;
};

/**
 * returns types for option ids
 */
RateableOptions::OptionType RateableOptions::getOptionType(int id)
{
  for (OptionInfo *o=optionInfos; o->id != -1; o++) {
    if (o->id == id)
      return o->optionType;
  }
  return notype;
};

/**
 * returns SlidingAverageRange for option and option class names
 */
int RateableOptions::getSlidingAverageRange(int id)
{
  for (OptionInfo *o=optionInfos; o->id != -1; o++) {
    if (o->id == id)
      return o->slidingAverageRange;
  }
  return -1;
};

/**
 * returns RatingBoost for option and option class names
 */
int RateableOptions::getRatingBoost(int id)
{
  for (OptionInfo *o=optionInfos; o->id != -1; o++) {
    if (o->id == id)
      return o->ratingBoost;
  }
  return -1;
};


/**
 * returns true, if id is an option class, else false
 */
bool RateableOptions::isOptionClass(int id)
{
  return getNumberOfClassMembers( (OptionClassID)id ) != noOption;
}


/**
 * returns true, if Oid is a member of the option class OCid, else false
 */
bool RateableOptions::isClassMember(OptionClassID OCid, OptionID Oid)
{
  switch (OCid) {
    case doSomething:
      switch (Oid) {
        case stand: return true;
      }
      break;
   
  }

  return false;
}


/*
 * Change log :
 * $Log: RateableOptions.cpp,v $
 * Revision 1.3  2004/07/22 22:38:34  kerdels
 * added DTT used by Open Challenge, RIP and Xabsl-Options will follow
 *
 *
 */
