/**
* @file TacticChooser.h
* 
* This file contains a generic class for Tactic Choosing.
* @author <A href=mailto:jochen@kerdels.de>Jochen Kerdels</A>
*/

#ifndef __TacticChooser_h_
#define __TacticChooser_h_

#include "Modules/BehaviorControl/BehaviorControl.h"
#include "CollectedBeliefs.h"
#include "RateableOptions.h"
#include "GlobalAnalyser.h"
#include "ChooserRaterInterfaces.h"


#include "Modules/BehaviorControl/GT2004BehaviorControl/GT2004DTT/DefGlobalAnalyser.h"

/**
* @class TacticChooser 
*
* The Base class for tactic choosing
*
* It is the task of tactic choosing to choose one option from all avaible options.
*/
class TacticChooser :  public BehaviorControlInterfaces
{
protected:

	CollectedBeliefs& collectedBeliefs;

	// TacticChooser engine(s)
  GlobalAnalyser* globalAnalyser[RateableOptions::numOfGlobalAnalysers];

  /**
  * Structure to communicate between TacticChooser und OptionRater 
  */
  ChooserRaterInterfaces& chooserRaterInterfaces;
  
public:

  /**
   * Constructor.
   * @param interfaces The parameters of the OptionRating module.
   * @param collectedBeliefs The collected-Beliefs to work on.
   * @param chooserRaterInterfaces We want to talk to the Option-Rating.

   */
  TacticChooser(const BehaviorControlInterfaces& interfaces,
                CollectedBeliefs& collectedBeliefs,
                ChooserRaterInterfaces& chooserRaterInterfaces)
    : BehaviorControlInterfaces(interfaces),
		  collectedBeliefs(collectedBeliefs),
      chooserRaterInterfaces(chooserRaterInterfaces)
  {
		// register global analysers ..

	  // DefGlobalAnalyser Engine..
	  globalAnalyser[RateableOptions::defaultGlobalAnalyser] =
		  new DefGlobalAnalyser(interfaces);

	}
  
  /** Destructor */
  virtual ~TacticChooser()
	{
		// freeing Global Analysers ..
	  delete(globalAnalyser[RateableOptions::defaultGlobalAnalyser]);
	}


	/** This method should be implemented by descendants */
	virtual RateableOptions::OptionID chooseOption() = 0;

  /** 
  * Is called for every incoming debug message.
  * @param message An interface to read the message from the queue
  * @return if the messag was read
  */
  virtual bool handleMessage(InMessage& message) {return false;}

};

#endif // __TacticChooser_h_

/*
 * Change log :
 * $Log: TacticChooser.h,v $
 * Revision 1.3  2004/07/22 22:38:35  kerdels
 * added DTT used by Open Challenge, RIP and Xabsl-Options will follow
 *
 *
 */
